package de.mannheim.uni.expansion.data.dbpedia.adapter.csv;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Serializable;
import java.util.Map.Entry;
import java.util.Set;
import java.util.zip.GZIPOutputStream;

import javax.inject.Inject;

import org.apache.commons.collections4.BidiMap;

import com.opencsv.CSVWriter;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNew;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.ListType;
import de.mannheim.uni.expansion.data.ontology.model.ValueId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.utils.CSVSerializer;
import de.mannheim.uni.expansion.utils.StringSerializer;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class DBPediaRepositoryCSVWriter {
	private final StringSerializer stringSerializer;
	private final CSVSerializer csvSerializer;

	@Inject
	public DBPediaRepositoryCSVWriter(StringSerializer stringSerializer, CSVSerializer csvSerializer) {
		this.stringSerializer = stringSerializer;
		this.csvSerializer = csvSerializer;
	}

	void write(DBpediaRepositoryNew repository, String folder, boolean newValueFormat) throws IOException {

		writeClassKeys(repository, folder);

		writeTableIndices(repository, folder);

		if (newValueFormat) {
			writeValuesNew(repository, folder);
		} else {
			writeValuesOld(repository, folder);
		}

		writePropertyIndices(repository, folder);

		writeInstanceIndices(repository, folder);

		writeColumnDataTypes(repository, folder);

	}

	void writeClassKeys(DBpediaRepositoryNew repository, String folder) throws IOException {
		CSVWriter writer = new CSVWriter(new FileWriter(folder + "classKeys"), ';', '"');
		for (Entry<ClassId, PropertyId> e : repository.getClassKeys().entrySet()) {
			String[] line = new String[2];
			line[0] = e.getKey().toString();
			line[1] = e.getValue().toString();
			writer.writeNext(line);
		}
		writer.close();
	}

	public void writeListTypes(DBpediaRepositoryNew repository, String folder) throws IOException {

		Set<ClassPropertyId> shortListTypes = repository.getListTypes(ListType.SHORT);
		Set<ClassPropertyId> mediumListTypes = repository.getListTypes(ListType.MEDIUM);
		Set<ClassPropertyId> longListTypes = repository.getListTypes(ListType.LONG);

		CSVWriter writer = new CSVWriter(new FileWriter(folder + "listTypes"), ';', '"');
		try {
			for (ClassPropertyId classPropertyId : repository.getListTypes()) {
				String[] line = new String[3];
				line[0] = classPropertyId.getClassId().toString();
				line[1] = classPropertyId.getPropertyId().toString();
				if (shortListTypes.contains(classPropertyId)) {
					line[2] = ListType.SHORT.toString();
				} else if (mediumListTypes.contains(classPropertyId)) {

					line[2] = ListType.MEDIUM.toString();
				} else if (longListTypes.contains(classPropertyId)) {

					line[2] = ListType.LONG.toString();
				} else {
					throw new IllegalStateException("The list type must be either short, medium or long");
				}
				writer.writeNext(line);
			}
		} finally {
			writer.close();
		}
	}

	void writeTableIndices(DBpediaRepositoryNew repository, String folder) throws IOException {
		CSVWriter writer = new CSVWriter(new FileWriter(folder + "tableIndex"), ';', '"');
		for (Entry<ClassId, Integer> e : repository.getTableIndeces().entrySet()) {
			String[] line = new String[2];
			line[0] = e.getKey().toString();
			line[1] = String.valueOf(e.getValue());
			writer.writeNext(line);
		}
		writer.close();
	}

	public void writeValuesNew(DBpediaRepositoryNew repository, String folder) throws FileNotFoundException, IOException {
		int groupNum = 32;
		int perGroup = repository.getValues().size() / groupNum;
		CSVWriter writer = null;
		int group = 0;
		int i = 0;
		for (Entry<ValueId, Object> e : repository.getValues().entrySet()) {
			if ((i % perGroup == 0 && group != (groupNum)) || group == 0) {
				if (writer != null) {
					writer.close();
				}
				writer = new CSVWriter(new OutputStreamWriter(new GZIPOutputStream(new FileOutputStream(folder + "valuesNew" + group))), ';', '"', '\\');
				group++;
			}
			Object o = e.getValue();
			String[] line = csvSerializer.getLine(o, 2);

			line[0] = e.getKey().getInstanceId().toString();
			line[1] = e.getKey().getPropertyId().toString();

			writer.writeNext(line);
			i++;
		}
		writer.close();

	}

	void writeValuesOld(DBpediaRepositoryNew repository, String folder) throws IOException {
		CSVWriter writer = new CSVWriter(new OutputStreamWriter(new GZIPOutputStream(new FileOutputStream(folder + "values"))), ';', '"');
		for (Entry<ValueId, Object> e : repository.getValues().entrySet()) {
			String[] line = new String[3];
			line[0] = e.getKey().getInstanceId().toString();
			line[1] = e.getKey().getPropertyId().toString();
			line[2] = stringSerializer.toString((Serializable) e.getValue());
			writer.writeNext(line);
		}
		writer.close();

	}

	void writePropertyIndices(DBpediaRepositoryNew repository, String folder) throws IOException {
		CSVWriter writer = new CSVWriter(new FileWriter(folder + "propertyIndices"), ';', '"');
		for (Entry<ClassId, BidiMap<PropertyId, Integer>> e : repository.getPropertyIndices().entrySet()) {
			String[] line = new String[3];
			line[0] = e.getKey().toString();
			for (Entry<PropertyId, Integer> b : e.getValue().entrySet()) {
				line[1] = b.getKey().toString();
				line[2] = String.valueOf(b.getValue());
				writer.writeNext(line);
			}
		}
		writer.close();

	}

	void writeInstanceIndices(DBpediaRepositoryNew repository, String folder) throws IOException {
		CSVWriter writer = new CSVWriter(new FileWriter(folder + "instanceIndices"), ';', '"');
		for (Entry<ClassId, BidiMap<InstanceId, Integer>> e : repository.getInstanceIndices().entrySet()) {
			String[] line = new String[3];
			line[0] = e.getKey().toString();
			for (Entry<InstanceId, Integer> b : e.getValue().entrySet()) {
				line[1] = b.getKey().toString();
				line[2] = String.valueOf(b.getValue());
				writer.writeNext(line);
			}
		}
		writer.close();

	}

	public void writeInstancesPerClassProperty(DBpediaRepositoryNew repository, String folder) throws IOException {

		int size = 0;
		for (Entry<ClassPropertyId, Set<InstanceId>> e : repository.getInstancesPerClassProperty().entrySet()) {

			size += e.getValue().size();
		}

		int groupNum = 32;
		int perGroup = size / groupNum;
		CSVWriter writer = null;
		int group = 0;
		int i = 0;
		for (Entry<ClassPropertyId, Set<InstanceId>> e : repository.getInstancesPerClassProperty().entrySet()) {

			String[] line = new String[3];
			line[0] = e.getKey().getClassId().toString();
			line[1] = e.getKey().getPropertyId().toString();
			for (InstanceId b : e.getValue()) {

				if ((i == perGroup && group != (groupNum)) || group == 0) {
					if (writer != null) {
						writer.close();
					}
					writer = new CSVWriter(new OutputStreamWriter(new GZIPOutputStream(new FileOutputStream(folder + "instancesPerClassProperty" + group))),
							';', '"', '\\');
					group++;
					i = 0;
				}

				line[2] = b.toString();
				// line[3] = String.valueOf(b.getValue());
				writer.writeNext(line);

				i++;
			}

		}
		writer.close();

	}

	public void writeInstanceIndicesNew(DBpediaRepositoryNew repository, String folder) throws IOException {

		int size = 0;
		for (Entry<ClassId, BidiMap<InstanceId, Integer>> e : repository.getInstanceIndices().entrySet()) {

			size += e.getValue().size();
		}

		int groupNum = 32;
		int perGroup = size / groupNum;
		CSVWriter writer = null;
		int group = 0;
		int i = 0;
		for (Entry<ClassId, BidiMap<InstanceId, Integer>> e : repository.getInstanceIndices().entrySet()) {

			String[] line = new String[3];
			line[0] = e.getKey().toString();
			for (Entry<InstanceId, Integer> b : e.getValue().entrySet()) {

				if ((i == perGroup && group != (groupNum)) || group == 0) {
					if (writer != null) {
						writer.close();
					}
					writer = new CSVWriter(new OutputStreamWriter(new GZIPOutputStream(new FileOutputStream(folder + "instanceIndicesNew" + group))), ';', '"',
							'\\');
					group++;
					i = 0;
				}

				line[1] = b.getKey().toString();
				line[2] = String.valueOf(b.getValue());
				writer.writeNext(line);

				i++;
			}

		}
		writer.close();

	}

	void writeColumnDataTypes(DBpediaRepositoryNew repository, String folder) throws IOException {
		CSVWriter writer = new CSVWriter(new FileWriter(folder + "columnDataTypes"), ';', '"');
		for (Entry<ClassPropertyId, ColumnDataType> e : repository.getColumnDataTypes().entrySet()) {
			String[] line = new String[3];
			line[0] = e.getKey().getClassId().toString();
			line[1] = e.getKey().getPropertyId().toString();
			line[2] = e.getValue().name();
			writer.writeNext(line);
		}
		writer.close();

	}
}
