package de.mannheim.uni.expansion.data.dbpedia.repository;

import java.util.Collections;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import org.apache.commons.collections4.BidiMap;
import org.apache.commons.collections4.bidimap.DualHashBidiMap;
import org.apache.commons.collections4.bidimap.UnmodifiableBidiMap;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.ListType;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.ValueId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.data.tables.model.CellIdx;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.PersistentParallel;
import de.mannheim.uni.tableprocessor.model.TableColumn;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

/*
 * IMMUTABLE
 * Data can be retrieved package private
 */
public class DBpediaRepositoryNew implements DBpediaRepositoryNewInterfaceRemote, DBpediaRepositoryNewInterface {

	final Map<ClassId, Integer> tableIndex;

	final Map<ClassId, BidiMap<PropertyId, Integer>> propertyIndices;

	final Map<ClassId, BidiMap<InstanceId, Integer>> instanceIndices;

	final Map<ClassPropertyId, ColumnDataType> columnDataTypes;

	final Map<ClassId, PropertyId> classKeys;

	// final Set<TripleId> valueExists;

	final Map<ClassPropertyNamePair, PropertyId> propertyRepositories;

	final Map<ValueId, Object> values;

	final Map<ClassPropertyId, Set<InstanceId>> instancesPerClassProperty;

	final Set<ClassPropertyId> listTypes;

	final EnumMap<ListType, Set<ClassPropertyId>> listTypesMap;

	private final Map<InstanceId, Set<ClassId>> instanceMappings;

	private final Map<ClassId, Integer> instanceCount;

	public DBpediaRepositoryNew(List<AnnotatedTablesReaderData> tables, List<Map<CellIdx, InstanceId>> columnMatchesList,
			Map<ClassPropertyId, ColumnDataType> columnDataTypes) {

		this.tableIndex = new HashMap<>();

		this.propertyIndices = new HashMap<>();

		this.instanceIndices = new HashMap<>();
		this.columnDataTypes = new HashMap<>(columnDataTypes);
		this.classKeys = new HashMap<>();
		this.propertyRepositories = new ConcurrentHashMap<ClassPropertyNamePair, PropertyId>();
		this.values = new HashMap<>();

		this.instanceCount = new HashMap<>();
		this.instanceMappings = new HashMap<>();

		for (int cIndex = 0; cIndex < tables.size(); cIndex++) {
			AnnotatedTablesReaderData table = tables.get(cIndex);
			Map<CellIdx, InstanceId> columnMatches = columnMatchesList.get(cIndex);
			AnnotatedTableMetaData md = table.getMetaData();
			ClassId classId = md.getDbPediaClass();

			tableIndex.put(classId, cIndex);
			this.propertyIndices.put(classId,
					new DualHashBidiMap<>(md.getProperties().entrySet().stream().collect(Collectors.toMap(Entry::getValue, Entry::getKey))));

			Set<InstanceId> instances = new HashSet<>();
			for (Entry<Integer, InstanceId> e : md.getInstances().entrySet()) {
				if (instances.contains(e.getValue())) {
					System.out.println(e.getValue().toString());
				} else {
					instances.add(e.getValue());
				}
			}

			this.instanceIndices.put(classId,
					new DualHashBidiMap<>(md.getInstances().entrySet().stream().collect(Collectors.toMap(Entry::getValue, Entry::getKey))));

			this.classKeys.put(classId, propertyIndices.get(classId).getKey(md.getKeyColumn()));

			for (int i = 0; i < table.getTableData().getColumns().size(); i++) {
				TableColumn tc = table.getTableData().getColumns().get(i);
				for (Entry<Integer, Object> v : tc.getValues().entrySet()) {
					ValueId vid = new ValueId(instanceIndices.get(classId).getKey(v.getKey()), propertyIndices.get(classId).getKey(i));
					InstanceId columnMatch = columnMatches.get(new CellIdx(i, v.getKey()));
					if (columnMatch != null) {
						values.put(vid, columnMatch.getReadableString());
					} else {
						values.put(vid, v.getValue());
					}

				}
			}

			for (InstanceId instanceId : instanceIndices.get(classId).keySet()) {
				this.instanceMappings.put(instanceId, this.tableIndex.keySet());
			}

			this.instanceCount.put(classId, this.instanceIndices.size());

		}

		for (Entry<ClassId, BidiMap<PropertyId, Integer>> entry : propertyIndices.entrySet()) {

			for (PropertyId propertyId : entry.getValue().keySet()) {
				String shortString = propertyId.getFinalPart();
				propertyRepositories.put(new ClassPropertyNamePair(entry.getKey(), shortString), propertyId);

			}
		}

		this.instancesPerClassProperty = getInstancesPerClassProperties();

		this.listTypes = new HashSet<>();
		this.listTypesMap = new EnumMap<>(ListType.class);

	}

	public DBpediaRepositoryNew(Map<ClassId, Integer> tableIndex, Map<ClassId, BidiMap<PropertyId, Integer>> propertyIndices,
			Map<ClassId, BidiMap<InstanceId, Integer>> instanceIndices, Map<ClassPropertyId, ColumnDataType> columnDataTypes,
			Map<ClassId, PropertyId> classKeys, Map<ValueId, Object> values, Set<ClassPropertyId> listTypes,
			EnumMap<ListType, Set<ClassPropertyId>> listTypesMap, ConcurrentHashMap<ClassPropertyId, Set<InstanceId>> instancesPerClassProperty) {
		this.tableIndex = tableIndex;
		this.propertyIndices = propertyIndices.entrySet().stream().collect(Collectors.toConcurrentMap(Entry::getKey, (e) -> {
			return UnmodifiableBidiMap.unmodifiableBidiMap(e.getValue());
		}));
		this.instanceIndices = instanceIndices.entrySet().stream().collect(Collectors.toConcurrentMap(Entry::getKey, (e) -> {
			return UnmodifiableBidiMap.unmodifiableBidiMap(e.getValue());
		}));
		this.columnDataTypes = columnDataTypes;
		this.classKeys = classKeys;
		this.values = values;

		this.propertyRepositories = new ConcurrentHashMap<ClassPropertyNamePair, PropertyId>();
		for (Entry<ClassId, BidiMap<PropertyId, Integer>> entry : propertyIndices.entrySet()) {
			ClassId classId = entry.getKey();
			for (PropertyId propertyId : entry.getValue().keySet()) {
				String shortString = propertyId.getFinalPart();
				propertyRepositories.put(new ClassPropertyNamePair(classId, shortString), propertyId);

			}
		}

		// this.valueExistsValueId = new HashSet<ValueId>();
		// for (TripleId tid : values) {
		// ValueId vid = new ValueId(tid.getInstanceId(), tid.getPropertyId());
		// valueExistsValueId.add(vid);
		// }

		instanceMappings = new HashMap<InstanceId, Set<ClassId>>();
		instanceCount = new HashMap<ClassId, Integer>();

		for (Entry<ClassId, BidiMap<InstanceId, Integer>> classInstances : instanceIndices.entrySet()) {

			ClassId classId = classInstances.getKey();
			for (InstanceId instance : classInstances.getValue().keySet()) {

				Set<ClassId> classes = instanceMappings.get(instance);
				if (classes == null) {
					classes = new HashSet<ClassId>();
					instanceMappings.put(instance, classes);
				}
				classes.add(classId);

				Integer count = instanceCount.get(classId);
				if (count == null) {
					count = 0;
				} else {
					count++;
				}
				instanceCount.put(classId, count);

				// if (previous != null) {
				// System.out.println("An Instance can not exist in multiple
				// DBPedia Tables");
				// }

			}

		}

		// this.listTypes = listTypes;

		if (listTypes == null) {

			this.listTypes = Collections.newSetFromMap(new ConcurrentHashMap<>());

			this.listTypesMap = new EnumMap<>(ListType.class);

			final Set<ClassPropertyId> shortListTypes;
			final Set<ClassPropertyId> mediumListTypes;
			final Set<ClassPropertyId> longListTypes;

			shortListTypes = Collections.newSetFromMap(new ConcurrentHashMap<>());
			mediumListTypes = Collections.newSetFromMap(new ConcurrentHashMap<>());
			longListTypes = Collections.newSetFromMap(new ConcurrentHashMap<>());

			ConcurrentHashMap<ClassPropertyId, AtomicInteger> otherTypeCounts = new ConcurrentHashMap<>();
			ConcurrentHashMap<ClassPropertyId, AtomicInteger> listTypeCounts = new ConcurrentHashMap<>();
			ConcurrentHashMap<ClassPropertyId, AtomicInteger> shortListTypeCounts = new ConcurrentHashMap<>();
			ConcurrentHashMap<ClassPropertyId, AtomicInteger> mediumListTypeCounts = new ConcurrentHashMap<>();
			ConcurrentHashMap<ClassPropertyId, AtomicInteger> longListTypeCounts = new ConcurrentHashMap<>();

			CHMHelper.Creator<AtomicInteger> aiCreator = () -> {
				return new AtomicInteger(0);
			};
			PersistentParallel.runAndFinish(this.values.entrySet(), (e) -> {

				InstanceId instanceId = e.getKey().getInstanceId();
				PropertyId propertyId = e.getKey().getPropertyId();

				boolean isListType = false;
				int listSize = 0;
				if (e.getValue() instanceof List) {
					isListType = true;
					listSize = ((List<?>) e.getValue()).size();
				}

				for (ClassId classId : this.instanceMappings.get(instanceId)) {
					if (propertyExists(classId, propertyId)) {
						if (isListType) {
							AtomicInteger ai = CHMHelper.getFromCHM(listTypeCounts, new ClassPropertyId(classId, propertyId), aiCreator);
							ai.incrementAndGet();

							if (listSize == 1) {
								CHMHelper.getFromCHM(shortListTypeCounts, new ClassPropertyId(classId, propertyId), aiCreator).incrementAndGet();

							} else if (listSize > 1 && listSize <= 4) {
								CHMHelper.getFromCHM(mediumListTypeCounts, new ClassPropertyId(classId, propertyId), aiCreator).incrementAndGet();
							} else if (listSize > 4) {
								CHMHelper.getFromCHM(longListTypeCounts, new ClassPropertyId(classId, propertyId), aiCreator).incrementAndGet();
							}
						} else {
							AtomicInteger ai = CHMHelper.getFromCHM(otherTypeCounts, new ClassPropertyId(classId, propertyId), aiCreator);
							ai.incrementAndGet();
						}

					}
				}

			} , "Find List Types Step 1");

			Set<ClassPropertyId> allClassPropertIds = new HashSet<>(otherTypeCounts.keySet());
			allClassPropertIds.addAll(listTypeCounts.keySet());

			AtomicInteger defaultAi = new AtomicInteger(0);

			PersistentParallel.runAndFinish(allClassPropertIds, (classPropertyId) -> {

				int listType = listTypeCounts.getOrDefault(classPropertyId, defaultAi).get();
				int otherType = otherTypeCounts.getOrDefault(classPropertyId, defaultAi).get();
				int shortListType = shortListTypeCounts.getOrDefault(classPropertyId, defaultAi).get();
				int mediumListType = mediumListTypeCounts.getOrDefault(classPropertyId, defaultAi).get();
				int longListType = longListTypeCounts.getOrDefault(classPropertyId, defaultAi).get();

				double proportion = ((double) listType) / (((double) listType) + ((double) otherType));
				if (proportion > 0.4) {
					this.listTypes.add(classPropertyId);

					double shortProportion = ((double) shortListType) / (((double) listType) + ((double) otherType));
					double mediumProportion = ((double) mediumListType) / (((double) listType) + ((double) otherType));
					double longProportion = ((double) longListType) / (((double) listType) + ((double) otherType));

					if (mediumProportion >= shortProportion && mediumProportion >= longProportion) {
						mediumListTypes.add(classPropertyId);
					} else if (longProportion >= mediumProportion && longProportion >= shortProportion) {
						longListTypes.add(classPropertyId);
					} else if (shortProportion >= mediumProportion && shortProportion >= longProportion) {
						shortListTypes.add(classPropertyId);
					}

				}

			} , "Find Lilst Types Step 2");
			this.listTypesMap.put(ListType.SHORT, shortListTypes);

			this.listTypesMap.put(ListType.MEDIUM, mediumListTypes);
			this.listTypesMap.put(ListType.LONG, longListTypes);

		} else {
			this.listTypes = listTypes;
			this.listTypesMap = listTypesMap;
		}

		System.out.println();

		System.out.println("AllTypes: " + columnDataTypes.size());

		System.out.println("ListTypes: " + this.listTypes.size());

		System.out.println("Short List Types: " + this.listTypesMap.get(ListType.SHORT).size());

		System.out.println("Medium List Types: " + this.listTypesMap.get(ListType.MEDIUM).size());

		System.out.println("Long List Types: " + this.listTypesMap.get(ListType.LONG).size());

		System.out.println();

		if (instancesPerClassProperty != null) {

			this.instancesPerClassProperty = instancesPerClassProperty.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
				return Collections.unmodifiableSet(e.getValue());
			}));
		} else {
			this.instancesPerClassProperty = getInstancesPerClassProperties();
		}

	}

	private Map<ClassPropertyId, Set<InstanceId>> getInstancesPerClassProperties() {
		ConcurrentHashMap<ClassPropertyId, Set<InstanceId>> instancesPerClassProperty2 = new ConcurrentHashMap<>();

		PersistentParallel<Entry<InstanceId, Integer>> pp = new PersistentParallel<Map.Entry<InstanceId, Integer>>("Compute ClassProperty Instance-Indices");

		Creator<Set<InstanceId>> hashSetCreator = () -> {
			return new HashSet<>();
		};

		for (ClassId classId : this.getClasses()) {
			try {
				pp.foreach(this.getInstances(classId).entrySet(), (e) -> {
					for (PropertyId propertyId : this.getProperties(classId).keySet()) {

						Set<InstanceId> set = CHMHelper.getFromCHM(instancesPerClassProperty2, new ClassPropertyId(classId, propertyId), hashSetCreator);

						if (this.hasValue(classId, propertyId, e.getKey())) {
							synchronized (set) {
								set.add(e.getKey());
							}

						}

					}
				});
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

		Map<ClassPropertyId, Set<InstanceId>> instancesPerClassProperty = instancesPerClassProperty2.entrySet().stream()
				.collect(Collectors.toMap(Entry::getKey, (e) -> {
					return Collections.unmodifiableSet(e.getValue());
				}));

		return instancesPerClassProperty;
	}

	@Override
	public Map<ClassId, PropertyId> getClassKeys() {
		return Collections.unmodifiableMap(this.classKeys);
	}

	@Override
	public Map<ValueId, Object> getValues() {
		return Collections.unmodifiableMap(this.values);
	}

	@Override
	public Set<ClassPropertyId> getListTypes() {
		return Collections.unmodifiableSet(this.listTypes);
	}

	@Override
	public Set<ClassPropertyId> getListTypes(ListType listType) {
		return Collections.unmodifiableSet(this.listTypesMap.get(listType));
	}

	@Override
	public Map<ClassId, Integer> getTableIndeces() {
		return Collections.unmodifiableMap(this.tableIndex);
	}

	@Override
	public Map<ClassId, BidiMap<PropertyId, Integer>> getPropertyIndices() {
		return Collections.unmodifiableMap(this.propertyIndices);
	}

	@Override
	public Map<ClassId, BidiMap<InstanceId, Integer>> getInstanceIndices() {
		return Collections.unmodifiableMap(this.instanceIndices);
	}

	@Override
	public Map<ClassPropertyId, ColumnDataType> getColumnDataTypes() {
		return Collections.unmodifiableMap(this.columnDataTypes);
	}

	@Override
	public Map<ClassPropertyId, Set<InstanceId>> getInstancesPerClassProperty() {
		return Collections.unmodifiableMap(this.instancesPerClassProperty);
	}

	@Override
	public Object getValue(TripleId key) {

		// return getValue(key.getClassId(), key.getInstanceId(),
		// key.getPropertyId());

		return getValue(new ValueId(key.getInstanceId(), key.getPropertyId()));
	}

	@Override
	public Object getValue(ClassId dbPediaClass, InstanceId dbPediaInstance, PropertyId propertyKey) {
		Object value = getValue(new ValueId(dbPediaInstance, propertyKey));
		return value;

		// Integer tableIndex = this.tableIndex.get(dbPediaClass);
		// if (tableIndex == null)
		// return null;
		//
		// Integer column =
		// this.propertyIndices.get(dbPediaClass).get(propertyKey);
		//
		// if (column == null) {
		// return null;
		// }
		//
		// Integer instanceIndex =
		// this.instanceIndices.get(dbPediaClass).get(dbPediaInstance);
		//
		// if (instanceIndex == null)
		// return null;
		//
		// return getValue(tableIndex, instanceIndex, column);
	}

	@Override
	public Object getValue(ValueId key) {

		return values.get(key);
	}

	// public Object getValue(int table, int instance, int column) {
	// try {
	// String fullKey = "DP_" + table + "_" + instance + "_" + column;
	//
	// SerializableDocument doc = bucket.get(fullKey,
	// SerializableDocument.class);
	// if (doc == null)
	// return doc;
	// else
	// return doc.content();
	// } catch (RuntimeException e) {
	// if (e.getCause() instanceof TimeoutException) {
	// return getValue(table, instance, column);
	// } else {
	// throw e;
	// }
	// }
	// }

	@Override
	public int getTotalNumberOfRows() {
		int count = 0;
		for (Map<InstanceId, Integer> table : instanceIndices.values()) {
			count += table.size();
		}

		return count;
	}

	@Override
	public Set<ClassId> getClasses() {
		return tableIndex.keySet();
	}

	@Override
	public BidiMap<InstanceId, Integer> getInstances(ClassId currentTable) {
		BidiMap<InstanceId, Integer> map = instanceIndices.get(currentTable);

		if (map != null) {

			return map;
		} else {
			return new DualHashBidiMap<InstanceId, Integer>();
		}

	}

	@Override
	public int getTotalNumberOfColumns() {
		int count = 0;
		for (Map<PropertyId, Integer> table : propertyIndices.values()) {
			count += table.size();
		}

		return count;
	}

	@Override
	public BidiMap<PropertyId, Integer> getProperties(ClassId currentTable) {
		return propertyIndices.get(currentTable);
	}

	@Override
	public int getTotalNumberOfClasses() {
		return tableIndex.size();
	}

	@Override
	public int getNumberOfRows(ClassId dbPediaClass) {

		BidiMap<InstanceId, Integer> bidiMap = instanceIndices.get(dbPediaClass);
		if (bidiMap == null) {
			return 0;
		} else {
			return bidiMap.size();
		}
	}

	@Override
	public int getNumberOfColumns(ClassId classId) {
		return propertyIndices.get(classId).size();
	}

	@Override
	public InstanceId getInstance(ClassId dbPediaClass, int index) {
		BidiMap<InstanceId, Integer> indices = instanceIndices.get(dbPediaClass);
		InstanceId instanceId = indices.getKey(index);
		return instanceId;
	}

	@Override
	public int getNumberOfHeaderRows(ClassId currentTable) {
		return 4;
	}

	@Override
	public int getInstanceIndex(InstanceId instanceId, ClassId classId) {
		if (classId == null)
			System.out.println("a");
		BidiMap<InstanceId, Integer> instances = instanceIndices.get(classId);

		if (instances != null) {
			Integer index = instances.get(instanceId);
			if (index != null) {
				return index;
			} else {
				throw new IllegalStateException("This instance does not exist in this class");
			}
		} else {
			throw new IllegalStateException("This class does not exist in the repository");
		}
	}

	@Override
	public int getNumberOfInstances() {
		return instanceMappings.size();
	}

	@Override
	public int getNumberOfInstances(ClassId classId) {
		return instanceCount.get(classId);
	}

	@Override
	public Set<ClassId> getClasses(InstanceId instanceId) {
		return instanceMappings.get(instanceId);
	}

	@Override
	public PropertyId getProperty(ClassId classId, int index) {
		BidiMap<PropertyId, Integer> properties = propertyIndices.get(classId);

		if (properties != null) {
			PropertyId propertyId = properties.getKey(index);
			if (propertyId != null) {
				return propertyId;
			} else {
				throw new IllegalStateException("There is no property for the given index in this class");
			}
		} else {
			throw new IllegalStateException("This class does not exist in the repository");
		}
	}

	@Override
	public ColumnDataType getDataType(ClassId dbPediaClass, PropertyId dBpediaProperty) {
		return columnDataTypes.get(new ClassPropertyId(dbPediaClass, dBpediaProperty));
	}

	@Override
	public boolean isKey(ClassId dbPediaClass, PropertyId dBpediaProperty) {
		PropertyId key = classKeys.get(dbPediaClass);
		if (key == null) {
			throw new IllegalStateException("This class does not exist in the repository");
		} else if (key.equals(dBpediaProperty)) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public PropertyId getClassKey(ClassId classId) {
		return this.classKeys.get(classId);
	}

	@Override
	public boolean hasValue(TripleId tripleId) {
		// if (getValue(tripleId) != null)
		// return true;
		// else
		// return false;

		return hasValue(new ValueId(tripleId.getInstanceId(), tripleId.getPropertyId()));
	}

	@Override
	public boolean hasValue(ValueId valueId) {
		// if (getValue(tripleId) != null)
		// return true;
		// else
		// return false;

		return values.get(valueId) != null;
	}

	@Override
	public boolean hasValue(ClassId classId, PropertyId propertyId, InstanceId instanceId) {
		// return hasValue(new TripleId(classId, propertyId, instanceId));
		return hasValue(new ValueId(instanceId, propertyId));
	}

	@Override
	public boolean hasValue(InstanceId instanceId, PropertyId propertyId) {
		return hasValue(new ValueId(instanceId, propertyId));
	}

	ConcurrentHashMap<ClassPropertyId, BidiMap<InstanceId, Integer>> instancesCache = new ConcurrentHashMap<>();

	@Override
	public Set<InstanceId> getInstances(ClassId classId, PropertyId propertyId) {
		Set<InstanceId> map = instancesPerClassProperty.get(new ClassPropertyId(classId, propertyId));

		if (map == null) {

			map = new HashSet<InstanceId>();
		}
		return map;

	}

	@Override
	public int getInstancesSize(ClassId classId, PropertyId propertyId) {
		Set<InstanceId> set = instancesPerClassProperty.get(new ClassPropertyId(classId, propertyId));
		int size = 0;
		if (set != null) {
			size = set.size();

		}

		return size;
	}

	@Override
	public boolean hasClass(ClassId classId) {
		return instanceCount.containsKey(classId);
	}

	@Override
	public PropertyId getPropertyFromShortName(ClassId classId, String propertyName) {
		PropertyId propertyId = propertyRepositories.get(new ClassPropertyNamePair(classId, propertyName));
		if (propertyId == null) {
			// System.out.println("Couldn't find this property : " +
			// propertyName);
			return null;
		} else {
			return propertyId;
		}
	}

	public static class ClassPropertyNamePair {
		private final ClassId classId;
		private final String propertyName;

		public ClassPropertyNamePair(ClassId classId, String propertyName) {
			super();
			this.classId = classId;
			this.propertyName = propertyName;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((classId == null) ? 0 : classId.hashCode());
			result = prime * result + ((propertyName == null) ? 0 : propertyName.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			ClassPropertyNamePair other = (ClassPropertyNamePair) obj;
			if (classId == null) {
				if (other.classId != null)
					return false;
			} else if (!classId.equals(other.classId))
				return false;
			if (propertyName == null) {
				if (other.propertyName != null)
					return false;
			} else if (!propertyName.equals(other.propertyName))
				return false;
			return true;
		}

		public ClassId getClassId() {
			return classId;
		}

		public String getPropertyName() {
			return propertyName;
		}

	}

	@Override
	public boolean propertyExists(ClassId classId, PropertyId propertyId) {
		BidiMap<PropertyId, Integer> map = propertyIndices.get(classId);
		if (map != null && map.containsKey(propertyId)) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public boolean isListType(ClassPropertyId classPropertyId) {
		return listTypes.contains(classPropertyId);
	}

	@Override
	public boolean isListType(ListType listType, ClassPropertyId classPropertyId) {
		return listTypesMap.get(listType).contains(classPropertyId);
	}

	@Override
	public Integer getPropertyIndex(ClassId classId, PropertyId propertyId) {
		BidiMap<PropertyId, Integer> indices = this.propertyIndices.get(classId);
		if (indices != null) {
			return indices.get(propertyId);
		}
		return null;
	}

	@Override
	public boolean hasInstance(ClassId classId, InstanceId instanceId) {
		BidiMap<InstanceId, Integer> map = instanceIndices.get(classId);
		if (map != null && map.containsKey(instanceId)) {
			return true;
		} else {
			return false;
		}
	}

}
