package de.mannheim.uni.expansion.data.tables.model;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.utils.CHMHelper;

public class AnnotatedTableMetaData implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2L;
	private final DomainSource source;
	private final HostId hostId;
	private final ClassId dbPediaClass;
	private final double classConf;
	private final int numHeaderRows;
	private final Map<Integer, List<PropertyId>> properties;
	// private final Map<Integer, Double> propertiesScores;
	private final Map<Integer, List<InstanceId>> instances;
	// private final Map<Integer, Double> instancesScores;
	private final int keyColumn;

	private final Map<InstanceId, List<Integer>> reverseInstances;
	private final Map<PropertyId, List<Integer>> reverseProperties;

	public final Map<PropertyIndexComposite, Double> propertiesScores;
	public final Map<InstanceIndexComposite, Double> instancesScores;

	private final Map<Integer, PropertyId> highestRatedProperties;
	private final Map<Integer, InstanceId> highestRatedInstances;

	private final List<Integer> yearsBeforeText;
	private final Integer maxYearBeforeText;
	private final List<Integer> yearsAfterText;
	private final Integer maxYearAfterText;
	private final List<Integer> yearsInContextBefore;
	private final Integer maxYearInContextBefore;
	private final List<Integer> yearsInTableTitle;
	private final Integer maxYearInTableTitle;
	private final List<Integer> yearsInPageTitle;
	private final Integer maxYearInPageTitle;

	private final Map<Integer, List<Integer>> propertiesYears;
	private final Map<Integer, Integer> propertiesMaxYear;

	private final String pageTitle;
	private final String tableTitle;
	private final String tableContextTimeStampBeforeTable;
	private final String textBeforeTable;
	private final String textAfterTable;

	public AnnotatedTableMetaData(DomainSource source, ClassId dbPediaClass, double classConf, int numHeaderRows, Map<Integer, List<PropertyId>> properties,
			Map<PropertyIndexComposite, Double> propertiesScores, Map<Integer, List<InstanceId>> instances, Map<InstanceIndexComposite, Double> instancesScores,
			int keyColumn) {
		this(source, dbPediaClass, classConf, numHeaderRows, properties, propertiesScores, instances, instancesScores, keyColumn, null, null, null, null, null,
				null, null, null, null, null, null);
	}

	public AnnotatedTableMetaData(DomainSource source, ClassId dbPediaClass, double classConf, int numHeaderRows, Map<Integer, List<PropertyId>> properties,
			Map<PropertyIndexComposite, Double> propertiesScores, Map<Integer, List<InstanceId>> instances, Map<InstanceIndexComposite, Double> instancesScores,
			int keyColumn, List<Integer> beforeTextYears, List<Integer> afterTextYears, List<Integer> contextBeforeYears, List<Integer> tableTitleYears,
			List<Integer> pageTitleYears, Map<Integer, List<Integer>> propertiesYears, String pageTitle, String tableTitle,
			String tableContextTimeStampBeforeTable, String textBeforeTable, String textAfterTable) {
		super();
		this.source = source;
		this.hostId = source.getHostId();
		this.dbPediaClass = dbPediaClass;
		this.classConf = classConf;
		this.numHeaderRows = numHeaderRows;
		this.propertiesScores = propertiesScores;
		this.instancesScores = instancesScores;
		this.keyColumn = keyColumn;

		// WORKING WITH INSTANCES
		Comparator<InstanceIndexComposite> instanceComparator = (i1, i2) -> {
			return instancesScores.get(i1).compareTo(instancesScores.get(i2));

		};

		HashMap<Integer, List<InstanceId>> tempInstances = new HashMap<Integer, List<InstanceId>>();
		for (Entry<Integer, List<InstanceId>> e : instances.entrySet()) {
			final Integer index = e.getKey();

			List<InstanceId> sortedList = e.getValue();
			if (sortedList != null) {
				sortedList = e.getValue().stream().map((p) -> {
					return new InstanceIndexComposite(index, p);
				}).sorted(instanceComparator).map((p) -> {
					return p.getInstanceId();
				}).collect(Collectors.toList());
			}

			tempInstances.put(e.getKey(), sortedList);

		}
		this.instances = tempInstances;

		highestRatedInstances = this.instances.entrySet().stream().filter((e) -> {
			return (e.getValue() != null && !e.getValue().isEmpty());
		}).collect(Collectors.toMap(Entry::getKey, (e) -> {

			return e.getValue().get(0);
		}));

		HashMap<InstanceId, List<Integer>> reverseInstancesTemp = new HashMap<InstanceId, List<Integer>>();

		for (Entry<Integer, List<InstanceId>> e : instances.entrySet()) {

			if (e.getValue() != null) {
				for (InstanceId id : e.getValue()) {
					List<Integer> indeces = reverseInstancesTemp.get(id);
					if (indeces == null) {
						indeces = new ArrayList<Integer>();
						reverseInstancesTemp.put(id, indeces);
					}
					indeces.add(e.getKey());
				}
			}

		}

		for (Entry<InstanceId, List<Integer>> e : reverseInstancesTemp.entrySet()) {
			final InstanceId instanceId = e.getKey();

			List<Integer> sortedList = e.getValue().stream().map((p) -> {
				return new InstanceIndexComposite(p, instanceId);
			}).sorted(instanceComparator).map((p) -> {
				return p.getIndex();
			}).collect(Collectors.toList());

			reverseInstancesTemp.put(e.getKey(), sortedList);

		}

		reverseInstances = reverseInstancesTemp;

		// WORKING WITH PROPERTIES
		Comparator<PropertyIndexComposite> propertyComparator = (p1, p2) -> {
			return propertiesScores.get(p1).compareTo(propertiesScores.get(p2));

		};

		HashMap<Integer, List<PropertyId>> tempProperties = new HashMap<Integer, List<PropertyId>>();
		for (Entry<Integer, List<PropertyId>> e : properties.entrySet()) {
			final Integer index = e.getKey();

			List<PropertyId> sortedList = e.getValue().stream().map((p) -> {
				return new PropertyIndexComposite(index, p);
			}).sorted(propertyComparator).map((p) -> {
				return p.getPropertyId();
			}).collect(Collectors.toList());

			tempProperties.put(e.getKey(), sortedList);

		}
		this.properties = tempProperties;

		highestRatedProperties = this.properties.entrySet().stream().filter((e) -> {
			return e.getValue() != null && !e.getValue().isEmpty();
		}).collect(Collectors.toMap(Entry::getKey, (e) -> {
			return e.getValue().get(0);
		}));

		HashMap<PropertyId, List<Integer>> reversePropertiesTemp = new HashMap<PropertyId, List<Integer>>();

		for (Entry<Integer, List<PropertyId>> e : properties.entrySet())

		{

			for (PropertyId id : e.getValue()) {
				List<Integer> indeces = reversePropertiesTemp.get(id);
				if (indeces == null) {
					indeces = new ArrayList<Integer>();
					reversePropertiesTemp.put(id, indeces);
				}
				indeces.add(e.getKey());
			}

		}

		for (

		Entry<PropertyId, List<Integer>> e : reversePropertiesTemp.entrySet())

		{
			final PropertyId propertyId = e.getKey();

			List<Integer> sortedList = e.getValue().stream().map((p) -> {
				return new PropertyIndexComposite(p, propertyId);
			}).sorted(propertyComparator).map((p) -> {
				return p.getIndex();
			}).collect(Collectors.toList());

			reversePropertiesTemp.put(e.getKey(), sortedList);

		}

		reverseProperties = reversePropertiesTemp;

		if (beforeTextYears != null && !beforeTextYears.isEmpty()) {
			this.yearsBeforeText = new ArrayList<>(beforeTextYears);
			this.maxYearBeforeText = Collections.max(beforeTextYears);
		} else {
			this.yearsBeforeText = new ArrayList<>();
			this.maxYearBeforeText = null;
		}

		if (afterTextYears != null && !afterTextYears.isEmpty()) {
			this.yearsAfterText = new ArrayList<>(afterTextYears);
			this.maxYearAfterText = Collections.max(afterTextYears);
		} else {
			this.yearsAfterText = new ArrayList<>();
			this.maxYearAfterText = null;
		}

		if (contextBeforeYears != null && !contextBeforeYears.isEmpty()) {
			this.yearsInContextBefore = new ArrayList<>(contextBeforeYears);
			this.maxYearInContextBefore = Collections.max(contextBeforeYears);
		} else {
			this.yearsInContextBefore = new ArrayList<>();
			this.maxYearInContextBefore = null;
		}

		if (tableTitleYears != null && !tableTitleYears.isEmpty()) {
			this.yearsInTableTitle = new ArrayList<>(tableTitleYears);
			this.maxYearInTableTitle = Collections.max(tableTitleYears);
		} else {
			this.yearsInTableTitle = new ArrayList<>();
			this.maxYearInTableTitle = null;
		}

		if (pageTitleYears != null && !pageTitleYears.isEmpty()) {
			this.yearsInPageTitle = new ArrayList<>(pageTitleYears);
			this.maxYearInPageTitle = Collections.max(pageTitleYears);
		} else {
			this.yearsInPageTitle = new ArrayList<>();
			this.maxYearInPageTitle = null;
		}

		if (propertiesYears != null && !propertiesYears.isEmpty()) {
			this.propertiesMaxYear = new HashMap<>();
			this.propertiesYears = new HashMap<>();
			for (Entry<Integer, List<Integer>> e : propertiesYears.entrySet()) {
				if (e.getValue() != null) {
					this.propertiesMaxYear.put(e.getKey(), Collections.max(e.getValue()));
					this.propertiesYears.put(e.getKey(), new ArrayList<>(e.getValue()));
				}
			}

		} else {
			this.propertiesMaxYear = null;
			this.propertiesYears = null;
		}

		this.pageTitle = pageTitle;
		this.tableTitle = tableTitle;
		this.tableContextTimeStampBeforeTable = tableContextTimeStampBeforeTable;
		this.textBeforeTable = textBeforeTable;
		this.textAfterTable = textAfterTable;
	}

	public List<Integer> getYearsBeforeText() {
		return yearsBeforeText;
	}

	public Integer getMaxYearBeforeText() {
		return maxYearBeforeText;
	}

	public List<Integer> getYearsAfterText() {
		return yearsAfterText;
	}

	public Integer getMaxYearAfterText() {
		return maxYearAfterText;
	}

	public List<Integer> getYearsInContextBefore() {
		return yearsInContextBefore;
	}

	public Integer getMaxYearInContextBefore() {
		return maxYearInContextBefore;
	}

	public List<Integer> getYearsInTableTitle() {
		return yearsInTableTitle;
	}

	public Integer getMaxYearInTableTitle() {
		return maxYearInTableTitle;
	}

	public List<Integer> getYearsInPageTitle() {
		return yearsInPageTitle;
	}

	public Integer getMaxYearInPageTitle() {
		return maxYearInPageTitle;
	}

	public Map<Integer, List<Integer>> getPropertiesYears() {
		if (propertiesYears == null)
			return null;
		else {
			Map<Integer, List<Integer>> ret = new HashMap<>();
			for (Entry<Integer, List<Integer>> e : this.propertiesYears.entrySet()) {
				ret.put(e.getKey(), Collections.unmodifiableList(e.getValue()));
			}
			return Collections.unmodifiableMap(ret);
		}
	}

	public Map<Integer, Integer> getPropertiesMaxYear() {
		if (propertiesMaxYear == null)
			return null;
		else
			return Collections.unmodifiableMap(propertiesMaxYear);
	}

	public String getPageTitle() {
		return pageTitle;
	}

	public String getTableTitle() {
		return tableTitle;
	}

	public String getTableContextTimeStampBeforeTable() {
		return tableContextTimeStampBeforeTable;
	}

	public String getTextBeforeTable() {
		return textBeforeTable;
	}

	public String getTextAfterTable() {
		return textAfterTable;
	}

	public DomainSource getSource() {
		return source;
	}

	public ClassId getDbPediaClass() {
		return dbPediaClass;
	}

	public double getClassConf() {
		return classConf;
	}

	public int getNumHeaderRows() {
		return numHeaderRows;
	}

	public Map<Integer, PropertyId> getProperties() {

		return highestRatedProperties;
	}

	public int getNumberOfRows() {
		return instances.size();
	}

	public Map<Integer, List<InstanceId>> getAllInstances() {
		return instances;
	}

	public Map<Integer, InstanceId> getInstances() {
		return highestRatedInstances;
	}

	public int getKeyColumn() {
		return keyColumn;
	}

	public int getNumOfColums() {
		return this.properties.size();
	}

	public int getRow(InstanceId instance) {
		return reverseInstances.get(instance).get(0);
	}

	public int getCol(PropertyId property) {
		return reverseProperties.get(property).get(0);
	}

	public HostId getHostId() {
		return hostId;
	}

	public InstanceId getInstance(int index) {
		return highestRatedInstances.get(index);
	}

	public PropertyId getProperty(int index) {
		return highestRatedProperties.get(index);
	}

	public Map<Integer, List<PropertyId>> getAllProperties() {
		return properties;
	}

	public Map<PropertyIndexComposite, Double> getAllPropertyScores() {
		return propertiesScores;
	}

	private transient ConcurrentHashMap<InstanceId, Double> maxScorePerEntity = new ConcurrentHashMap<>();

	public double getScore(InstanceId instanceId) {
		Double score = CHMHelper.getFromCHM(maxScorePerEntity, instanceId, () -> {
			double max = 0.0;
			for (Integer i : this.reverseInstances.get(instanceId)) {
				double tempScore = instancesScores.get(new InstanceIndexComposite(i, instanceId));
				if (tempScore > max)
					max = tempScore;
			}
			return max;
		});
		if (score != null) {
			return score;
		} else {
			return 0.0;
		}

	}

	public Map<InstanceIndexComposite, Double> getInstancesScores() {
		return this.instancesScores;
	}

	public Double getInstanceScore(InstanceIndexComposite instanceIndexComposite) {
		return this.instancesScores.get(instanceIndexComposite);
	}

	public Double getPropertyScore(PropertyIndexComposite propertyIndexComposite) {
		return this.propertiesScores.get(propertyIndexComposite);
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {

		in.defaultReadObject();
		this.maxScorePerEntity = new ConcurrentHashMap<>();
	}

	public AnnotatedTableMetaData filterOut(Set<PropertyId> removeProperties, Set<InstanceId> removeInstances) {

		Map<Integer, List<PropertyId>> properties = new HashMap<>();
		Map<Integer, List<InstanceId>> instances = new HashMap<>();
		Map<PropertyIndexComposite, Double> propertiesScores = new HashMap<>(this.propertiesScores);
		Map<InstanceIndexComposite, Double> instancesScores = new HashMap<>(this.instancesScores);

		for (Entry<Integer, List<PropertyId>> e : this.properties.entrySet()) {
			for (PropertyId propertyId : e.getValue()) {
				if (!removeProperties.contains(propertyId)) {
					CHMHelper.getFromMap(properties, e.getKey(), () -> new ArrayList<>()).add(propertyId);
				} else {
					propertiesScores.remove(new PropertyIndexComposite(e.getKey(), propertyId));
				}
			}
		}

		for (Entry<Integer, List<InstanceId>> e : this.instances.entrySet()) {
			for (InstanceId instanceId : e.getValue()) {
				if (!removeProperties.contains(instanceId)) {
					CHMHelper.getFromMap(instances, e.getKey(), () -> new ArrayList<>()).add(instanceId);
				} else {
					propertiesScores.remove(new InstanceIndexComposite(e.getKey(), instanceId));
				}
			}
		}

		return new AnnotatedTableMetaData(getSource(), getDbPediaClass(), getClassConf(), getNumHeaderRows(), properties, propertiesScores, instances,
				instancesScores, getKeyColumn(), getYearsBeforeText(), getYearsAfterText(), getYearsInContextBefore(), getYearsInTableTitle(),
				getYearsInPageTitle(), getPropertiesYears(), getPageTitle(), getTableTitle(), getTableContextTimeStampBeforeTable(), getTextBeforeTable(),
				getTextAfterTable());
	}

}
