package de.mannheim.uni.expansion.data.tables.model;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.inject.Inject;

import de.mannheim.uni.tableprocessor.ColumnType;
import de.mannheim.uni.tableprocessor.ColumnTypeGuesser;
import de.mannheim.uni.tableprocessor.IO.ConvertFileToTable;
import de.mannheim.uni.tableprocessor.IO.StringNormalizer;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;
import de.mannheim.uni.tableprocessor.model.TableColumnBuilder;
import de.mannheim.uni.units.UnitParserDomi;
import de.mannheim.uni.units.Unit_domi;
import de.mannheim.uni.util.Variables;
import de.mannheim.uni.utils.concurrent.Consumer;
import de.mannheim.uni.utils.concurrent.Parallel;
import de.mannheim.uni.utils.concurrent.Producer;
import de.mannheim.uni.utils.data.Pair;

public class TableFactory {

	private Boolean cleanHeader = true;

	private ColumnTypeGuesser typeGuesser;
	private ConvertFileToTable convertFileToTable;

	@Inject
	public TableFactory(ColumnTypeGuesser typeGuesser, ConvertFileToTable convertFileToTable) {

		this.typeGuesser = typeGuesser;
		this.convertFileToTable = convertFileToTable;
	}

	private Table readWebTable(AnnotatedTableId id, AnnotatedTableMetaData metaData, List<String[]> rows) {
		if (Variables.useUnitDetection) {
			UnitParserDomi.readInUnits();
		}
		// create new table
		final Table table = new Table();
		Table result = table;
		// take care of the header of the table
		table.setHeader(id.getTableName());

		table.setNumHeaderRows(metaData.getNumHeaderRows());
		table.setSource(metaData.getSource().getSource());
		table.setHeader(id.getTableName());
		table.setFullPath(id.getTableName());

		final List<TableColumnBuilder> colBuilders = new LinkedList<>();

		try {

			// read headers
			String[] columnNames = rows.get(0);

			final Unit_domi[] columnUnits = new Unit_domi[columnNames.length];

			// set the header for each column (take the first row!)
			int colIdx = 0;
			for (String columnName : columnNames) {
				TableColumnBuilder b = new TableColumnBuilder(table);

				// set the header
				String header = columnName;
				if (cleanHeader) {
					header = StringNormalizer.cleanWebHeader(StringNormalizer.simpleStringNormalization(columnName, false));
				}
				b.setHeader(header);

				// parse units from headers
				columnUnits[colIdx++] = UnitParserDomi.parseUnitFromHeader(columnName);

				colBuilders.add(b);
				table.getColumns().add(b.getColumn());
			}
			table.setNumHeaderRows(1);

			final int rowNumber = 1;

			// read the file sequentially, but process the rows in parallel
			// (processing is much slower than reading)
			new Parallel<Pair<Integer, String[]>>().producerConsumer(new Producer<Pair<Integer, String[]>>() {
				@Override
				public void execute() {
					int row = rowNumber;

					for (int i = 1; i < rows.size(); i++) {
						String[] values = rows.get(i);
						produce(new Pair<>(row++, values));
					}

				}
			}, new Consumer<Pair<Integer, String[]>>() {
				@Override
				public void execute(Pair<Integer, String[]> parameter) {
					int rowNumber = parameter.getFirst();
					String[] values = parameter.getSecond();

					int columnIndex = 0;

					String last = null;
					int duplicateCount = 0;

					// check for spanning cells
					for (String columnValue : values) {

						if (columnValue != null && columnValue.equals(last)) {
							duplicateCount++;
						}
						last = columnValue;
					}

					if (duplicateCount > convertFileToTable.getSpanningCellThreshold() && convertFileToTable.getSpanningCellThreshold() > 0) {
						// this row contains spanning cells, so we ignore it
						return;
					}

					for (String columnValue : values) {

						if (columnIndex < table.getColumns().size()) {

							TableColumnBuilder b = colBuilders.get(columnIndex);

							// TODO: replace all HTML encodings
							if (columnValue.contains("&mdash")) {
								columnValue = columnValue.replace("&mdash", "-");
							}

							// handle lists
							if (convertFileToTable.checkIfList(columnValue)) {
								// valueType = ColumnDataType.list;
								// split values, normalise, determine type, ...
							} else {
								// normalise the value
								if (Variables.normalizeValues) {
									columnValue = convertFileToTable.normaliseValue(columnValue);
								}
							}

							// guess the type
							ColumnType valueType = typeGuesser.guessTypeForValue(columnValue, columnUnits[columnIndex]);

							if (convertFileToTable.checkIfList(columnValue)) {
								List<String> columnValues;
								columnValue = columnValue.replace("{", "");
								columnValue = columnValue.replace("}", "");
								columnValues = Arrays.asList(columnValue.split("\\|"));
								b.addValue(rowNumber, columnValues, valueType);
							} else {
								b.addValue(rowNumber, columnValue, valueType);
							}
						}
						columnIndex++;
					}
				}
			});

		} catch (Exception e) {
			e.printStackTrace();
			result = null;
		}

		for (TableColumnBuilder b : colBuilders) {
			b.buildColumn();
		}

		for (TableColumn c : table.getColumns()) {
			c.setDataSource(table.getHeader());
		}

		table.getColumns().get(metaData.getKeyColumn()).setKey(true);

		return result;
	}

	private Table readLODTable(AnnotatedTableId id, AnnotatedTableMetaData metaData, List<String[]> rows) {
		if (Variables.useUnitDetection) {
			UnitParserDomi.readInUnits();
		}

		final Table table = new Table();
		// take care of the header of the table
		table.setHeader(id.getTableName());

		table.setNumHeaderRows(metaData.getNumHeaderRows());
		table.setSource(metaData.getSource().getSource());
		table.setHeader(id.getTableName());
		table.setFullPath(id.getTableName());

		final List<TableColumnBuilder> colBuilders = new LinkedList<>();

		try {
			String[] columnNames;
			String[] columntypes;
			String[] columnURIs;
			// read the property names
			columnNames = rows.get(0);

			// read the property URIs
			columnURIs = rows.get(1);

			// read the datatypes
			columntypes = rows.get(2);

			final Unit_domi[] columnUnits = new Unit_domi[columnNames.length];

			// process all properties (=columns)
			int i = 0;
			for (String columnName : columnNames) {

				// replace trailing " for the last column
				columntypes[i] = columntypes[i].replace("\"", "");
				columnURIs[i] = columnURIs[i].replace("\"", "");
				columnName = columnName.replace("\"", "");

				// create the ColumnBuilder
				TableColumnBuilder b = new TableColumnBuilder(table);
				b.setHeader(columnName);
				b.setUri(columnURIs[i]);

				// set the type if it's a primitive
				// TODO what about other primitive types?
				String datatype = columntypes[i];
				switch (datatype) {
				case "XMLSchema#date":
				case "XMLSchema#gYear":
					b.setDataType(ColumnDataType.date);
					break;
				case "XMLSchema#double":
				case "XMLSchema#float":
				case "XMLSchema#nonNegativeInteger":
				case "XMLSchema#positiveInteger":
				case "XMLSchema#integer":
				case "XMLSchema#negativeInteger":
					b.setDataType(ColumnDataType.numeric);
					break;
				case "XMLSchema#string":
					b.setDataType(ColumnDataType.string);
					break;

				default:
					b.setDataType(ColumnDataType.unknown);
				}

				columnUnits[i] = UnitParserDomi.parseUnitFromHeader(columnName);

				colBuilders.add(b);
				// add the column to the table
				table.getColumns().add(b.getColumn());
				i++;
			}

			// skip the last header

			table.setNumHeaderRows(4);

			// start processing the table contents

			// long start = System.currentTimeMillis();

			// the absolute row number in the file (offset of 4!)
			final int rowIndex = 4;

			new Parallel<Pair<Integer, String[]>>().producerConsumer(new Producer<Pair<Integer, String[]>>() {
				@Override
				public void execute() {

					int row = rowIndex;

					for (int i = 4; i < rows.size(); i++) {
						produce(new Pair<>(row++, rows.get(i)));
					}

					table.setTotalNumOfRows(row);
					// System.err.println("num rows: " + row);

				}
			}, new Consumer<Pair<Integer, String[]>>() {
				@Override
				public void execute(Pair<Integer, String[]> parameter) {

					int rowIndex = parameter.getFirst();

					// iterate all columns
					int columnIndex = 0;
					for (String columnValue : parameter.getSecond()) {
						TableColumnBuilder b = null;

						if (columnIndex >= colBuilders.size()) {
							return;
						} else {
							b = colBuilders.get(columnIndex);
						}
						ColumnType valueType = new ColumnType(b.getDataType(), null);

						if (!columnValue.equalsIgnoreCase(Variables.nullValue)) {

							// guess the type if not already set
							boolean list = false;
							if (table.getColumns().get(columnIndex).getDataType() == ColumnDataType.unknown) {

								// detect the unit and guess the type
								// TODO: use the determined datatype? e.g. for
								// populationDensity?
								valueType = typeGuesser.guessTypeForValue(columnValue, columnUnits[columnIndex]);

							}
							if (convertFileToTable.checkIfList(columnValue)) {
								List<String> columnValues;
								columnValue = columnValue.replace("{", "");
								columnValue = columnValue.replace("}", "");
								columnValues = Arrays.asList(columnValue.split("\\|"));
								b.addValue(rowIndex, columnValues, valueType);
								list = true;
							}

							if (!list) {
								// add the new value
								b.addValue(rowIndex, columnValue, valueType);
							}
						}
						columnIndex++;
					}
				}
			});

			// while ((fileLine = in.readLine()) != null) {
			//
			// // handle the column splitting
			// fileLine = fileLine.substring(1, fileLine.length() - 1);
			// nextLine = fileLine.split(Variables.delimiter);
			//
			// // report progress
			// if (rowIndex % 1000 == 0) {
			// System.out.println(rowIndex + " rows were read in " +
			// DurationFormatUtils.formatDuration(System.currentTimeMillis() -
			// start, "HH:mm:ss.S"));
			// }
			//
			// // iterate all columns
			// int columnIndex = 0;
			// for (String columnValue : nextLine) {
			//
			// TableColumnBuilder b = colBuilders.get(columnIndex);
			// ColumnType valueType = new ColumnType(b.getDataType(), null);
			//
			// if (!columnValue.equalsIgnoreCase(Variables.nullValue)) {
			//
			// // normalise the value
			//// if (Variables.normalizeValues) {
			//// String columnValueNormalized =
			// StringNormalizer.clearString(columnValue, true);
			//// columnValueNormalized =
			// StringNormalizer.simpleStringNormalization(columnValueNormalized,
			// true);
			//// columnValue = columnValueNormalized;
			//// }
			//
			// // handle lists
			//// if (checkIfList(columnValue)) {
			//// // lists are currently not handeled, so just take the first
			// value
			//// //columnValue = columnValue.split("\\|")[0];
			//// //columnValue = columnValue.replace("{", "");
			//// table.getColumns().get(columnIndex).setDataType(ColumnDataType.list);
			//// }
			//
			// // guess the type if not already set
			// boolean list = false;
			// if (table.getColumns().get(columnIndex).getDataType() ==
			// ColumnDataType.unknown) {
			//
			// // detect the unit and guess the type
			// //TODO: use the determined datatype? e.g. for populationDensity?
			// valueType = typeGuesser.guessTypeForValue(columnValue,
			// b.getHeader());
			//
			// }
			// if (checkIfList(columnValue)) {
			// List<String> columnValues;
			// columnValue = columnValue.replace("{", "");
			// columnValue = columnValue.replace("}", "");
			// columnValues = Arrays.asList(columnValue.split("\\|"));
			// b.addValue(rowIndex, columnValues, valueType);
			// list = true;
			// }
			//
			// if (!list) {
			// // add the new value
			// b.addValue(rowIndex, columnValue, valueType);
			// }
			// }
			// columnIndex++;
			// }
			// rowIndex++;
			// }

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		// finalise the column building
		for (TableColumnBuilder b : colBuilders) {
			b.buildColumn();
		}

		table.getColumns().get(metaData.getKeyColumn()).setKey(true);

		return table;
	}

	public Table readTable(TableType tableType, AnnotatedTableId id, AnnotatedTableMetaData metaData, List<String[]> rows) {
		switch (tableType) {
		case LOD_TABLE:
			return readLODTable(id, metaData, rows);
		case WEB_TABLE:
			return readWebTable(id, metaData, rows);
		case JSON_TABLE:
			throw new IllegalStateException("THIS FACTORY IS ONLY FOR READING CSV TABLES!");

		}

		throw new IllegalStateException("Could not find TableReader for given type. ");
	}

	public enum TableType {
		WEB_TABLE, LOD_TABLE, JSON_TABLE;
	}
}
