package de.mannheim.uni.expansion.data.tables.reader;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.atomic.AtomicInteger;

import javax.inject.Inject;

import com.google.gson.Gson;

import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.data.tables.model.DomainSource;
import de.mannheim.uni.expansion.data.tables.model.InstanceIndexComposite;
import de.mannheim.uni.expansion.data.tables.model.PropertyIndexComposite;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.timestamps.HeidelTimeWrapper;
import de.mannheim.uni.tableprocessor.IO.JsonTableParser;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.json.AnnotatedTable;
import de.mannheim.uni.utils.data.Pair;

public class AnnotatedTableParserJSON implements AnnotatedTableParser {

	private final JsonTableParser jsonTableParser = new JsonTableParser();
	private final Gson gson = new Gson();
	private final HeidelTimeWrapper heidelTimeWrapper;

	// FIXME: Convert to nonblocking using threadlocal
	private final AtomicInteger ai = new AtomicInteger(0);

	@Inject
	public AnnotatedTableParserJSON(HeidelTimeWrapper heidelTimeWrapper) {
		this.heidelTimeWrapper = heidelTimeWrapper;
	}

	@Override
	public AnnotatedTablesReaderData get(byte[] content, String name, FileEnding fileEnding, TableType tableType) throws IOException, InvalidTableException {
		InputStreamReader reader = new InputStreamReader(new ByteArrayInputStream(content));
		try {
			AnnotatedTable data = gson.fromJson(reader, AnnotatedTable.class);

			AnnotatedTableId id = new AnnotatedTableId(name.substring(0, name.length() - 5));

			Table table = jsonTableParser.parseJson(data.getTable(), name);
			if (table == null) {
				return null;
			}

			table.setFullPath(name);
			AnnotatedTableMetaData md = getMetaData(data);

			return new AnnotatedTablesReaderData(id, md, table);
		} catch (NullPointerException e) {
			e.printStackTrace();
			throw e;
		} finally {
			reader.close();
		}

	}

	private AnnotatedTableMetaData getMetaData(AnnotatedTable data) {

		DomainSource source = new DomainSource(data.getTable().getUrl());
		ClassId classId = new ClassId(data.getMapping().getMappedClass().getFirst());
		double classConf = data.getMapping().getMappedClass().getSecond();
		int numHeaderRows = data.getTable().getNumberOfHeaderRows();
		Map<Integer, List<PropertyId>> properties = new HashMap<>();
		Map<PropertyIndexComposite, Double> propertiesScores = new HashMap<>();

		for (Entry<Integer, Pair<String, Double>> e : data.getMapping().getMappedProperties().entrySet()) {
			PropertyId propertyId = new PropertyId(e.getValue().getFirst());
			List<PropertyId> mappingList = CHMHelper.getFromMap(properties, e.getKey(), () -> new ArrayList<>());
			mappingList.add(propertyId);

			propertiesScores.put(new PropertyIndexComposite(e.getKey(), propertyId), e.getValue().getSecond());
		}

		Map<Integer, List<InstanceId>> instances = new HashMap<>();
		Map<InstanceIndexComposite, Double> instancesScores = new HashMap<>();

		for (Entry<Integer, Pair<String, Double>> e : data.getMapping().getMappedInstances().entrySet()) {
			InstanceId instanceId = new InstanceId(e.getValue().getFirst());
			List<InstanceId> mappingList = CHMHelper.getFromMap(instances, e.getKey(), () -> new ArrayList<>());
			mappingList.add(instanceId);

			instancesScores.put(new InstanceIndexComposite(e.getKey(), instanceId), e.getValue().getSecond());
		}

		int keyColumn = data.getTable().getKeyColumnIndex();

		List<Integer> beforeTextYears = null;
		List<Integer> afterTextYears = null;
		List<Integer> contextBeforeYears = null;
		List<Integer> tableTitleYears = null;
		List<Integer> pageTitleYears = null;

		Map<Integer, List<Integer>> propertiesYears = new HashMap<>();

		if (heidelTimeWrapper != null) {
			beforeTextYears = heidelTimeWrapper.getYears(data.getTable().getTextBeforeTable());
			afterTextYears = heidelTimeWrapper.getYears(data.getTable().getTextAfterTable());
			contextBeforeYears = heidelTimeWrapper.getYears(data.getTable().getTableContextTimeStampBeforeTable());
			tableTitleYears = heidelTimeWrapper.getYears(data.getTable().getTitle());
			pageTitleYears = heidelTimeWrapper.getYears(data.getTable().getPageTitle());

			String[] columHeaders = data.getTable().getColumnHeaders();
			for (int i = 0; i < columHeaders.length; i++) {
				List<Integer> columnYear = heidelTimeWrapper.getYears(columHeaders[i]);
				if (columnYear != null) {
					propertiesYears.put(i, columnYear);
				}
			}
		}
		return new AnnotatedTableMetaData(source, classId, classConf, numHeaderRows, properties, propertiesScores, instances, instancesScores, keyColumn,
				beforeTextYears, afterTextYears, contextBeforeYears, tableTitleYears, pageTitleYears, propertiesYears, data.getTable().getPageTitle(),
				data.getTable().getTitle(), data.getTable().getTableContextTimeStampBeforeTable(), data.getTable().getTextBeforeTable(),
				data.getTable().getTextAfterTable());

	}

	@Override
	public AnnotatedTableMetaData getMetaData(byte[] content, String name, FileEnding fileEnding) throws IOException, InvalidTableException {
		InputStreamReader reader = new InputStreamReader(new ByteArrayInputStream(content));
		try {
			AnnotatedTable data = gson.fromJson(reader, AnnotatedTable.class);
			return getMetaData(data);
		} finally {
			reader.close();
		}
	}

	@Override
	public int getColumnCount() {
		return this.ai.get();
	}

}
