package de.mannheim.uni.expansion.data.tables.reader;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Consumer;

import javax.inject.Named;

import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;
import org.objenesis.strategy.StdInstantiatorStrategy;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryo.Kryo.DefaultInstantiatorStrategy;
import com.esotericsoftware.kryo.io.Input;
import com.esotericsoftware.kryo.serializers.CompatibleFieldSerializer;
import com.esotericsoftware.kryo.util.MapReferenceResolver;

import de.mannheim.uni.expansion.data.DataNamedProperties;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.utils.BatchConsumerProducer;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.expansion.utils.TableCycler;

public class AnnotatedTablesKryoReader implements AnnotatedTablesReader {

	private final String folder;

	private ThreadLocal<Kryo> kryoLocal;

	private final PersistentParallel3<byte[]> kryoPersistentP;

	private final PersistentParallel3<AnnotatedTablesReaderData> matchingPersistentP = new PersistentParallel3<>("RUN COLUMN MATCHING", 40);

	private final boolean references;

	public AnnotatedTablesKryoReader(@Named(DataNamedProperties.TABLES_OUTPUT_FOLDERS) String folder, boolean references) {
		this.folder = folder;

		kryoLocal = new ThreadLocal<Kryo>();

		this.references = references;

		if (references) {
			kryoPersistentP = new PersistentParallel3<>("READ GZ TABLE ARCHIVE", 400);
		} else {
			kryoPersistentP = new PersistentParallel3<>("READ GZ TABLE ARCHIVE", 400);
		}

	}

	public Kryo getKryo() {
		Kryo kryo = kryoLocal.get();
		if (kryo == null) {
			kryo = new Kryo(new FixedClassResolver(), new MapReferenceResolver());
			kryo.setInstantiatorStrategy(new DefaultInstantiatorStrategy(new StdInstantiatorStrategy()));
			kryo.setReferences(references);
			kryo.setDefaultSerializer(CompatibleFieldSerializer.class);
			kryoLocal.set(kryo);
		}
		return kryo;
	};

	public String getFolder() {
		return folder;
	}

	private List<byte[]> readGzFile(File file) throws IOException {
		List<byte[]> contents = new ArrayList<>();

		TarArchiveInputStream tarStream = new TarArchiveInputStream(new GzipCompressorInputStream(new FileInputStream(file)));

		TarArchiveEntry entry;

		while ((entry = tarStream.getNextTarEntry()) != null) {

			byte[] content = new byte[(int) entry.getSize()];

			tarStream.read(content, 0, content.length);

			contents.add(content);
			// String name = entry.getName().split("/")[1];

		}

		tarStream.close();

		return contents;
	}

	@Override
	public void runForEachTable(Consumer<AnnotatedTablesReaderData> fullConsumer) {
		runForEachTable(fullConsumer, 40);
	}

	public void runForEachTable(Consumer<AnnotatedTablesReaderData> fullConsumer, int batchSize, boolean reportProgress) {

		final BatchConsumerProducer<byte[]> batchConsumerProducer;
		batchConsumerProducer = new BatchConsumerProducer<>("READ KRYO", (content) -> {
			Input input = new Input(new ByteArrayInputStream(content));
			AnnotatedTablesReaderData data = getKryo().readObject(input, AnnotatedTablesReaderData.class);
			data = new AnnotatedTablesReaderData(data.getId(), data.getMetaData(), TableCycler.addCyclicReferences(data.getTableData()));
			input.close();

			if (fullConsumer != null) {
				fullConsumer.accept(data);
			}

		} , 2000, 2);

		PersistentParallel3<File> reader = new PersistentParallel3<>("READ KRYO GZ FILE", 1, 2, false);

		File[] files = new File(folder).listFiles();

		try {
			AtomicInteger ai = new AtomicInteger(0);
			reader.foreach(Arrays.asList(files), (file) -> {
				System.out.println("Now reading table " + ai.incrementAndGet() + " out of " + files.length);

				try {

					batchConsumerProducer.addBatch(readGzFile(file));
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			});

			batchConsumerProducer.finish();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// try {
		// PersistentParallel.forLoop(0, files.length, new Consumer<Integer>() {
		//
		// @Override
		// public void accept(Integer i) {
		//
		// try {
		//
		// TarArchiveInputStream tarStream = new TarArchiveInputStream(new
		// GzipCompressorInputStream(new FileInputStream(files[i])));
		//
		// TarArchiveEntry entry = tarStream.getNextTarEntry();
		//
		// while ((entry = tarStream.getNextTarEntry()) != null) {
		//
		// byte[] content = new byte[(int) entry.getSize()];
		//
		// tarStream.read(content, 0, content.length);
		//
		// // String name = entry.getName().split("/")[1];
		//
		// Input input = new Input(new ByteArrayInputStream(content));
		// AnnotatedTablesReaderData data = getKryo().readObject(input,
		// AnnotatedTablesReaderData.class);
		// input.close();
		//
		// // Table td = magnitudeFixer.get(data.getMetaData(),
		// // data.getTableData());
		//
		// data = new AnnotatedTablesReaderData(data.getId(),
		// data.getMetaData(), data.getTableData());
		// if (fullConsumer != null) {
		// fullConsumer.accept(data);
		// }
		//
		// }
		//
		// tarStream.close();
		// } catch (IOException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		// }

		// }, "READ TABLES KRYO");
		// } catch (Exception e) {
		// e.printStackTrace();
		// e.getCause().printStackTrace();
		// }

	}

	public void runForEachTable(Consumer<AnnotatedTablesReaderData> fullConsumer, int batchSize) {
		runForEachTable(fullConsumer, batchSize, false);

	}
}
