package de.mannheim.uni.expansion.data.tables.reader;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;

import javax.inject.Inject;
import javax.inject.Named;

import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryo.io.Output;
import com.esotericsoftware.kryo.serializers.CompatibleFieldSerializer;

import de.mannheim.uni.expansion.data.DataNamedProperties;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.utils.ConsumerHelper;
import de.mannheim.uni.expansion.utils.TableCycler;

public class AnnotatedTablesWriter {

	private int fileCount = 0;
	private int entryCount = 0;
	private int groupNum = -1;
	private int perGroup = 20000;
	private TarArchiveOutputStream tarFile;
	private String folder;
	private ThreadLocal<Kryo> kryoLocal;
	private ConsumerHelper<AnnotatedTablesReaderData> consumerHelper;

	@Inject
	public AnnotatedTablesWriter(@Named(DataNamedProperties.TABLES_OUTPUT_FOLDERS) String folder) {
		try {
			this.folder = folder;
			tarFile = new TarArchiveOutputStream(new GZIPOutputStream(new FileOutputStream(folder + "/tables" + fileCount + ".tar.gz")));

			kryoLocal = new ThreadLocal<Kryo>();

			consumerHelper = ConsumerHelper.withLinkedQueue("TABLE STORER", 1, (data) -> {
				Kryo kryo = kryoLocal.get();
				if (kryo == null) {
					kryo = new Kryo();
					kryo.setDefaultSerializer(CompatibleFieldSerializer.class);
					kryoLocal.set(kryo);
				}
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				Output output = new Output(baos);
				kryo.writeObject(output,
						new AnnotatedTablesReaderData(data.getId(), data.getMetaData(), TableCycler.removeCyclicReferences(data.getTableData())));
				output.close();
				try {
					writeEntry(baos.toByteArray());
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} , new AnnotatedTablesReaderData(null, null, null));

			consumerHelper.start();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}

	}

	public synchronized void setGroupNum(int groupNum, int numberOfFiles) {
		this.groupNum = groupNum;
		this.perGroup = numberOfFiles / groupNum;
	}

	public synchronized void writeEntry(byte[] bytes) throws IOException {

		if (entryCount == perGroup && fileCount != (groupNum - 1)) {
			finishAndClose();
			fileCount++;
			tarFile = new TarArchiveOutputStream(new GZIPOutputStream(new FileOutputStream(folder + "/tables" + fileCount + ".tar.gz")));
			entryCount = 0;
		}

		TarArchiveEntry entry = new TarArchiveEntry(String.valueOf(entryCount));
		entry.setSize(bytes.length);

		tarFile.putArchiveEntry(entry);

		tarFile.write(bytes);

		tarFile.closeArchiveEntry();

		entryCount++;
	}

	public synchronized void finishAndClose() throws IOException {

		tarFile.finish();

		tarFile.close();
	}

	public void add(AnnotatedTablesReaderData data) {
		consumerHelper.add(data);
	}

	public void finishCloseAndAwait() throws InterruptedException, IOException {
		consumerHelper.finishCloseAndAwait();
		finishAndClose();

	}

	public void writeAll(AnnotatedTablesRepository annotatedTablesRepository) {

		this.setGroupNum(32, annotatedTablesRepository.getAllTableIds().size());

		for (AnnotatedTableId id : annotatedTablesRepository.getAllTableIds()) {
			AnnotatedTablesReaderData data = new AnnotatedTablesReaderData(id, annotatedTablesRepository.getMetaData(id),
					annotatedTablesRepository.getData(id));
			add(data);
		}

		try {
			finishCloseAndAwait();
		} catch (InterruptedException | IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
