package de.mannheim.uni.expansion.tablefusion;

import java.io.IOException;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.List;
import java.util.Map;

import javax.inject.Named;
import javax.inject.Singleton;

import dagger.Module;
import dagger.Provides;
import de.mannheim.uni.expansion.data.DataNamedProperties;
import de.mannheim.uni.expansion.data.dbpedia.adapter.csv.DBPediaRepositoryCSVReader;
import de.mannheim.uni.expansion.data.dbpedia.adapter.csv.DBpediaRepositoryCSVAdapter;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNew;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterfaceImpl;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterfaceRemote;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTablesReaderData;
import de.mannheim.uni.expansion.data.tables.model.CellIdx;
import de.mannheim.uni.expansion.utils.CSVSerializer;
import de.mannheim.uni.expansion.utils.StringSerializer;

@Module(library = true)
public class DBpediaModule {

	private final String dbpediaRepository;
	private final FileEnding dbpediaFileEnding;
	private RepositoryLoading repositoryLoading;

	private final List<AnnotatedTablesReaderData> table;

	private final DBpediaMode mode;
	private final List<Map<CellIdx, InstanceId>> columnMatches;

	public static enum DBpediaMode {

		Repository, Table;
	}
	

	public void setRepositoryLoading(RepositoryLoading repositoryLoading) {
		this.repositoryLoading = repositoryLoading;
	}

	public DBpediaModule(String dbpediaRepository, FileEnding dbpediaFileEnding, RepositoryLoading repositoryLoading) {
		super();
		this.dbpediaRepository = dbpediaRepository;
		this.dbpediaFileEnding = dbpediaFileEnding;
		this.repositoryLoading = repositoryLoading;
		this.table = null;
		this.columnMatches = null;
		this.mode = DBpediaMode.Repository;
	}

	public DBpediaModule(List<AnnotatedTablesReaderData> tables, List<Map<CellIdx, InstanceId>> columnMatchesList, String dbpediaRepository,
			FileEnding dbpediaFileEnding,
			RepositoryLoading repositoryLoading) {
		super();
		this.dbpediaRepository = dbpediaRepository;
		this.dbpediaFileEnding = dbpediaFileEnding;
		this.repositoryLoading = repositoryLoading;
		this.table = tables;
		this.columnMatches = columnMatchesList;
		this.mode = DBpediaMode.Table;
	}

	@Provides
	@Singleton
	public DBpediaRepositoryNewInterface getDBpedia(DBpediaRepositoryCSVAdapter csvAdapter, StringSerializer stringSerializer, CSVSerializer csvSerializer) {

		switch (mode) {
		case Repository:
			return getDbPediaRepository(csvAdapter);
		case Table:
			return getDbPediaFromTable(table, columnMatches, stringSerializer, csvSerializer);

		}
		return null;

	}

	private DBpediaRepositoryNewInterface getDbPediaFromTable(List<AnnotatedTablesReaderData> table2, List<Map<CellIdx, InstanceId>> columnMatches2,
			StringSerializer stringSerializer,
			CSVSerializer csvSerializer) {
		DBPediaRepositoryCSVReader reader = new DBPediaRepositoryCSVReader(stringSerializer, csvSerializer);

		try {
			return new DBpediaRepositoryNew(table2, columnMatches2, reader.getColumnDataTypes(dbpediaRepository));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	private DBpediaRepositoryNewInterface getDbPediaRepository(DBpediaRepositoryCSVAdapter csvAdapter) {

		try {

			if (repositoryLoading == RepositoryLoading.LOCAL_ONLY || repositoryLoading == RepositoryLoading.REMOTE_LOAD_LOCALLY) {
				DBpediaRepositoryNew dBpediaRepositoryNew = csvAdapter.read(dbpediaRepository, true);

				if (repositoryLoading == RepositoryLoading.REMOTE_LOAD_LOCALLY) {
					DBpediaRepositoryNewInterfaceRemote stub = (DBpediaRepositoryNewInterfaceRemote) UnicastRemoteObject.exportObject(dBpediaRepositoryNew, 0);

					// Bind the remote object's stub in the registry
					Registry registry = RemoteRegistry.getPublishingRegistry();
					registry.bind("DBpediaRepositoryNewInterfaceRemote", stub);
				}
				return dBpediaRepositoryNew;
			} else if (repositoryLoading == RepositoryLoading.REMOTE_LOAD_REMOTELY) {
				try {
					Registry registry = RemoteRegistry.getReadingRegistry();
					DBpediaRepositoryNewInterfaceRemote stub = (DBpediaRepositoryNewInterfaceRemote) registry.lookup("DBpediaRepositoryNewInterfaceRemote");
					return new DBpediaRepositoryNewInterfaceImpl(stub);
				} catch (Exception e) {
					System.err.println("Client exception: " + e.toString());
					e.printStackTrace();
					throw new RuntimeException(e);
				}
			} else {
				throw new IllegalStateException("illegal value for repository loading");
			}

		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	@Provides
	@Named(DataNamedProperties.DBPEDIA_REPOSITORY_PATH)
	String getDBPediaRepositoryPath() {
		return this.dbpediaRepository;
	}
}
