package de.mannheim.uni.expansion.tablefusion;

import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import javax.inject.Named;
import javax.inject.Singleton;

import dagger.Module;
import dagger.Provides;
import de.mannheim.uni.expansion.data.DataNamedProperties;
import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId.FileEnding;
import de.mannheim.uni.expansion.data.tables.model.TableFactory.TableType;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParser;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParserCSV;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTableParserJSON;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesGzReader;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesKryoReader;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesReader;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepositoryImpl;
import de.mannheim.uni.expansion.data.tables.repository.MagnitudeFixer;
import de.mannheim.uni.expansion.data.tables.repository.memory.AnnotatedTablesRepositoryImplMemoryFactory;
import de.mannheim.uni.expansion.data.tables.repository.remote.AnnotatedTablesRepositoryRemote;
import de.mannheim.uni.tableprocessor.ColumnTypeGuesser;
import de.mannheim.uni.tableprocessor.IO.ConvertFileToTable;

@Module(includes = { DBpediaModule.class, OntologyModule.class, HeidelTimeModule.class, SimilarityModule.class }, library = true)
public class TablesModule {

	private final String mappingFolder;
	private final String tablesOutputFolders;
	private RepositoryLoading repositoryLoading;
	private boolean loadFromCache;
	private final TableType tableType;
	private final FileEnding tablesFileEnding;
	private final String badAnnotatedTablesFile;
	private boolean filter;
	private DBpediaRepositoryNewInterface classFilter;

	public TablesModule(String mappingFolder, String tablesOutputFolders, RepositoryLoading repositoryLoading, boolean loadFromCache, TableType tableType,
			FileEnding tablesFileEnding, String badAnnotatedTablesFile, boolean filter) {
		super();
		this.mappingFolder = mappingFolder;
		this.tablesOutputFolders = tablesOutputFolders;
		this.repositoryLoading = repositoryLoading;
		this.loadFromCache = loadFromCache;
		this.tableType = tableType;
		this.tablesFileEnding = tablesFileEnding;
		this.badAnnotatedTablesFile = badAnnotatedTablesFile;
		this.filter = filter;
	}

	public void setRepositoryLoading(RepositoryLoading repositoryLoading) {
		this.repositoryLoading = repositoryLoading;
	}

	public void setLoadFromCache(boolean loadFromCache) {
		this.loadFromCache = loadFromCache;
	}

	public void setFilter(boolean filter) {
		this.filter = filter;
	}

	public void setClassFilter(DBpediaRepositoryNewInterface dbpedia) {
		this.classFilter = dbpedia;
	}

	@Provides
	@Singleton
	public AnnotatedTablesRepository loadAnnotatedTables(AnnotatedTablesReader annotatedTablesReader, MagnitudeFixer magnitudeFixer) {

		try {
		// AnnotatedTablesRepositoryImplMemoryFactory factory = new
		// AnnotatedTablesRepositoryImplMemoryFactory(reader);
		// return factory.getRepository();
		if (repositoryLoading == RepositoryLoading.LOCAL_ONLY || repositoryLoading == RepositoryLoading.REMOTE_LOAD_LOCALLY) {

				AnnotatedTablesRepositoryImplMemoryFactory repositoryFactory = new AnnotatedTablesRepositoryImplMemoryFactory(annotatedTablesReader, filter,
						classFilter, magnitudeFixer);
				AnnotatedTablesRepository annotatedTablesRepository = repositoryFactory.getRepository();

				if (repositoryLoading == RepositoryLoading.REMOTE_LOAD_LOCALLY) {
					Registry registry = RemoteRegistry.getPublishingRegistry();

					AnnotatedTablesRepositoryRemote annotatedTablesRepositoryRemote = (AnnotatedTablesRepositoryRemote) annotatedTablesRepository;
					AnnotatedTablesRepositoryRemote stubAnno = (AnnotatedTablesRepositoryRemote) UnicastRemoteObject
							.exportObject(annotatedTablesRepositoryRemote, 0);
					registry.bind("AnnotatedTablesRepositoryRemote", stubAnno);

				}

				return annotatedTablesRepository;
		} else {
			try {
					Registry registry = RemoteRegistry.getReadingRegistry();
				AnnotatedTablesRepositoryRemote stub = (AnnotatedTablesRepositoryRemote) registry.lookup("AnnotatedTablesRepositoryRemote");
				return new AnnotatedTablesRepositoryImpl(stub);
			} catch (Exception e) {
				System.err.println("Client exception: " + e.toString());
				e.printStackTrace();
					throw new RuntimeException(e);
			}

		}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}

	}


	@Provides
	@Singleton
	public AnnotatedTablesReader getReader(AnnotatedTablesKryoReader kryoReader, AnnotatedTablesGzReader gzReader) {
		if (loadFromCache) {
			return kryoReader;
		} else {
			return gzReader;
		}
	}

	@Provides
	@Singleton
	AnnotatedTablesKryoReader getKryoReader() {
		return new AnnotatedTablesKryoReader(tablesOutputFolders, true);
	}

	@Provides
	@Singleton
	AnnotatedTablesGzReader getGZReader(AnnotatedTableParser parser) {
		return new AnnotatedTablesGzReader(mappingFolder, tablesFileEnding, tableType, parser, badAnnotatedTablesFile);
	}

	@Provides
	@Named(DataNamedProperties.TABLES_MAPPING_FOLDER)
	String getTablesMappingFolder() {
		return this.mappingFolder;
	}

	@Provides
	@Named(DataNamedProperties.TABLES_OUTPUT_FOLDERS)
	String getTablesOutputFolders() {
		return this.tablesOutputFolders;
	}

	@Provides
	AnnotatedTableParser getParser(AnnotatedTableParserCSV annotatedTableParserCSV, AnnotatedTableParserJSON annotatedTableParserJSON) {

		switch (this.tableType) {
		case JSON_TABLE:
			return annotatedTableParserJSON;
		case LOD_TABLE:
			return annotatedTableParserCSV;
		case WEB_TABLE:
			return annotatedTableParserCSV;

		}
		throw new IllegalStateException("YOU CAN NOT INJECT A PARSER WITHOUT A TABLETYPE");
	}

	@Provides
	ConvertFileToTable getConvertFileToToable() {
		return new ConvertFileToTable();
	}

	@Provides
	ColumnTypeGuesser getColumnTypeGuesser() {
		return new ColumnTypeGuesser();
	}

	@Provides
	@Named(DataNamedProperties.BAD_ANNOTATED_TABLES_FILE)
	String getBadAnnotatedTablesFile() {
		return this.badAnnotatedTablesFile;
	}

	@Provides
	@Named(DataNamedProperties.TABLES_MAPPING_ENDING)
	FileEnding getTabledFileEnding() {
		return this.tablesFileEnding;
	}

	@Provides
	@Named(DataNamedProperties.TABLES_TYPE)
	TableType getTableType() {
		return this.tableType;
	}
}
