package de.mannheim.uni.expansion.tablefusion.core.fusing.impl.generic;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;

public class GenericMedianFuser<T> implements BaseFuser {

	private final TypeChecker<T> typeChecker;
	private final TypeCaster<T> typeCaster;
	private final ListSorter<T> listSorter;
	private final Averager<T> averager;
	private final Comparer<T> comparer;

	public GenericMedianFuser(TypeChecker<T> typeChecker, TypeCaster<T> typeCaster, ListSorter<T> listSorter, Averager<T> averager, Comparer<T> comparer) {
		this.typeCaster = typeCaster;
		this.typeChecker = typeChecker;
		this.listSorter = listSorter;
		this.averager = averager;
		this.comparer = comparer;
	}

	@FunctionalInterface
	public interface TypeChecker<T> {
		boolean check(Object o);
	}

	@FunctionalInterface
	public interface TypeCaster<T> {
		T cast(Object o);
	}

	@FunctionalInterface
	public interface ListSorter<T> {
		void sort(List<T> o);
	}

	@FunctionalInterface
	public interface Averager<T> {
		T get(T object1, double score1, T object2, double score2);
	}

	@FunctionalInterface
	public interface Comparer<T> {
		int compare(T object, T other);
	}

	public FusedValue simpleFuse(List<Object> list) {
		List<T> values = new ArrayList<T>();

		for (Object o : list) {

			if (typeChecker.check(o)) {
				values.add(typeCaster.cast(o));
			}

		}

		listSorter.sort(values);

		if (values.size() > 0) {
			if (values.size() % 2 != 0) {
				int idx = (values.size() / 2);

				return new FusedValue(values.get(idx), (double) values.size());

			} else {
				int idx1 = (values.size() / 2);
				int idx2 = (values.size() / 2) - 1;

				return new FusedValue(averager.get(values.get(idx1), 0.5, values.get(idx2), 0.5), (double) values.size());
			}
		} else {
			return null;
		}
	}

	public FusedValue fuse(List<? extends ValueProbabilityPair> values, Double threshold) {
		if (values.size() == 0) {
			return null;
		}
		double sum = 0.0;
		List<Pair> pairs = new ArrayList<Pair>();

		for (ValueProbabilityPair pair : values) {

			Double probability = pair.getScore();

			Object o = pair.getValue();

			if (typeChecker.check(o)) {
				pairs.add(new Pair(typeCaster.cast(o), probability));
				sum += Math.abs(probability);
			}

		}

		if (threshold == null || sum > threshold) {

			Collections.sort(pairs);

			double median = sum / 2.0;

			double current = 0.0;

			T value = null;

			Pair lastPair = null;
			for (Pair p : pairs) {
				current += Math.abs(p.getScore());

				if (median > current) {
					continue;
				} else {
					if (lastPair == null) {
						value = p.getValue();
						break;
					} else {
						// double diff = p.getScore();
						double s1 = (current) - median;
						double s2 = median - (current - Math.abs(p.getScore()));

						value = averager.get(p.getValue(), s1, lastPair.getValue(), s2);
					}
				}

				lastPair = p;
			}

			if (value == null)
				return null;
			return new FusedValue(value, sum);

		}
		return null;
	}

	private class Pair implements Comparable<Pair> {
		private final T value;
		private final Double score;

		public T getValue() {
			return value;
		}

		public Double getScore() {
			return score;
		}

		public Pair(T value, Double score) {
			super();
			this.value = value;
			this.score = score;
		}

		@Override
		public int compareTo(Pair other) {
			return comparer.compare(this.value, other.value);
		}

	}

	@Override
	public boolean checkType(Object value) {
		return typeChecker.check(value);
	}

}
