package de.mannheim.uni.expansion.tablefusion.analysis;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.FusedResultsSingle;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.FusionPerformance;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.PRFMPerformance;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.PerformancePR;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.PrecisionPerformance;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.expansion.utils.ValueProbabilityPair;
import de.mannheim.uni.expansion.utils.containers.Pair;
import de.mannheim.uni.expansion.utils.containers.Quad;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class AccuracyCalculator {

	private final DBpediaRepositoryNewInterface dbPediaRepository;

	private final OntologyRepository ontologyRepository;

	private final SimilarityCalculator similarityCalculator;

	private final PersistentParallel3<Entry<TripleId, FusedValue>> parallel;

	private final MaxCorrectFinder maxCorrectFinder;

	private final ValueEvaluator valueEvaluator;

	@Inject
	public AccuracyCalculator(DBpediaRepositoryNewInterface dbPediaRepository, SimilarityCalculator similarityCalculator, OntologyRepository ontologyRepository,
			MaxCorrectFinder maxCorrectFinder, ValueEvaluator valueEvaluator) {
		this.dbPediaRepository = dbPediaRepository;
		this.similarityCalculator = similarityCalculator;
		this.parallel = new PersistentParallel3<>("AccuracyCalculator");
		this.ontologyRepository = ontologyRepository;
		this.maxCorrectFinder = maxCorrectFinder;
		this.valueEvaluator = valueEvaluator;
	}

	private static final Creator<FusedResultsSingle> fusedResultsCreator = () ->

	{
		return new FusedResultsSingle();
	};

	public FusionPerformance<PrecisionPerformance> analyzeAccuracy(Map<TripleId, FusedValue> fusedValues) {
		Quad<FusedResultsSingle, Map<ColumnDataType, FusedResultsSingle>, Map<ClassId, FusedResultsSingle>, Map<ClassPropertyId, FusedResultsSingle>> fusedResults = analyzeFusedValues(
				fusedValues);
		final FusedResultsSingle fr = fusedResults.getA();
		Map<ColumnDataType, FusedResultsSingle> perDataTypeMap = fusedResults.getB();

		Map<ClassId, FusedResultsSingle> perClassIdMap = fusedResults.getC();

		Map<ClassPropertyId, FusedResultsSingle> perClassPropertyMap = fusedResults.getD();

		PrecisionPerformance totalPerformance = getPerformance(fr);
		Map<ColumnDataType, PrecisionPerformance> perDataTypePerformance = perDataTypeMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
			return getPerformance(e.getValue());
		}));

		Map<ClassId, PrecisionPerformance> perClassIdPerformance = perClassIdMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
			return getPerformance(e.getValue());
		}));

		Map<ClassPropertyId, PrecisionPerformance> perClassPropertyIdPerformance = perClassPropertyMap.entrySet().stream()
				.collect(Collectors.toMap(Entry::getKey, (e) -> {
					return getPerformance(e.getValue());
				}));

		return new FusionPerformance<PrecisionPerformance>(totalPerformance, perDataTypePerformance, perClassIdPerformance, perClassPropertyIdPerformance);

	}

	public FusionPerformance<PerformancePR> analyzeAccuracy(GroupedValues groupedValues, Map<TripleId, FusedValue> fusedValues, boolean fuseLists) {
		return analyzeAccuracy(groupedValues, fusedValues, fuseLists, null);
	}

	public FusionPerformance<PerformancePR> analyzeAccuracy(GroupedValues groupedValues, Map<TripleId, FusedValue> fusedValues, boolean fuseLists,
			FilteredValues filteredValues) {

		Quad<FusedResultsSingle, Map<ColumnDataType, FusedResultsSingle>, Map<ClassId, FusedResultsSingle>, Map<ClassPropertyId, FusedResultsSingle>> fusedResults = analyzeFusedValues(
				fusedValues, filteredValues);
		final FusedResultsSingle fr = fusedResults.getA();
		Map<ColumnDataType, FusedResultsSingle> perDataTypeMap = fusedResults.getB();

		Map<ClassId, FusedResultsSingle> perClassIdMap = fusedResults.getC();

		Map<ClassPropertyId, FusedResultsSingle> perClassPropertyMap = fusedResults.getD();

		int max = maxCorrectFinder.getMaxCorrectValues(groupedValues, fuseLists, filteredValues);
		Map<ColumnDataType, Integer> maxPerDataType = maxCorrectFinder.getMaxCorrectValuePerDataType(groupedValues, fuseLists, filteredValues);
		Map<ClassId, Integer> maxPerClassId = maxCorrectFinder.getMaxCorrectValuePerClass(groupedValues, fuseLists, filteredValues);
		Map<ClassPropertyId, Integer> maxPerClassPropertyId = maxCorrectFinder.getMaxCorrectValuePerClassPropertyId(groupedValues, fuseLists, filteredValues);

		PerformancePR totalPerformance = getPerformance(fr, max);
		Map<ColumnDataType, PerformancePR> perDataTypePerformance = perDataTypeMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
			int maxValue = maxPerDataType.getOrDefault(e.getKey(), 0);
			return getPerformance(e.getValue(), maxPerDataType.getOrDefault(e.getKey(), 0));
		}));

		Map<ClassId, PerformancePR> perClassIdPerformance = perClassIdMap.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {

			return getPerformance(e.getValue(), maxPerClassId.getOrDefault(e.getKey(), 0));
		}));

		Map<ClassPropertyId, PerformancePR> perClassPropertyIdPerformance = perClassPropertyMap.entrySet().stream()
				.collect(Collectors.toMap(Entry::getKey, (e) -> {
					return getPerformance(e.getValue(), maxPerClassPropertyId.getOrDefault(e.getKey(), 0));
				}));

		return new FusionPerformance<PerformancePR>(totalPerformance, perDataTypePerformance, perClassIdPerformance, perClassPropertyIdPerformance);

		// if (print) {
		// for (SimilarityType type : SimilarityType.values()) {
		// Double accuracy = fr.getAccuracy(type);
		//
		// if (string != null) {
		// System.out.print("Results for " + string + " - " + type.toString() +
		// " - ");
		//
		// if (accuracy.equals(Double.NaN)) {
		// System.out.println("NO MATCHES");
		// } else {
		// System.out.println("Accuracy achieved: " + accuracy + " (out of " +
		// fr.getOverlappingTriples(type) + ")");
		// }
		// }
		// }
		// }

	}

	private Quad<FusedResultsSingle, Map<ColumnDataType, FusedResultsSingle>, Map<ClassId, FusedResultsSingle>, Map<ClassPropertyId, FusedResultsSingle>> analyzeFusedValues(
			Map<TripleId, FusedValue> fusedValues) {
		return analyzeFusedValues(fusedValues, null);
	}

	private Quad<FusedResultsSingle, Map<ColumnDataType, FusedResultsSingle>, Map<ClassId, FusedResultsSingle>, Map<ClassPropertyId, FusedResultsSingle>> analyzeFusedValues(
			Map<TripleId, FusedValue> fusedValues, FilteredValues filteredValues) {
		final FusedResultsSingle fr = new FusedResultsSingle();
		ConcurrentHashMap<ColumnDataType, FusedResultsSingle> perDataTypeMap = new ConcurrentHashMap<ColumnDataType, FusedResultsSingle>();

		ConcurrentHashMap<ClassId, FusedResultsSingle> perClassIdMap = new ConcurrentHashMap<ClassId, FusedResultsSingle>();

		ConcurrentHashMap<ClassPropertyId, FusedResultsSingle> perClassPropertyMap = new ConcurrentHashMap<ClassPropertyId, FusedResultsSingle>();

		try {
			parallel.foreach(fusedValues.entrySet(), new Consumer<Map.Entry<TripleId, FusedValue>>() {

				@SuppressWarnings("unchecked")
				@Override
				public void accept(Entry<TripleId, FusedValue> e) {

					ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey().getClassId(), e.getKey().getPropertyId());
					ColumnDataType cdt = ontologyRepository.getDataType(classPropertyId);

					FusedResultsSingle perDataType = CHMHelper.getFromCHM(perDataTypeMap, cdt, fusedResultsCreator);

					FusedResultsSingle perClassId = CHMHelper.getFromCHM(perClassIdMap, e.getKey().getClassId(), fusedResultsCreator);
					FusedResultsSingle perClassPropertyId = CHMHelper.getFromCHM(perClassPropertyMap, classPropertyId, fusedResultsCreator);

					boolean correctValue = false;
					FusedValue fusedValue = e.getValue();
					Object o = dbPediaRepository.getValue(e.getKey());

					if (o == null) {
						fr.incNew();
						perDataType.incNew();
						perClassId.incNew();
						perClassPropertyId.incNew();
						return;
					}
					if (cdt == null) {
						// System.out.println("A: CDT == null, please check!");
					}
					if (filteredValues != null) {
						Set<Object> lookingFor = filteredValues.getFilteredValues(e.getKey());
						if (lookingFor == null || lookingFor.size() == 0) {
							return;
						} else if (o instanceof List<?> && fusedValue.isList()) {
							List<Object> correctList = new ArrayList<>(lookingFor);

							List<Object> fusedList = fusedValue.getSimpleList();

							List<Object> valuesToEliminate = new ArrayList<>((List<Object>) o);

							FusedValue newFusedList = eliminateValuesButKeep(valuesToEliminate, fusedValue, correctList, cdt);

							if (newFusedList.listSize() == 0 || newFusedList.listSize() < lookingFor.size()) {
								return;
							} else {
								correctValue = valueEvaluator.correct(e.getKey(), o, newFusedList, cdt, filteredValues);
								int dfdf = 123;
							}

						}

					} else {
						if (o instanceof List<?> && fusedValue.isList()) {
							List<Object> correctList = (List<Object>) o;

							if (fusedValue.listSize() < correctList.size()) {
								return;
							} else if (fusedValue.listSize() == 0) {
								return;
							}
						}
						correctValue = valueEvaluator.correct(e.getKey(), o, fusedValue, cdt);
					}

					if (correctValue) {
						fr.incCorrect();
						perDataType.incCorrect();
						perClassId.incCorrect();
						perClassPropertyId.incCorrect();
					} else {
						if (fusedValue instanceof List<?>) {
							int fdfd = 123;
						}
						fr.incWrong();
						perDataType.incWrong();
						perClassId.incWrong();
						perClassPropertyId.incWrong();
					}

				}

			});
		} catch (Exception e) {
			e.printStackTrace();
		}

		return new Quad<>(fr, perDataTypeMap, perClassIdMap, perClassPropertyMap);
	}

	private PrecisionPerformance getPerformance(FusedResultsSingle results) {
		return new PrecisionPerformance(results.getCorrectTriples(), results.getWrongTriples(), results.getNewTriples());
	}

	private PerformancePR getPerformance(FusedResultsSingle results, int max) {
		return new PerformancePR(results.getCorrectTriples(), results.getWrongTriples(), max, results.getNewTriples());

	}

	public FusionPerformance<PRFMPerformance> averageAccuracies(List<FusionPerformance<PerformancePR>> fusionResults) {

		List<PerformancePR> groupedTotal = new ArrayList<>();

		Map<ColumnDataType, List<PerformancePR>> groupedPerDataType = new HashMap<>();
		Map<ClassId, List<PerformancePR>> groupedPerClassId = new HashMap<>();
		Map<ClassPropertyId, List<PerformancePR>> groupedPerClassPropertyId = new HashMap<>();

		for (FusionPerformance<PerformancePR> fp : fusionResults) {
			groupedTotal.add(fp.getTotal());

			for (Entry<ColumnDataType, PerformancePR> e : fp.getPerDataType().entrySet()) {
				List<PerformancePR> list = CHMHelper.getFromMap(groupedPerDataType, e.getKey(), () -> new ArrayList<>());
				list.add(e.getValue());
			}

			for (Entry<ClassId, PerformancePR> e : fp.getPerClassId().entrySet()) {
				List<PerformancePR> list = CHMHelper.getFromMap(groupedPerClassId, e.getKey(), () -> new ArrayList<>());
				list.add(e.getValue());
			}

			for (Entry<ClassPropertyId, PerformancePR> e : fp.getPerProperty().entrySet()) {
				List<PerformancePR> list = CHMHelper.getFromMap(groupedPerClassPropertyId, e.getKey(), () -> new ArrayList<>());
				list.add(e.getValue());
			}
		}

		Function<Entry<?, List<PerformancePR>>, PRFMPerformance> average = (entry) -> {
			return new PRFMPerformance(entry.getValue());
		};

		PRFMPerformance total = new PRFMPerformance(groupedTotal);
		Map<ColumnDataType, PRFMPerformance> perDataType = groupedPerDataType.entrySet().stream().collect(Collectors.toMap(Entry::getKey, average));

		Map<ClassId, PRFMPerformance> perClassId = groupedPerClassId.entrySet().stream().collect(Collectors.toMap(Entry::getKey, average));
		Map<ClassPropertyId, PRFMPerformance> perClassPropertyId = groupedPerClassPropertyId.entrySet().stream()
				.collect(Collectors.toMap(Entry::getKey, average));

		return new FusionPerformance<PRFMPerformance>(total, perDataType, perClassId, perClassPropertyId);
	}

	private List<Object> eliminateValues(List<Object> valuesToRemove, List<Object> fusedList, ColumnDataType cdt) {
		SimilarityMeasure measure = similarityCalculator.getMeasure(cdt);

		TreeMap<Double, Pair<Integer, Integer>> sims = new TreeMap<>();

		for (int a = 0; a < valuesToRemove.size(); a++) {
			for (int b = 0; b < fusedList.size(); b++) {
				double similarity = measure.getQuickSimilarity(valuesToRemove.get(a), fusedList.get(b));
				if (similarity > measure.getEquivalenceThreshold())
					sims.put(similarity, new Pair<>(a, b));

			}
		}

		Set<Integer> removeIndeces = new HashSet<>();
		for (int a = 0; a < valuesToRemove.size(); a++) {
			removeIndeces.add(a);
		}

		Set<Integer> fusedIndices = new HashSet<>();
		for (int b = 0; b < fusedList.size(); b++) {
			fusedIndices.add(b);
		}

		while (!removeIndeces.isEmpty() && !fusedIndices.isEmpty()) {
			Entry<Double, Pair<Integer, Integer>> e = sims.pollLastEntry();
			if (e == null) {
				break;
			}
			if (removeIndeces.contains(e.getValue().getFirst()) && fusedIndices.contains(e.getValue().getSecond())) {
				removeIndeces.remove(e.getValue().getFirst());
				fusedIndices.remove(e.getValue().getSecond());
			}
		}

		List<Object> retList = new ArrayList<>();
		for (Integer index : fusedIndices) {
			retList.add(fusedList.get(index));
		}

		return retList;
	}

	private FusedValue eliminateValuesButKeep(List<Object> valuesToRemove, FusedValue fusedValue, List<Object> keep, ColumnDataType cdt) {
		SimilarityMeasure measure = similarityCalculator.getMeasure(cdt);

		Set<Integer> keepIndices = new HashSet<>();
		List<Object> fusedList = fusedValue.getSimpleList();

		for (int k = 0; k < keep.size(); k++) {
			for (int f = 0; f < fusedList.size(); f++) {
				double similarity = measure.getQuickSimilarity(keep.get(k), fusedList.get(f));
				if (similarity > measure.getEquivalenceThreshold())
					keepIndices.add(f);

			}
		}

		TreeMap<Double, Pair<Integer, Integer>> eliminateSims = new TreeMap<>();

		for (int v = 0; v < valuesToRemove.size(); v++) {
			for (int f = 0; f < fusedList.size(); f++) {
				if (keepIndices.contains(f))
					continue;
				else {
					double similarity = measure.getQuickSimilarity(valuesToRemove.get(v), fusedList.get(f));
					if (similarity > measure.getEquivalenceThreshold())
						eliminateSims.put(similarity, new Pair<>(v, f));
				}
			}
		}

		Set<Integer> removeIndeces = new HashSet<>();
		for (int a = 0; a < valuesToRemove.size(); a++) {
			removeIndeces.add(a);
		}

		Set<Integer> fusedIndices = new HashSet<>();
		for (int b = 0; b < fusedList.size(); b++) {
			fusedIndices.add(b);
		}

		while (!removeIndeces.isEmpty() && !fusedIndices.isEmpty()) {
			Entry<Double, Pair<Integer, Integer>> e = eliminateSims.pollLastEntry();
			if (e == null) {
				break;
			}
			if (removeIndeces.contains(e.getValue().getFirst()) && fusedIndices.contains(e.getValue().getSecond())) {
				removeIndeces.remove(e.getValue().getFirst());
				fusedIndices.remove(e.getValue().getSecond());
			}
		}

		Set<Object> keepSet = new HashSet<>();
		for (Integer index : fusedIndices) {
			keepSet.add(fusedList.get(index));
		}
		Set<ValueProbabilityPair> newMap = fusedValue.getScoredList().stream().filter((e) -> keepSet.contains(e.getValue())).collect(Collectors.toSet());
		return new FusedValue(newMap);
	}

	private List<Object> getEquivalentToLookingFor(List<Object> fusedList, List<Object> lookingFor, ColumnDataType cdt) {
		SimilarityMeasure measure = similarityCalculator.getMeasure(cdt);

		TreeMap<Double, Pair<Integer, Integer>> sims = new TreeMap<>();

		for (int a = 0; a < lookingFor.size(); a++) {
			for (int b = 0; b < fusedList.size(); b++) {
				double similarity = measure.getQuickSimilarity(lookingFor.get(a), fusedList.get(b));
				if (similarity > measure.getEquivalenceThreshold())
					sims.put(similarity, new Pair<>(a, b));

			}
		}

		Set<Integer> lookingForIndices = new HashSet<>();
		for (int a = 0; a < lookingFor.size(); a++) {
			lookingForIndices.add(a);
		}

		Set<Integer> fusedIndices = new HashSet<>();
		for (int b = 0; b < fusedList.size(); b++) {
			fusedIndices.add(b);
		}

		List<Object> retList = new ArrayList<>();

		while (!lookingForIndices.isEmpty() && !fusedIndices.isEmpty()) {
			Entry<Double, Pair<Integer, Integer>> e = sims.pollLastEntry();
			if (e == null) {
				break;
			}
			if (lookingForIndices.contains(e.getValue().getFirst()) && fusedIndices.contains(e.getValue().getSecond())) {
				lookingForIndices.remove(e.getValue().getFirst());
				fusedIndices.remove(e.getValue().getSecond());
				retList.add(fusedList.get(e.getValue().getSecond()));
			}
		}

		return retList;
	}

}
