package de.mannheim.uni.expansion.tablefusion.analysis;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map.Entry;
import java.util.OptionalDouble;

import javax.inject.Inject;
import javax.inject.Named;

import com.opencsv.CSVWriter;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.NamedProperties;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.InputProfileStoreCreator;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.InputProfileStoreCreator.InputProfileStore;
import de.mannheim.uni.expansion.utils.PersistentParallel;

public class AnnotatedInputProfiler {

	private final AnnotatedTablesRepository annotatedTablesRepository;
	private final String outputFolder;

	@Inject
	public AnnotatedInputProfiler(AnnotatedTablesRepository annotatedTablesRepository, @Named(NamedProperties.INPUT_PROFILING_FOLDER) String outputFolder) {
		this.annotatedTablesRepository = annotatedTablesRepository;
		this.outputFolder = outputFolder;
	}

	public InputProfileStore getProfiled() {

		InputProfileStoreCreator inputProfileStoreCreator = new InputProfileStoreCreator();

		try {
			new PersistentParallel<AnnotatedTableId>("Profile Input Data").foreach(annotatedTablesRepository.getAllTableIds(), (id) -> {

				AnnotatedTableMetaData data = annotatedTablesRepository.getMetaData(id);

				inputProfileStoreCreator.incrementTotalTables();

				if (data.getDbPediaClass() != null)
					inputProfileStoreCreator.increment(data.getDbPediaClass());

				int instanceCount = 0;
				for (InstanceId dbpediaInstnace : data.getInstances().values()) {
					if (dbpediaInstnace != null) {
						inputProfileStoreCreator.increment(dbpediaInstnace);
						instanceCount++;
					}
				}

				for (PropertyId dbpediaProperty : data.getProperties().values()) {
					if (dbpediaProperty != null) {
						inputProfileStoreCreator.increment(dbpediaProperty);

						inputProfileStoreCreator.incrementTableCount(new ClassPropertyId(data.getDbPediaClass(), dbpediaProperty));
						inputProfileStoreCreator.incrementValueCount(new ClassPropertyId(data.getDbPediaClass(), dbpediaProperty), instanceCount);

						for (InstanceId dbpediaInstnace : data.getInstances().values()) {
							if (dbpediaInstnace != null) {
								inputProfileStoreCreator.incrementCompositeInstanceValueCount(new ClassPropertyId(data.getDbPediaClass(), dbpediaProperty),
										dbpediaInstnace);
							}

						}

					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return inputProfileStoreCreator.get();
	}

	public void write(InputProfileStore store) throws IOException {
		write(store, this.outputFolder);
	}

	public void write(InputProfileStore store, String outputFolder) throws IOException {
		CSVWriter writerClasses = new CSVWriter(new FileWriter(new File(outputFolder + "/classes.csv")));

		String[] classesHeaderLine = new String[2];
		classesHeaderLine[0] = "Class";
		classesHeaderLine[1] = "Table Count";

		writerClasses.writeNext(classesHeaderLine);

		for (Entry<ClassId, Integer> e : store.getClassCount().entrySet()) {
			String[] line = new String[2];
			line[0] = e.getKey().toString();
			line[1] = e.getValue().toString();
			writerClasses.writeNext(line);
		}

		writerClasses.close();

		CSVWriter writerProperties = new CSVWriter(new FileWriter(new File(outputFolder + "/properties.csv")));

		String[] propertyHeaderLine = new String[2];
		propertyHeaderLine[0] = "Property";
		propertyHeaderLine[1] = "Table Count";

		writerProperties.writeNext(propertyHeaderLine);

		for (Entry<PropertyId, Integer> e : store.getPropertyCount().entrySet()) {
			String[] line = new String[2];
			line[0] = e.getKey().toString();
			line[1] = e.getValue().toString();
			writerProperties.writeNext(line);
		}

		writerProperties.close();

		CSVWriter writerInstances = new CSVWriter(new FileWriter(new File(outputFolder + "/instances.csv")));

		String[] instancesHeaderLine = new String[2];
		instancesHeaderLine[0] = "Instance";
		instancesHeaderLine[1] = "Table Count";

		writerInstances.writeNext(instancesHeaderLine);

		for (Entry<InstanceId, Integer> e : store.getInstanceCount().entrySet()) {
			String[] line = new String[2];
			line[0] = e.getKey().toString();
			line[1] = e.getValue().toString();
			writerInstances.writeNext(line);
		}

		writerInstances.close();

		CSVWriter compositeWriter = new CSVWriter(new FileWriter(new File(outputFolder + "/composite.csv")));

		String[] headerLine = new String[7];
		headerLine[0] = "Class";
		headerLine[1] = "Property";
		headerLine[2] = "Total Tables";
		headerLine[3] = "Total Values";
		headerLine[4] = "Total Instances";
		headerLine[5] = "Average values per Instance";
		headerLine[6] = "Median Values per Instance";

		compositeWriter.writeNext(headerLine);

		for (Entry<ClassPropertyId, Integer> e : store.getCompositeTableCount().entrySet()) {
			String[] line = new String[7];
			line[0] = e.getKey().getClassId().toString();
			line[1] = e.getKey().getPropertyId().toString();
			line[2] = e.getValue().toString();
			line[3] = String.valueOf(store.getCompositeValueCount(e.getKey()));

			line[4] = String.valueOf(store.getCompositeInstanceCount(e.getKey()).size());

			OptionalDouble optionalAverage = store.getAverageNumberOfInstances(e.getKey());

			if (optionalAverage.isPresent()) {
				line[5] = String.valueOf(optionalAverage.getAsDouble());
			} else {
				line[5] = "";
			}

			OptionalDouble optionalMedian = store.getMedianNumberOfInstances(e.getKey());
			if (optionalMedian.isPresent()) {
				line[6] = String.valueOf(optionalMedian.getAsDouble());
			} else {
				line[6] = "";
			}

			compositeWriter.writeNext(line);
		}

		compositeWriter.close();
	}
}
