package de.mannheim.uni.expansion.tablefusion.analysis;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.services.grouping.GroupedValuesRepository;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class MaxCorrectFinder {

	private final GroupedValuesRepository groupedValuesRepository;
	private final OntologyRepository ontologyRepository;
	private final ValueEvaluator valueEvaluator;

	private final PersistentParallel3<Entry<TripleId, MatchedValues>> getMaxCorrectValues = new PersistentParallel3<Map.Entry<TripleId, MatchedValues>>(
			"MaxCorrectValuesTotal");
	private final PersistentParallel3<Entry<TripleId, MatchedValues>> getMaxCorrectValuePerDataType = new PersistentParallel3<Map.Entry<TripleId, MatchedValues>>(
			"MaxCorrectValuesDataType");

	private final PersistentParallel3<Entry<TripleId, MatchedValues>> getMaxCorrectValuePerClass = new PersistentParallel3<Map.Entry<TripleId, MatchedValues>>(
			"MaxCorrectValuesClass");
	private final PersistentParallel3<Entry<TripleId, MatchedValues>> getMaxCorrectValuePerClassPropertyId = new PersistentParallel3<Map.Entry<TripleId, MatchedValues>>(
			"MaxCorrectValuesClassPropertyId");

	@Inject
	public MaxCorrectFinder(GroupedValuesRepository groupedValuesRepository, OntologyRepository ontologyRepository, ValueEvaluator valueEvaluator) {
		this.groupedValuesRepository = groupedValuesRepository;
		this.ontologyRepository = ontologyRepository;
		this.valueEvaluator = valueEvaluator;
	}

	public void printMaxCorrectValuesInCorpus(boolean fuseLists) {

		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();

		int maxCorrect = getMaxCorrectValues(groupedValues, fuseLists);

		System.out.println("Max Correct Values in Corpus: " + maxCorrect);

		System.out.println("Max Correct Values for DataType");
		for (Entry<ColumnDataType, Integer> e : getMaxCorrectValuePerDataType(groupedValues, fuseLists).entrySet()) {
			System.out.println(e.getKey().toString() + ": " + e.getValue());
		}
	}

	public int getMaxCorrectValues(GroupedValues groupedValues, boolean fuseLists) {
		return getMaxCorrectValues(groupedValues, fuseLists, null);
	}

	public int getMaxCorrectValues(GroupedValues groupedValues, boolean fuseLists, FilteredValues filteredValues) {

		AtomicInteger maxCorrect = new AtomicInteger();

		try {
			getMaxCorrectValues.foreach(groupedValues.entrySet(), (e) -> {

				// ColumnDataType cdt =
				// ontologyRepository.getDataType(e.getKey().getClassId(),
				// e.getKey().getPropertyId());

				if (correct(e.getKey(), e.getValue().getValues(), fuseLists, filteredValues)) {
					maxCorrect.incrementAndGet();

					// AtomicInteger maxCorrectPerDataType =
					// CHMHelper.getFromCHM(dataTypes, cdt, () -> new
					// AtomicInteger());
					// maxCorrectPerDataType.incrementAndGet();
				}

			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return maxCorrect.get();
	}

	public Map<ColumnDataType, Integer> getMaxCorrectValuePerDataType(GroupedValues groupedValues, boolean fuseLists) {

		return getMaxCorrectValuePerDataType(groupedValues, fuseLists, null);
	}

	public Map<ColumnDataType, Integer> getMaxCorrectValuePerDataType(GroupedValues groupedValues, boolean fuseLists, FilteredValues filteredValues) {
		ConcurrentHashMap<ColumnDataType, AtomicInteger> dataTypes = new ConcurrentHashMap<>();

		try {
			getMaxCorrectValuePerDataType.foreach(groupedValues.entrySet(), (e) -> {

				if (correct(e.getKey(), e.getValue().getValues(), fuseLists, filteredValues)) {

					ColumnDataType cdt = ontologyRepository.getDataType(e.getKey().getClassId(), e.getKey().getPropertyId());

					AtomicInteger maxCorrectPerDataType = CHMHelper.getFromCHM(dataTypes, cdt, () -> new AtomicInteger());
					maxCorrectPerDataType.incrementAndGet();
				}

			});
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		return dataTypes.entrySet().parallelStream().collect(Collectors.toMap(Entry::getKey, e -> e.getValue().get()));
	}

	public Map<ClassId, Integer> getMaxCorrectValuePerClass(GroupedValues groupedValues, boolean fuseLists, FilteredValues filteredValues) {
		ConcurrentHashMap<ClassId, AtomicInteger> map = new ConcurrentHashMap<>();

		try {
			getMaxCorrectValuePerClass.foreach(groupedValues.entrySet(), (e) -> {

				if (correct(e.getKey(), e.getValue().getValues(), fuseLists, filteredValues)) {

					AtomicInteger maxCorrect = CHMHelper.getFromCHM(map, e.getKey().getClassId(), () -> new AtomicInteger());
					maxCorrect.incrementAndGet();
				}

			});
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		return map.entrySet().parallelStream().collect(Collectors.toMap(Entry::getKey, e -> e.getValue().get()));
	}

	public Map<ClassPropertyId, Integer> getMaxCorrectValuePerClassPropertyId(GroupedValues groupedValues, boolean fuseLists, FilteredValues filteredValues) {
		ConcurrentHashMap<ClassPropertyId, AtomicInteger> map = new ConcurrentHashMap<>();

		try {
			getMaxCorrectValuePerClassPropertyId.foreach(groupedValues.entrySet(), (e) -> {

				if (correct(e.getKey(), e.getValue().getValues(), fuseLists, filteredValues)) {

					AtomicInteger maxCorrect = CHMHelper.getFromCHM(map, new ClassPropertyId(e.getKey()), () -> new AtomicInteger());
					maxCorrect.incrementAndGet();
				}

			});
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		return map.entrySet().parallelStream().collect(Collectors.toMap(Entry::getKey, e -> e.getValue().get()));
	}

	private boolean correct(TripleId tid, List<SingleValue> values, boolean fuseLists, FilteredValues filteredValues) {
		List<Object> combinedValues = values.stream().map(SingleValue::getValue).collect(Collectors.toList());
		return valueEvaluator.maxCorrect(tid, combinedValues, fuseLists, filteredValues);

	}
}
