package de.mannheim.uni.expansion.tablefusion.analysis;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.similarity.Similarity;
import de.mannheim.uni.expansion.tablefusion.core.similarity.Similarity.SimilarityType;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.list.OverlapSimilarity;
import de.mannheim.uni.expansion.utils.PersistentParallel;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class ValueEvaluator {

	private final DBpediaRepositoryNewInterface dbPediaRepository;

	private final OntologyRepository ontologyRepository;

	private final SimilarityCalculator similarityCalculator;

	private final PersistentParallel<Entry<TripleId, Object>> parallel;

	private final OverlapSimilarity overlapSimilarity;

	@Inject
	public ValueEvaluator(DBpediaRepositoryNewInterface dbPediaRepository, SimilarityCalculator similarityCalculator, OntologyRepository ontologyRepository,
			OverlapSimilarity overlapSimilarity) {
		this.dbPediaRepository = dbPediaRepository;
		this.similarityCalculator = similarityCalculator;
		this.parallel = new PersistentParallel<>("AccuracyCalculator");
		this.ontologyRepository = ontologyRepository;
		this.overlapSimilarity = overlapSimilarity;
	}

	public Similarity getSimilarity(Object correctValue, Object fusedValue, ColumnDataType cdt, SimilarityType type) {
		return new Similarity(this.similarityCalculator.getSimilarity(cdt, correctValue, fusedValue), type, cdt);
	}

	public Similarity getSimilarity(TripleId tripleId, Object fusedValue, boolean fuseLists) {

		boolean isList = dbPediaRepository.isListType(tripleId.getClassPropertyId());
		SimilarityType type = SimilarityType.DIRECT;
		if (fuseLists && isList) {
			type = SimilarityType.LIST_OVERLAP;
		}
		Object correctValue = dbPediaRepository.getValue(tripleId);
		if (correctValue == null)
			return null;
		ColumnDataType cdt = ontologyRepository.getDataType(tripleId.getClassId(), tripleId.getPropertyId());
		if (cdt == null) {
			System.out.println("B: CDT == null, please check!");
			return null;
		}
		return new Similarity(this.similarityCalculator.getSimilarity(cdt, correctValue, fusedValue), type, cdt);
	}

	public boolean correct(Similarity similarity, ColumnDataType cdt) {

		if (similarity.getSim() > similarityCalculator.getEquivalenceThreshold(cdt)) {
			return true;
		} else {
			return false;
		}

	}

	public boolean correct(Similarity similarity, TripleId key) {
		ColumnDataType cdt = ontologyRepository.getDataType(key.getClassId(), key.getPropertyId());
		return correct(similarity, cdt);
	}

	// public boolean correct(TripleId tripleId, Object correctValue, Object
	// fusedValue, ColumnDataType cdt, Set<Object> filteredValues) {
	//
	// if (correctValue == null)
	// return false;
	// else if (cdt == null) {
	// System.out.println("B: CDT == null, please check!");
	// return false;
	// }
	//
	// else if (filteredValues != null && fusedValue instanceof List<?>) {
	//
	// List<Object> fusedList = new ArrayList<>((List<Object>) fusedValue);
	//
	// List<Object> correctList = new ArrayList<>(filteredValues);
	//
	// double sim = similarityCalculator.getSimilarity(cdt, fusedList,
	// correctList);
	// if (sim > 0.95) {
	// return true;
	// } else {
	// return false;
	// }
	//
	// } else {
	// return correct(correctValue, fusedValue, cdt);
	// }
	// // double sim = this.similarityCalculator.getSimilarity(cdt,
	// // correctValue, fusedValue);
	// // double equivlanceThreshold =
	// // this.similarityCalculator.getEquivalenceThreshold(cdt);
	// //
	// // if (sim > equivlanceThreshold)
	// // return true;
	// // else
	// // return false;
	// }

	public boolean correct(TripleId tripleId, Object correctValue, FusedValue fusedValue, ColumnDataType cdt) {
		return correct(tripleId, correctValue, fusedValue, cdt, null);
	}

	@SuppressWarnings("unchecked")
	public boolean correct(TripleId tripleId, Object correctValue, FusedValue fusedValue, ColumnDataType cdt, FilteredValues filteredValues) {

		if (correctValue == null)
			return false;
		else if (cdt == null) {
			System.out.println("B: CDT == null, please check!");
			return false;
		} else if (dbPediaRepository.isListType(tripleId.getClassPropertyId())) {
			if (filteredValues != null) {
				if (!(fusedValue.isList()))
					throw new IllegalStateException("No List provided even though we are in the ListExpansion use case");
				else {
					Set<Object> lookingFor = filteredValues.getFilteredValues(tripleId);
					if (lookingFor != null && lookingFor.size() > 0) {
						List<Object> correctList = new ArrayList<>(lookingFor);

						// Set<Object> valueToEliminate = new
						// HashSet<>((List<Object>) correctValue);
						// valueToEliminate.removeAll(correctList);
						// for (Iterator<Object> it = fusedList.iterator();
						// it.hasNext();) {
						// if (valueToEliminate.contains(it.next())) {
						// it.remove();
						// }
						//
						// }

						double sim = overlapSimilarity.getMAP(correctList, fusedValue.getRankedList(), similarityCalculator.getMeasure(cdt));

						if (sim > 0.9)
							return true;
						else
							return false;

					}

					else {
						return false;
					}
				}
			} else {
				if (correctValue instanceof List<?> && fusedValue.isList()) {

					List<Object> correctList = (List<Object>) correctValue;

					double sim = overlapSimilarity.getMAP(correctList, fusedValue.getRankedList(), similarityCalculator.getMeasure(cdt));
					if (sim > 0.9) {
						return true;
					} else {
						return false;
					}

				} else {
					List<Object> correctList;
					if (!(correctValue instanceof List<?>)) {
						correctList = new ArrayList<>();
						correctList.add(correctValue);
					} else {
						correctList = (List<Object>) correctValue;
					}

					List<Object> fusedValueList;
					if (!(fusedValue.isList())) {
						fusedValueList = new ArrayList<>();
						fusedValueList.add(fusedValue.getSingle());
					} else {
						fusedValueList = fusedValue.getSimpleList();
					}

					int overlap = overlapSimilarity.getOverlap(fusedValueList, correctList, similarityCalculator.getMeasure(cdt));

					if (overlap == 1) {
						return true;
					} else {
						return false;
					}
				}
			}
			//
			// int shorterSize = groupedSet.size();
			// // if (groupedSet.size() < shorterSize) {
			// // shorterSize = groupedSet.size();
			// // }
			// if (correctList.size() < shorterSize) {
			// shorterSize = correctList.size();
			// }
			//
			// if (dbPediaRepository.isListType(ListType.SHORT,
			// classPropertyId)) {
			// if (shorterSize > 1) {
			// shorterSize = 1;
			// }
			// } else if (dbPediaRepository.isListType(ListType.MEDIUM,
			// classPropertyId)) {
			// if (shorterSize > 2) {
			// shorterSize = 2;
			// }
			// } else if (dbPediaRepository.isListType(ListType.LONG,
			// classPropertyId)) {
			// if (shorterSize > 5) {
			// shorterSize = 5;
			// }
			// }
			//
			// if (overlap >= shorterSize) {
			// return true;
			// } else {
			// return false;
			// }

		} else {

			double sim = this.similarityCalculator.getSimilarity(cdt, correctValue, fusedValue.getSingle());

			double equivlanceThreshold = this.similarityCalculator.getEquivalenceThreshold(cdt);

			if (sim > equivlanceThreshold)
				return true;
			else
				return false;

		}

		// if (cdt == ColumnDataType.numeric) {
		//
		// double sim =
		// this.similarityCalculator.getSimilarity(ColumnDataType.numeric,
		// correctValue, fusedValue);
		// if (sim >
		// this.similarityCalculator.getEquivalenceThreshold(ColumnDataType.numeric))
		// {
		//
		// correct = true;
		// } else {
		// // System.out.println();
		// }
		//
		// } else if (cdt == ColumnDataType.date) {
		// double sim =
		// this.similarityCalculator.getSimilarity(ColumnDataType.date,
		// correctValue, fusedValue);
		// if (sim >
		// this.similarityCalculator.getEquivalenceThreshold(ColumnDataType.date))
		// {
		//
		// correct = true;
		// } else {
		// // System.out.println();
		// }
		//
		// } else if (cdt == ColumnDataType.string) {
		// double sim =
		// this.similarityCalculator.getSimilarity(ColumnDataType.string,
		// correctValue, fusedValue);
		// if (sim >
		// this.similarityCalculator.getEquivalenceThreshold(ColumnDataType.string))
		// {
		//
		// correct = true;
		// } else {
		// // System.out.println();
		// }
		// } else {
		// // System.out.println("UNKOWN TYPE" + cdt.toString());
		// }
		// return correct;
	}

	public Boolean correct(TripleId tripleId, FusedValue fusedValue) {
		Object correctValue = dbPediaRepository.getValue(tripleId);
		if (correctValue == null)
			return null;
		ColumnDataType cdt = ontologyRepository.getDataType(tripleId.getClassId(), tripleId.getPropertyId());
		if (cdt == null) {
			System.out.println("B: CDT == null, please check!");
			return null;
		}
		return correct(tripleId, correctValue, fusedValue, cdt, null);
	}

	public boolean maxCorrect(TripleId tripleId, List<Object> values, boolean fuseLists, FilteredValues filteredValues) {

		ClassPropertyId classPropertyId = tripleId.getClassPropertyId();
		Object correctValue = dbPediaRepository.getValue(tripleId);
		if (correctValue == null)
			return false;
		ColumnDataType cdt = ontologyRepository.getDataType(tripleId.getClassId(), tripleId.getPropertyId());
		if (cdt == null) {
			System.out.println("B: CDT == null, please check!");
			return false;
		}

		else if (fuseLists && dbPediaRepository.isListType(classPropertyId)) {

			Set<Object> groupedSet = new HashSet<>();
			for (Object v : values) {
				if (v instanceof List<?>) {
					@SuppressWarnings("unchecked")
					List<Object> valueList = (List<Object>) v;

					groupedSet.addAll(valueList);
				} else {
					groupedSet.add(v);
				}

			}

			if (groupedSet.size() == 0)
				return false;

			if (filteredValues != null) {
				Set<Object> lookingFor = filteredValues.getFilteredValues(tripleId);
				if (lookingFor != null && lookingFor.size() > 0) {

					if (overlapSimilarity.hasOverlap(new ArrayList<>(lookingFor), new ArrayList<>(groupedSet), similarityCalculator.getMeasure(cdt),
							lookingFor.size())) {
						return true;
					} else {
						return false;
					}

				} else {
					return false;
				}

			} else {

				List<Object> groupedList = new ArrayList<>(groupedSet);

				if (correctValue instanceof List<?>) {
					List<Object> correctList = (List<Object>) correctValue;
					return overlapSimilarity.hasOverlap(correctList, groupedList, similarityCalculator.getMeasure(cdt), correctList.size());
				} else {

					List<Object> correctList = new ArrayList<>();
					correctList.add(correctValue);

					int overlap = overlapSimilarity.getOverlap(correctList, groupedList, similarityCalculator.getMeasure(cdt));

					if (overlap == 1) {
						return true;
					} else {
						return false;
					}
				}

			}

		} else {
			boolean correct = false;
			for (Object v : values) {

				if (this.similarityCalculator.isEqual(cdt, correctValue, v)) {
					correct = true;
					break;
				}

			}
			return correct;
		}
	}

}
