package de.mannheim.uni.expansion.tablefusion.core.scoring;

import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.TableColumnId;
import de.mannheim.uni.expansion.tablefusion.analysis.Accuracy;
import de.mannheim.uni.expansion.tablefusion.analysis.ValueEvaluator;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ColumnScores;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.PersistentParallel4;
import de.mannheim.uni.expansion.utils.SimpleConcurrentCache;
import de.mannheim.uni.expansion.utils.containers.Pair;

public class ProbabilitiesCalculator {

	private static SimpleConcurrentCache<GroupedValues, ColumnScores> cache = new SimpleConcurrentCache<>();
	private final ValueEvaluator valueEvaluator;

	@Inject
	public ProbabilitiesCalculator(ValueEvaluator valueEvaluator) {
		super();
		this.valueEvaluator = valueEvaluator;
	}

	public ColumnScores byProperty(GroupedValues learningSet) {

		if (cache.hasCacheFor(learningSet)) {
			return cache.get(learningSet);
		} else {

			Map<TableColumnId, Map<TripleId, FusedValue>> grouped = groupByTableColumnId(learningSet);

			ConcurrentHashMap<PropertyId, Map<TableColumnId, Probability>> probabilties = new ConcurrentHashMap<PropertyId, Map<TableColumnId, Probability>>();

			PersistentParallel4.runAndFinish(grouped.entrySet(), (e) -> {

				Pair<PropertyId, Accuracy> ret = getAccuracy(e.getValue());

				Map<TableColumnId, Probability> propertyMap = CHMHelper.getFromCHM(probabilties, ret.getFirst(), () -> new ConcurrentHashMap<>());

				propertyMap.put(e.getKey(), new Probability(ret.getSecond()));

			} , "KBTCalculator");

			ColumnScores cs = new ColumnScores(probabilties);

			cache.add(learningSet, cs);
			return cs;
		}
	}

	private Pair<PropertyId, Accuracy> getAccuracy(Map<TripleId, FusedValue> values) {
		double correct = 0.0;
		double incorrect = 0.0;
		int total = 0;
		PropertyId propertyId = null;
		for (Entry<TripleId, FusedValue> e : values.entrySet()) {
			Boolean isCorrect = valueEvaluator.correct(e.getKey(), e.getValue());
			if (isCorrect != null) {
				total++;
				if (isCorrect) {
					correct++;
				} else {
					incorrect++;
				}
			}

			if (propertyId == null) {
				propertyId = e.getKey().getPropertyId();
			} else if (!propertyId.equals(e.getKey().getPropertyId())) {
				throw new IllegalStateException("THIS COLUMN HAS VALUES WITH DIFFERENT PROPERTY MAPPINGS. THIS IS ABSOLUTELY HARAM!");

			}
		}

		return new Pair<PropertyId, Accuracy>(propertyId, new Accuracy((correct) / (correct + incorrect), total));
	}

	private Map<TableColumnId, Map<TripleId, FusedValue>> groupByTableColumnId(GroupedValues learningSet) {
		ConcurrentHashMap<TableColumnId, Map<TripleId, FusedValue>> groupedValues = new ConcurrentHashMap<TableColumnId, Map<TripleId, FusedValue>>();

		PersistentParallel4.runAndFinish(learningSet.entrySet(), (e) -> {
			for (SingleValue sv : e.getValue().getValues()) {
				Map<TripleId, FusedValue> tableColumnIdMap = CHMHelper.getFromCHM(groupedValues, sv.getId().getTableColumnId(),
						() -> new ConcurrentHashMap<>());

				FusedValue fusedValue = FusedValue.fromObject(sv.getValue());

				tableColumnIdMap.put(e.getKey(), fusedValue);
			}
		} , "KBTCalculator - Group Values by TableColumnId");

		return groupedValues;
	}
}
