package de.mannheim.uni.expansion.tablefusion.core.scoring;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.BiFunction;
import java.util.stream.Collectors;

import javax.inject.Inject;

import org.apache.commons.lang.mutable.MutableInt;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.ValueId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.model.CellId;
import de.mannheim.uni.expansion.data.tables.model.TableColumnId;
import de.mannheim.uni.expansion.data.tables.repository.AnnotatedTablesRepository;
import de.mannheim.uni.expansion.tablefusion.analysis.Accuracy;
import de.mannheim.uni.expansion.tablefusion.core.fusing.BaseFuser;
import de.mannheim.uni.expansion.tablefusion.core.fusing.FuserRepository;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues.SingleValue;
import de.mannheim.uni.expansion.tablefusion.core.scoring.YearEquivalenceClusterer.Cluster;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.TripleIdScores;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.tablefusion.core.similarity.measures.NumericSimilarity;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.PersistentParallel4;
import de.mannheim.uni.expansion.utils.SimpleConcurrentCache;
import de.mannheim.uni.expansion.utils.ThreadSafeLocalCalendar;
import de.mannheim.uni.expansion.utils.containers.Pair;
import de.mannheim.uni.expansion.utils.containers.Tri;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;
import de.mannheim.uni.utils.concurrent.Consumer;

public class TimeStampsScoringCalculator {

	private static class TS_METHOD extends Pair<String, BiFunction<TripleId, CellId, Integer>> {

		public TS_METHOD(String first, BiFunction<TripleId, CellId, Integer> second) {
			super(first, second);
		}

	}

	int minYear = 2000;
	int maxYear = 2014;
	double maxDiff = maxYear - minYear + 1.0;
	public final TS_METHOD CELL_YEAR = new TS_METHOD("CELL_YEAR", this::getCellYear);
	public final TS_METHOD MAX_COLUMN_YEAR = new TS_METHOD("MAX_COLUMN_YEAR", this::getMaxColumnYear);
	public final TS_METHOD MAX_TABLE_TILE = new TS_METHOD("MAX_TABLE_TILE", this::getMaxTableTitleYear);
	public final TS_METHOD MAX_PAGE_TITLE = new TS_METHOD("MAX_PAGE_TITLE", this::getMaxPageTitleYear);
	public final TS_METHOD TABLE_YEAR_HIERARCHY = new TS_METHOD("TABLE_YEAR_HIERARCHY", this::getTableYearHiearchy);
	public final TS_METHOD MOST_COMMON_TABLE_TEXT_YEAR = new TS_METHOD("MOST_COMMON_TABLE_TEXT_YEAR", this::getMostCommonTableTextYear);
	public final TS_METHOD GET_HIERARICAL_YEAR = new TS_METHOD("GET_HIERARICAL_YEAR", this::getHiearicalYear);
	public final TS_METHOD GET_FULL_HIERARICAL_YEAR = new TS_METHOD("GET_FULL_HIERARICAL_YEAR", this::getFullHiearicalYear);

	private final AnnotatedTablesRepository annotatedTablesRepository;
	private final NumericSimilarity numericSimilarity;
	private final DBpediaRepositoryNewInterface dbpediaRepository;
	private final ThreadSafeLocalCalendar calendar;
	private final YearEquivalenceClusterer yearEquivalenceClusterer;
	private final FuserRepository fuserRepository;
	private final SimilarityCalculator similarityCalculator;

	@Inject
	public TimeStampsScoringCalculator(AnnotatedTablesRepository annotatedTablesRepository, NumericSimilarity numericSimilarity,
			DBpediaRepositoryNewInterface dbpediaRepository, ThreadSafeLocalCalendar calendar, YearEquivalenceClusterer yearEquivalenceClusterer,
			FuserRepository fuserRepository, SimilarityCalculator similarityCalculator) {

		this.annotatedTablesRepository = annotatedTablesRepository;
		this.numericSimilarity = numericSimilarity;
		this.dbpediaRepository = dbpediaRepository;
		this.calendar = calendar;
		this.yearEquivalenceClusterer = yearEquivalenceClusterer;
		this.fuserRepository = fuserRepository;
		this.similarityCalculator = similarityCalculator;
	}

	private boolean yearInRange(Integer year) {
		if (year != null && year >= minYear && year <= maxYear)
			return true;
		else
			return false;
	}

	public Integer getMaxColumnYear(TripleId tid, CellId aid) {

		AnnotatedTableMetaData md = annotatedTablesRepository.getMetaData(aid.getTableId());

		Integer year = null;

		if (md.getPropertiesMaxYear() != null) {
			Integer iYear = md.getPropertiesMaxYear().get(aid.getColumnIdx().getColumnIdx());

			if (yearInRange(year)) {
				if (year == null || iYear > year) {
					year = iYear;
				}
			}
		}

		return year;
	}

	public Integer getCellYear(TripleId tid, CellId aid) {

		PropertyId activeYearsStartYear = new PropertyId("http://dbpedia.org/ontology/activeYearsStartYear");
		AnnotatedTableMetaData md = annotatedTablesRepository.getMetaData(aid.getTableId());
		Table td = annotatedTablesRepository.getData(aid.getTableId());

		Integer year = null;
		for (int i = 0; i < td.getColumns().size(); i++) {
			if (td.getColumns().get(i).getDataType() == ColumnDataType.date) {
				PropertyId propertyId = md.getProperty(i);
				if (propertyId == null || propertyId.equals(activeYearsStartYear)) {
					Object oYear = td.getColumns().get(i).getValues().get(aid.getRowIdx().getRowIdx());
					if (oYear instanceof Date) {
						calendar.getCalendar().setTime((Date) oYear);
						int iYear = calendar.getCalendar().get(Calendar.YEAR);

						if (year == null || iYear > year) {
							year = iYear;
						}
					}
				}
			}

		}

		return year;
	}

	public Integer getMaxTableTitleYear(TripleId tid, CellId aid) {
		AnnotatedTableMetaData md = annotatedTablesRepository.getMetaData(aid.getTableId());

		Integer year = md.getMaxYearInTableTitle();
		if (yearInRange(year)) {
			return year;
		} else {
			return null;
		}

	}

	public Integer getMaxPageTitleYear(TripleId tid, CellId aid) {
		AnnotatedTableMetaData md = annotatedTablesRepository.getMetaData(aid.getTableId());

		Integer year = md.getMaxYearInPageTitle();
		if (yearInRange(year)) {
			return year;
		} else {
			return null;
		}

	}

	public Integer getTableYearHiearchy(TripleId tid, CellId aid) {
		Integer tableTitleYear = getMaxTableTitleYear(tid, aid);
		if (tableTitleYear != null) {
			return tableTitleYear;
		} else {
			return getMostCommonTableTextYear(tid, aid);
		}
	}

	public Integer getMostCommonTableTextYear(TripleId tid, CellId aid) {
		AnnotatedTableMetaData md = annotatedTablesRepository.getMetaData(aid.getTableId());

		Map<Integer, MutableInt> yearCount = new HashMap<>();

		Consumer<List<Integer>> addYears = (list) -> {
			if (list != null) {
				for (Integer year : list) {
					if (yearInRange(year)) {
						CHMHelper.getFromMap(yearCount, year, () -> new MutableInt(0)).add(1);
					}

				}
			}
		};

		addYears.execute(md.getYearsAfterText());
		addYears.execute(md.getYearsBeforeText());

		if (!yearCount.isEmpty()) {
			Integer year = yearCount.entrySet().stream().max((e1, e2) -> {
				if (e1.getValue().intValue() > e2.getValue().intValue()) {
					return 1;
				} else if (e1.getValue().intValue() < e2.getValue().intValue()) {
					return -1;
				} else if (e1.getKey() > e2.getKey()) {
					return 1;
				} else
					return -1;
			}).get().getKey();

			return year;
		} else {
			return null;
		}
	}

	public Integer getHiearicalYear(TripleId tid, CellId aid) {
		Integer columnYear = getMaxColumnYear(tid, aid);
		if (columnYear != null)
			return columnYear;
		else {
			Integer cellYear = getCellYear(tid, aid);
			if (cellYear != null)
				return cellYear;
			else {
				Integer tableYear = getMaxTableTitleYear(tid, aid);
				if (tableYear != null) {
					return tableYear;
				} else {
					Integer pageTitle = getMaxPageTitleYear(tid, aid);
					return pageTitle;
				}
			}
		}
	}

	public Integer getFullHiearicalYear(TripleId tid, CellId aid) {
		Integer columnYear = getMaxColumnYear(tid, aid);
		if (columnYear != null)
			return columnYear;
		else {
			Integer cellYear = getCellYear(tid, aid);
			if (cellYear != null)
				return cellYear;
			else {
				Integer tableYear = getMaxTableTitleYear(tid, aid);
				if (tableYear != null) {
					return tableYear;
				} else {
					Integer pageTitle = getMaxPageTitleYear(tid, aid);
					if (pageTitle != null) {
						return pageTitle;
					} else {
						return getMostCommonTableTextYear(tid, aid);
					}
				}
			}
		}
	}

	// public ColumnScores getColumnYearsScores(GroupedValues groupedValues) {
	//
	// return genericYearScorer(groupedValues, this::getMaxColumnYear);
	//
	// }
	//
	// public ColumnScores getTableYears(GroupedValues groupedValues) {
	//
	// return genericYearScorer(groupedValues, this::getTableYearHiearchy);
	//
	// }
	//
	// public ColumnScores getMaxTableTitleYear(GroupedValues groupedValues) {
	//
	// return genericYearScorer(groupedValues, this::getMaxTableTitleYear);
	//
	// }
	//
	// public ColumnScores getHierarichalYears(GroupedValues groupedValues) {
	// return genericYearScorer(groupedValues, this::getHiearicalYear);
	// }
	//
	// public ColumnScores getFullHierarichalYears(GroupedValues groupedValues)
	// {
	// return genericYearScorer(groupedValues, this::getFullHiearicalYear);
	// }

	private static SimpleConcurrentCache<Tri<GroupedValues, String, Boolean>, TripleIdScores> cache = new SimpleConcurrentCache<>();

	public ScoresContainer genericYearScorer(GroupedValues groupedValues, TS_METHOD tsMethod) {

		return genericYearScorer(groupedValues, tsMethod, false);
	}

	public static ConcurrentHashMap<ClassPropertyId, MutableInt> original = new ConcurrentHashMap<>();
	public static ConcurrentHashMap<ClassPropertyId, MutableInt> propogated = new ConcurrentHashMap<>();

	public ScoresContainer genericYearScorer(GroupedValues groupedValues, TS_METHOD tsMethod, boolean propagate) {

		Tri<GroupedValues, String, Boolean> cacheKey = new Tri<>(groupedValues, tsMethod.getFirst(), propagate);
		// cache = new SimpleConcurrentCache<>();
		if (cache.hasCacheFor(cacheKey)) {
			return cache.get(cacheKey);
		} else {

			BiFunction<TripleId, CellId, Integer> yearGetter = tsMethod.getSecond();
			ConcurrentHashMap<TripleId, Map<CellId, Probability>> valueScores = new ConcurrentHashMap<>();

			PersistentParallel4.runAndFinish(groupedValues.entrySet(), (gv) -> {

				Set<TableColumnId> propertyToTableMapping = new HashSet<>();

				for (SingleValue sv : gv.getValue().getValues()) {
					propertyToTableMapping.add(sv.getId().getTableColumnId());
				}

				SimilarityMeasure measure = similarityCalculator.getMeasure(gv.getValue().getDataType());
				BaseFuser baseFuser = fuserRepository.getBaseFuser(gv.getValue().getDataType());

				TripleId tid = gv.getKey();
				int kbYear = getKBYear(tid.getValueId());
				SortedMap<Integer, Set<CellId>> sortedByYear = new TreeMap<>();
				List<Cluster> clusters = new ArrayList<>();
				Map<CellId, Probability> subCHMMap = new HashMap<>();
				List<SingleValue> noScoreSV = new ArrayList<>();
				for (SingleValue sv : gv.getValue().getValues()) {
					CellId aid = sv.getId();
					Integer year = yearGetter.apply(tid, aid);

					if (year != null) {
						CHMHelper.getFromMap(sortedByYear, year, () -> new HashSet<>()).add(aid);
						if (propagate) {
							clusters = yearEquivalenceClusterer.addValue(clusters, measure, baseFuser, sv.getValue(), year);
						}
					} else {
						noScoreSV.add(sv);
					}

				}

				if (propagate) {

					CHMHelper.getFromCHM(original, tid.getClassPropertyId(), () -> new MutableInt(0)).add(sortedByYear.entrySet().stream().map((e) -> {
						return e.getValue().size();
					}).mapToInt(Integer::intValue).sum());

					for (Iterator<SingleValue> it = noScoreSV.iterator(); it.hasNext();) {
						SingleValue sv = it.next();
						Integer year = yearEquivalenceClusterer.getYear(clusters, measure, sv.getValue());
						if (year != null) {
							CHMHelper.getFromMap(sortedByYear, year, () -> new HashSet<>()).add(sv.getId());
							it.remove();
							CHMHelper.getFromCHM(propogated, tid.getClassPropertyId(), () -> new MutableInt(0)).add(1);
						}
					}

				}

				Set<CellId> noScore = noScoreSV.stream().map((e) -> {
					return e.getId();
				}).collect(Collectors.toSet());

				int divider = sortedByYear.size() + (noScore.isEmpty() ? 0 : 1);
				double step = 1.0 / divider;

				int i = 1;
				double lowestScore = 1.0;
				for (Entry<Integer, Set<CellId>> e : sortedByYear.entrySet()) {

					double score = getYearSimilarity(kbYear, e.getKey());

					// if (e.getKey() >= 2012) {
					for (CellId aid : e.getValue()) {
						subCHMMap.put(aid, new Probability(new Accuracy(score, 1)));
					}

					if (score < lowestScore) {
						lowestScore = score;
					}
					// } else {
					// for (TableColumnId aid : e.getValue()) {
					// subCHMMap.put(aid, new Probability(new Accuracy(0.0,
					// 1)));
					// }
					// }

					i++;

				}

				// if (sortedByYear.isEmpty()) {
				// for (CellId aid : noScore) {
				// subCHMMap.put(aid, new Probability(new Accuracy(0.0, 1)));
				// }
				// } else {
				// for (CellId aid : noScore) {
				// subCHMMap.put(aid, new Probability(new Accuracy(0.0, 1)));
				// }
				// }

				int dfdf = 123123;
				if (!subCHMMap.isEmpty()) {
					valueScores.put(tid, subCHMMap);
				}

			} , "ColumnYearsScores");

			TripleIdScores cs = new TripleIdScores(valueScores);
			cache.add(cacheKey, cs);

			return cs;
		}

	}

	private static PropertyId yearProperty = new PropertyId("http://uni-mannheim.de/year");

	public int getKBYear(ValueId vid) {
		Object o = dbpediaRepository.getValue(new ValueId(vid.getInstanceId(), yearProperty));
		if (o != null) {
			if (o instanceof Date) {
				calendar.getCalendar().setTime((Date) o);
				int ret = calendar.getCalendar().get(Calendar.YEAR);

				return ret;
			}
		}

		throw new IllegalStateException();
	}

	public double getYearSimilarity(int kbYear, int matchedYear) {
		if (matchedYear == kbYear)
			return 1.0;
		else {

			int diff = Math.abs(kbYear - matchedYear);
			if (diff < 4) {
				return 1.0 - (diff / 4.0);
			}
		}

		return 0.0;
	}
}
