package de.mannheim.uni.expansion.tablefusion.evaluation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import javax.inject.Inject;

import com.google.common.util.concurrent.AtomicDouble;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.PairData;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.ValueData;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.ValuePair;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.PersistentParallel;

public class EvaluationNormalizer {

	private final DBpediaRepositoryNewInterface dbPediaRepository;

	@Inject
	public EvaluationNormalizer(DBpediaRepositoryNewInterface dbPediaRepository) {
		this.dbPediaRepository = dbPediaRepository;

	}

	/**
	 * Normalizes the proportion given in statistics based on actual values
	 * provided in valuePairMap, a given result size and a maxProportion for one
	 * class.
	 * 
	 * Steps (after each step values are normalized, i.e., they are changed so
	 * that the overall proportion adds up to 1.0) - Classes that are not in
	 * source sorted (did not make the minimum count) are filtered out - Classes
	 * that are larger then maxProportion are reduced to maxProportion - Classes
	 * that have proportion too small (their proportion does not compute to 1
	 * result given resultSize) are filtered out
	 * 
	 * Classes with small proportion are filtered out last. This gives them a
	 * change to reach at least 1 result
	 * 
	 * @param statistics
	 * @param valuePairsMap
	 * @param resultSize
	 * @param maxProportion
	 * @return
	 */
	public <T> Map<ClassPropertyId, Double> normalizeStatistics(Map<ClassPropertyId, Double> statistics, Map<ClassPropertyId, Map<InstanceId, T>> valuePairsMap,
			int resultSize, double maxProportion) {

		double minProportion = 1.0 / resultSize;

		AtomicDouble firstCorrection = new AtomicDouble(1.0);

		statistics = statistics.entrySet().stream().filter((e) -> {
			if (!valuePairsMap.containsKey(e.getKey())) {
				firstCorrection.addAndGet(-e.getValue());
				return false;
			} else {
				return true;
			}
		}).collect(Collectors.toMap(Entry::getKey, Entry::getValue));

		AtomicDouble secondCorrection = new AtomicDouble(1.0);

		statistics = statistics.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
			Double corrected = e.getValue() / firstCorrection.get();
			if (corrected > maxProportion) {
				secondCorrection.addAndGet(-(corrected - maxProportion));
				corrected = maxProportion;
			}
			return corrected;
		}));

		AtomicDouble thirdCorrection = new AtomicDouble(1.0);

		statistics = statistics.entrySet().stream().filter((e) -> {
			Double corrected = e.getValue() / secondCorrection.get();
			if (corrected < minProportion) {
				thirdCorrection.addAndGet(-corrected);
				return false;
			} else {
				return true;
			}

		}).collect(Collectors.toMap(Entry::getKey, (e) -> {
			Double corrected = e.getValue() / secondCorrection.get();
			return corrected;
		}));

		statistics = statistics.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
			Double corrected = e.getValue() / thirdCorrection.get();
			return corrected;
		}));

		return statistics;
	}

	/**
	 * Creates a valuePairMap based on a map of fusedValues
	 * 
	 * @param fusedValues
	 * @return
	 */
	public Map<ClassPropertyId, Map<InstanceId, PairData>> getValuePairsMap(Map<TripleId, FusedValue> fusedValues) {
		return getValuePairsMap(fusedValues, null);
	}

	/**
	 * Creates a valuePairMap based on a map of fusedValues but only when there
	 * is no value in DBpedia. Also ensures that map has a minSourceCount
	 * 
	 * @param fusedValues
	 * @return
	 */
	public Map<ClassPropertyId, Map<InstanceId, ValueData>> getValueMapNoOverlap(Map<TripleId, Object> fusedValues) {
		return getValueMapNoOverlap(fusedValues, null);
	}

	/**
	 * Creates a valuePairMap based on a map of fusedValues but only when there
	 * is no value in DBpedia. Also ensures that map has a minSourceCount
	 * 
	 * @param fusedValues
	 * @return
	 */
	public Map<ClassPropertyId, Map<InstanceId, ValueData>> getValueMapNoOverlap(Map<TripleId, Object> fusedValues,
			Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> minSourceCount) {

		ConcurrentHashMap<ClassPropertyId, Map<InstanceId, ValueData>> valueMap = new ConcurrentHashMap<>();

		Creator<Map<InstanceId, ValueData>> valuePairListCreator = () -> {
			return new HashMap<>();
		};

		PersistentParallel.runAndFinish(fusedValues.entrySet(), (e) -> {
			ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey());
			InstanceId instanceId = e.getKey().getInstanceId();
			if (minSourceCount == null || minSourceCount.containsKey(classPropertyId) && minSourceCount.get(classPropertyId).containsKey(instanceId)) {
				Object correctValue = dbPediaRepository.getValue(e.getKey());
				if (correctValue == null) {

					ValueData pairData = new ValueData(e.getKey(), e.getValue());
					Map<InstanceId, ValueData> list = CHMHelper.getFromCHM(valueMap, classPropertyId, valuePairListCreator);
					synchronized (list) {
						list.put(e.getKey().getInstanceId(), pairData);
					}
				}
			}
		} , "Group Values by Class Property Id");

		return valueMap;
	}

	/***
	 * Creates a valuePairMap based on a map of fusedValues and a map with a
	 * filtered (or not) source count
	 * 
	 * @param fusedValues
	 * @param minSourceCount
	 * @return
	 */
	public Map<ClassPropertyId, Map<InstanceId, PairData>> getValuePairsMap(Map<TripleId, FusedValue> fusedValues,
			Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> minSourceCount) {
		ConcurrentHashMap<ClassPropertyId, Map<InstanceId, PairData>> valuePairsMap = new ConcurrentHashMap<>();

		Creator<Map<InstanceId, PairData>> valuePairListCreator = () -> {
			return new HashMap<>();
		};

		PersistentParallel.runAndFinish(fusedValues.entrySet(), (e) -> {
			ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey());
			InstanceId instanceId = e.getKey().getInstanceId();
			if (minSourceCount == null || minSourceCount.containsKey(classPropertyId) && minSourceCount.get(classPropertyId).containsKey(instanceId)) {
				Object correctValue = dbPediaRepository.getValue(e.getKey());
				if (correctValue != null && !(correctValue instanceof List)) {
					ValuePair pair = new ValuePair(e.getValue().getSingle(), correctValue);

					PairData pairData = new PairData(e.getKey(), pair);
					Map<InstanceId, PairData> list = CHMHelper.getFromCHM(valuePairsMap, classPropertyId, valuePairListCreator);
					synchronized (list) {
						list.put(e.getKey().getInstanceId(), pairData);
					}
				}
			}
		} , "Group Values by Class Property Id");

		return valuePairsMap;
	}

	/**
	 * Return a list of pair data that samples data from the valuePairMap based
	 * on proportions provided in statistics and for a given resultSize using
	 * the values with highest number of source tables extracted from
	 * sourceSorting.
	 * 
	 * @param valuePairsMap
	 * @param statistics
	 * @param sourceSorted
	 * @param resultSize
	 * @return
	 */
	public <T> List<T> getPairData(Map<ClassPropertyId, Map<InstanceId, T>> valuePairsMap, Map<ClassPropertyId, Double> statistics,
			Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sourceSorted, int resultSize) {
		List<T> pairData = new ArrayList<>();

		double sum = 0.0;
		for (Entry<ClassPropertyId, Double> e : statistics.entrySet()) {
			ClassPropertyId classPropertyId = e.getKey();
			Double proportion = e.getValue();

			sum += proportion;

			int get = (int) (resultSize * proportion);

			Map<InstanceId, T> valuePairsData = valuePairsMap.get(classPropertyId);
			if (valuePairsData != null) {
				if (valuePairsData.size() < get) {
					for (T pd : valuePairsData.values()) {
						pairData.add(pd);
					}
				} else {

					LinkedHashMap<InstanceId, Integer> sortedInstances = sourceSorted.get(classPropertyId);

					int i = 0;
					for (Entry<InstanceId, Integer> instance : sortedInstances.entrySet()) {
						if (i == get)
							break;
						T pd = valuePairsData.get(instance.getKey());
						if (pd != null) {
							pairData.add(pd);
							i++;
						}

					}

				}

			}

		}

		System.out.println(sum);
		return pairData;
	}

	public Map<ClassPropertyId, Double> getStatistics(Map<TripleId, Object> fusedValues) {

		ConcurrentHashMap<ClassPropertyId, AtomicInteger> countsPerClass = new ConcurrentHashMap<>();

		PersistentParallel.runAndFinish(fusedValues.entrySet(), (e) -> {
			ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey());
			AtomicInteger count = CHMHelper.getFromCHM(countsPerClass, classPropertyId, () -> {
				return new AtomicInteger();
			});
			count.incrementAndGet();
		} , "Count Proportions of ClassPropertyId");

		int countTotal = 0;

		for (Entry<ClassPropertyId, AtomicInteger> e : countsPerClass.entrySet()) {
			countTotal += e.getValue().get();
		}

		Map<ClassPropertyId, Double> statistics = new HashMap<>();
		for (Entry<ClassPropertyId, AtomicInteger> e : countsPerClass.entrySet()) {

			double countPerClass = e.getValue().get();

			statistics.put(e.getKey(), countPerClass / countTotal);
		}

		return statistics;

	}

}
