package de.mannheim.uni.expansion.tablefusion.evaluation;

import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.function.Function;

import javax.inject.Inject;

import com.opencsv.CSVWriter;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.analysis.ValueEvaluator;
import de.mannheim.uni.expansion.tablefusion.evaluation.getters.ReadableGetter;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.PairData;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.ValueData;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;

public class EvaluationWriter {

	private final OntologyRepository ontologyRepository;

	private final ValueEvaluator accuracyCalculator;

	@Inject
	public EvaluationWriter(OntologyRepository ontologyRepository, ValueEvaluator accuracyCalculator) {
		super();
		this.ontologyRepository = ontologyRepository;
		this.accuracyCalculator = accuracyCalculator;
	}

	private final SimpleDateFormat dt = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
	private final DecimalFormat dfmt = new DecimalFormat(",000.0");

	public final ReadableGetter generalReadableGetter = (o) -> {

		Function<Object, String> getReadableSingle = (obj) -> {
			if (obj instanceof String) {
				return obj.toString();
			} else if (obj instanceof Date) {
				return dt.format((Date) obj);
			} else if (obj instanceof Double) {
				return dfmt.format((Double) obj);
			}
			throw new IllegalStateException("CAN'T RECOGNIZE THIS FORMAT: " + o.getClass() + "::" + o.toString());
		};

		if (o instanceof List) {
			List<String> newList = new ArrayList<>();
			for (Object obj : (List<?>) o) {
				newList.add(getReadableSingle.apply(obj));
			}
			return newList.toString();
		} else {
			return getReadableSingle.apply(o);
		}

	};

	public void write(String filePath, List<PairData> pairData) throws IOException {
		ReadableGetter readableGetter = generalReadableGetter;
		FileWriter fw = new FileWriter(filePath);
		CSVWriter csvWriter = new CSVWriter(fw);

		int columnsSize = 8;
		String[] headerLine = new String[columnsSize];

		headerLine[0] = "Type";
		headerLine[1] = "Class";
		headerLine[2] = "Property";
		headerLine[3] = "Instance";
		headerLine[4] = "DBpediaReadable";
		headerLine[5] = "FusedReadble";
		headerLine[6] = "FusionCorrect";
		headerLine[7] = "ManualCorrect";

		csvWriter.writeNext(headerLine);

		for (PairData vd : pairData) {
			String[] line = new String[columnsSize];
			line[0] = ontologyRepository.getDataType(new ClassPropertyId(vd.getTripleId())).toString();
			line[1] = vd.getTripleId().getClassId().getFinalPart();
			line[2] = vd.getTripleId().getPropertyId().getFinalPart();
			line[3] = vd.getTripleId().getInstanceId().getReadableString();
			line[4] = readableGetter.get(vd.getValuePair().getDBPediaValue());
			line[5] = readableGetter.get(vd.getValuePair().getFusedValue());
			if (accuracyCalculator.correct(vd.getTripleId(), FusedValue.fromObject(vd.getValuePair().getFusedValue()))) {
				line[6] = "1";
			} else {
				line[6] = "0";

			}
			line[7] = "";
			csvWriter.writeNext(line);
		}

		csvWriter.close();
	}

	void writeValueData(String filePath, List<ValueData> valueData) throws IOException {
		ReadableGetter readableGetter = generalReadableGetter;
		FileWriter fw = new FileWriter(filePath);
		CSVWriter csvWriter = new CSVWriter(fw);

		int columnsSize = 6;
		String[] headerLine = new String[columnsSize];

		headerLine[0] = "Type";
		headerLine[1] = "Class";
		headerLine[2] = "Property";
		headerLine[3] = "Instance";
		headerLine[4] = "FusedReadble";
		headerLine[5] = "ManualCorrect";

		csvWriter.writeNext(headerLine);

		for (ValueData vd : valueData) {
			String[] line = new String[columnsSize];
			line[0] = ontologyRepository.getDataType(new ClassPropertyId(vd.getTripleId())).toString();
			line[1] = vd.getTripleId().getClassId().getFinalPart();
			line[2] = vd.getTripleId().getPropertyId().getFinalPart();
			line[3] = vd.getTripleId().getInstanceId().getReadableString();
			line[4] = readableGetter.get(vd.getValue());
			line[5] = "";
			csvWriter.writeNext(line);
		}

		csvWriter.close();

	}
}
