package de.mannheim.uni.expansion.tablefusion.evaluation.analyzer;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.function.Consumer;

import javax.inject.Inject;
import javax.inject.Named;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.ListType;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.tablefusion.NamedProperties;
import de.mannheim.uni.expansion.tablefusion.adapters.learningsets.LearningSetsRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankRepository;
import de.mannheim.uni.expansion.tablefusion.adapters.ranking.RankRepositoryImpl;
import de.mannheim.uni.expansion.tablefusion.analysis.AccuracyCalculator;
import de.mannheim.uni.expansion.tablefusion.analysis.FilteredValues;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.FusionPerformance;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.InputProfileStoreCreator.InputProfileStore;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.PRFMPerformance;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.PerformancePR;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.performance.PERFMPerformanceI;
import de.mannheim.uni.expansion.tablefusion.analysis.writer.PerformanceResultsWriter;
import de.mannheim.uni.expansion.tablefusion.analysis.writer.ResultsCategory;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.scoring.ProbabilitiesCalculator;
import de.mannheim.uni.expansion.tablefusion.core.scoring.TimeStampsScoringCalculator;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ColumnScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.HostScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.InstanceScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.RegressionScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.RegressionScoresClassPropertyId;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.TableScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.thresholdpacks.ThresholdContainer;
import de.mannheim.uni.expansion.tablefusion.learning.LearningSetCreator.CreatedSets;
import de.mannheim.uni.expansion.tablefusion.learning.ListExpansionFilterCreator;
import de.mannheim.uni.expansion.tablefusion.optimization.ScoreFusionThresholdFinder;
import de.mannheim.uni.expansion.tablefusion.optimization.ScoreFusionThresholdFinderFull;
import de.mannheim.uni.expansion.tablefusion.services.GroupedValuesFilterer;
import de.mannheim.uni.expansion.tablefusion.services.ScoresFilterer;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionDataType;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionMethod;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionPreproccesser;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionRunner;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategy;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategyRepository;
import de.mannheim.uni.expansion.tablefusion.services.fusion.strategies.Scorer;
import de.mannheim.uni.expansion.tablefusion.services.fusion.strategies.ScorerRepository;
import de.mannheim.uni.expansion.tablefusion.services.grouping.GroupedValuesRepository;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.MKDIR;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class AccuracyAnalyzer {

	private final String resultsFolder;
	private final GroupedValuesRepository groupedValuesRepository;
	private final ScoreFusionThresholdFinder scoreFusionThresholdFinder;
	private final LearningSetsRepository learningSetsRepository;
	private final ProbabilitiesCalculator probabilitiesCalculator;
	private final FusionRunner fusionRunner;

	private final ScoreFusionThresholdFinderFull scoreFusionThresholdFinderUsingScorer;

	private final AccuracyCalculator accuracyCalculator;

	private final RankRepository rankRepository;

	private final OntologyRepository ontologyRepository;

	private final GroupedValuesFilterer listTypeFilterer;

	private final ListExpansionFilterCreator listExpansionFilterCreator;

	private final ScorerRepository scorerRepository;
	private final ScoresFilterer scoresFilterer;

	private final DBpediaRepositoryNewInterface dbpedia;

	@Inject
	public AccuracyAnalyzer(@Named(NamedProperties.RESULTS_FOLDER) String resultsFolder, GroupedValuesRepository groupedValuesRepository,
			ScoreFusionThresholdFinder scoreFusionThresholdFinder, LearningSetsRepository learningSetsRepository,
			ProbabilitiesCalculator probabilitiesCalculator, FusionRunner fusionRunner, AccuracyCalculator accuracyCalculator, RankRepository rankRepository,
			OntologyRepository ontologyRepository, GroupedValuesFilterer listTypeFilterer, ListExpansionFilterCreator listExpansionFilterCreator,
			ScorerRepository scorerRepository, ScoresFilterer scoresFilterer, ScoreFusionThresholdFinderFull scoreFusionThresholdFinderUsingScorer,
			DBpediaRepositoryNewInterface dbpedia) {
		super();
		this.resultsFolder = resultsFolder;
		this.groupedValuesRepository = groupedValuesRepository;
		this.scoreFusionThresholdFinder = scoreFusionThresholdFinder;
		this.learningSetsRepository = learningSetsRepository;
		this.probabilitiesCalculator = probabilitiesCalculator;
		this.fusionRunner = fusionRunner;
		this.accuracyCalculator = accuracyCalculator;
		this.rankRepository = rankRepository;
		this.ontologyRepository = ontologyRepository;
		this.listTypeFilterer = listTypeFilterer;
		this.listExpansionFilterCreator = listExpansionFilterCreator;
		this.scorerRepository = scorerRepository;
		this.scoresFilterer = scoresFilterer;
		this.scoreFusionThresholdFinderUsingScorer = scoreFusionThresholdFinderUsingScorer;
		this.dbpedia = dbpedia;
	}

	public void getPerformance(String dataset) throws IOException {
		String folderPath = resultsFolder + "/" + "timestamps";
		MKDIR.createDirUnchecked(new File(folderPath));

		dfmt.applyPattern(pattern);

		List<FusionDataType> methodTypes = new ArrayList<>();
		methodTypes.add(FusionDataType.REFERENCE);
		methodTypes.add(FusionDataType.STRING);
		methodTypes.add(FusionDataType.NUMERIC);
		methodTypes.add(FusionDataType.DATE);

		ArrayList<FusionStrategy> direct = new ArrayList<>();
		ArrayList<FusionStrategy> cv = new ArrayList<>();
		ArrayList<FusionStrategy> leNormal = new ArrayList<>();
		ArrayList<FusionStrategy> leCV = new ArrayList<>();

		Consumer<FusionStrategy> addStrategy = (fs) -> {

			switch (fs.geTestingMethod()) {
			case CROSS_VALIDATED:
				cv.add(fs);
				break;
			case DIRECT:
				direct.add(fs);
				break;
			case LIST_EXPANSION_DIRECT:
				leNormal.add(fs);
				break;
			case LIST_EXPANSION_CROSS_VALIDATED:
				leCV.add(fs);
				break;

			}

		};

		addStrategy.accept(FusionStrategyRepository.NORMAL);
		// // addStrategy.accept(FusionStrategyRepository.RANDOM);
		//
		// addStrategy.accept(FusionStrategyRepository.KBT);
		addStrategy.accept(FusionStrategyRepository.KBT_ClassPropertyId_Threshold);
		// addStrategy.accept(FusionStrategyRepository.KBT_PropertyId_Threshold);
		// addStrategy.accept(FusionStrategyRepository.Regression);
		// addStrategy.accept(FusionStrategyRepository.Regression_Propagated);
		// addStrategy.accept(FusionStrategyRepository.Regression_Multiplied_Propagated);
		// addStrategy.accept(FusionStrategyRepository.Regression_Single_Propagated);

		addStrategy.accept(FusionStrategyRepository.Regression_ClassProperty);
		addStrategy.accept(FusionStrategyRepository.Regression_ClassProperty_Propagated);
		// addStrategy.accept(FusionStrategyRepository.Regression_ClassProperty_Multiplied_Propagated);
		// addStrategy.accept(FusionStrategyRepository.Regression_ClassProperty_Single_Propagated);
		// addStrategy.accept(FusionStrategyRepository.Regression_PropertyId);
		// // //
		// addStrategy.accept(FusionStrategyRepository.RegressionTrees);
		//
		// addStrategy.accept(FusionStrategyRepository.RegressionTrees_ClassProperty);
		// //
		// addStrategy.accept(FusionStrategyRepository.RegressionTrees_PropertyId);
		// //
		// addStrategy.accept(FusionStrategyRepository.Timestamps_Regression);
		addStrategy.accept(FusionStrategyRepository.Timestamps_Regression_ClassProperty);
		// addStrategy.accept(FusionStrategyRepository.Timestamps_Regression_Propagated);
		addStrategy.accept(FusionStrategyRepository.Timestamps_Regression_ClassProperty_Propagated);
		// addStrategy.accept(FusionStrategyRepository.Timestamps_Regression_PropertyId);
		// addStrategy.accept(FusionStrategyRepository.Combined_HierarchyTS_TableText);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_Combined_HierarchyTS_TableText_Weighted);
		// addStrategy.accept(FusionStrategyRepository.KBT_Combined_HierarchyTS_TableText_Multiplied);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_Combined_HierarchyTS_TableText_WeightedMultiplied);

		// addStrategy.accept(FusionStrategyRepository.PageRank);
		// addStrategy.accept(FusionStrategyRepository.PageRank_Regression);
		addStrategy.accept(FusionStrategyRepository.PageRank_Regression_ClassProperty);

		addStrategy.accept(FusionStrategyRepository.PageRank_TS_ClassProperty);
		addStrategy.accept(FusionStrategyRepository.PageRank_TS_Propagated_ClassProperty);
		// addStrategy.accept(FusionStrategyRepository.PageRank_KBT_Regression);
		addStrategy.accept(FusionStrategyRepository.PageRank_KBT_Regression_ClassProperty);
		addStrategy.accept(FusionStrategyRepository.FullRegression);
		addStrategy.accept(FusionStrategyRepository.FullRegressionPropagated);

		// addStrategy.accept(FusionStrategyRepository.CellTS);
		// addStrategy.accept(FusionStrategyRepository.CellTS_Propagated);
		//
		// addStrategy.accept(FusionStrategyRepository.ColumnTS);
		// addStrategy.accept(FusionStrategyRepository.ColumnTS_Propagated);
		// addStrategy.accept(FusionStrategyRepository.KBT_ColumnTS_Max_Weighted);
		// addStrategy.accept(FusionStrategyRepository.KBT_ColumnTS_Max_Multiplied);
		// //
		// addStrategy.accept(FusionStrategyRepository.TableTextTS);
		// addStrategy.accept(FusionStrategyRepository.TableTextTS_Propagated);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_TableTextTS_MostCommon_Weighted);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_TableTextTS_MostCommon_Multiplied);
		// //
		// addStrategy.accept(FusionStrategyRepository.HierarchyTS);
		// addStrategy.accept(FusionStrategyRepository.HierarchyTS_Propagated);
		// addStrategy.accept(FusionStrategyRepository.KBT_HierarchyTS_Weighted);
		// addStrategy.accept(FusionStrategyRepository.KBT_HierarchyTS_Multiplied);
		// //
		// addStrategy.accept(FusionStrategyRepository.FullHierarchyTS);
		// addStrategy.accept(FusionStrategyRepository.FullHierarchyTS_Propagated);
		// addStrategy.accept(FusionStrategyRepository.KBT_FullHierarchyTS_Weighted);
		// addStrategy.accept(FusionStrategyRepository.KBT_FullHierarchyTS_Multiplied);
		// //
		// addStrategy.accept(FusionStrategyRepository.PageTitleTs);
		// addStrategy.accept(FusionStrategyRepository.PageTitleTs_Propagated);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_PageTitleTS_Weighted);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_PageTitleTS_Multiplied);
		// //
		// addStrategy.accept(FusionStrategyRepository.TableTitleTS);
		// addStrategy.accept(FusionStrategyRepository.TableTitleTS_Propagated);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_TableTitleTS_Weighted);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_TableTitleTS_Multiplied);
		// //
		// addStrategy.accept(FusionStrategyRepository.TableYearsHierarchy);
		// addStrategy.accept(FusionStrategyRepository.TableYearsHierarchy_Propagated);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_TableYearsHierarch_Weighted);
		// // //
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_TableYearsHierarch_Resized);
		// //
		// addStrategy.accept(FusionStrategyRepository.KBT_TableYearsHierarch_Multiplied);

		// addStrategy.accept(FusionStrategyRepository.CS_WEIGHTED);
		// addStrategy.accept(FusionStrategyRepository.CS_LIST_EXPANSION);
		// leNormal.add(FusionStrategyRepository.NORMAL);

		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();

		Set<PropertyId> propertySet = new HashSet<>();
		// propertySet.add(new
		// PropertyId("http://dbpedia.org/ontology/capital"));
		// propertySet.add(new
		// PropertyId("http://dbpedia.org/ontology/PopulatedPlace/areaTotal"));
		// propertySet.add(new
		// PropertyId("http://dbpedia.org/ontology/populationTotal"));
		// propertySet.add(new
		// PropertyId("http://dbpedia.org/ontology/populationDensity"));

		// groupedValues = this.listTypeFilterer.getByProperty(propertySet,
		// groupedValues);
		for (ClassId classId : this.dbpedia.getClasses()) {
			propertySet.addAll(this.dbpedia.getProperties(classId).keySet());
		}

		groupedValues = this.listTypeFilterer.getByProperty(propertySet, groupedValues);

		PerformanceResultsWriter performanceResultsWriter = new PerformanceResultsWriter();

		analyzeAccuracy(dataset, folderPath, performanceResultsWriter, "", methodTypes, groupedValues, direct, new ArrayList<>(), cv, new ArrayList<>(),
				new ArrayList<>(), false, true);

		System.out.println("Original: " + TimeStampsScoringCalculator.original.toString());
		System.out.println("Propogated: " + TimeStampsScoringCalculator.propogated.toString());

		System.out.println("PageRankTotal: " + RankRepositoryImpl.countTotal.get());
		System.out.println("PageRankMissing: " + RankRepositoryImpl.countMissing.get());

		// getPerformanceHierarchyTimeStampsFiltered();
		// getPerformanceTimeStampsFilteredColumnScores();
	}

	private void getPerformanceHierarchyTimeStampsFiltered() throws IOException {
		String folderPath = resultsFolder + "/" + "timestampsHiearchyFiltered";
		MKDIR.createDirUnchecked(new File(folderPath));

		dfmt.applyPattern(pattern);

		List<FusionDataType> methodTypes = new ArrayList<>();
		methodTypes.add(FusionDataType.REFERENCE);
		methodTypes.add(FusionDataType.STRING);
		methodTypes.add(FusionDataType.NUMERIC);
		methodTypes.add(FusionDataType.DATE);

		ArrayList<FusionStrategy> direct = new ArrayList<>();
		ArrayList<FusionStrategy> cv = new ArrayList<>();
		ArrayList<FusionStrategy> leNormal = new ArrayList<>();
		ArrayList<FusionStrategy> leCV = new ArrayList<>();

		Consumer<FusionStrategy> addStrategy = (fs) -> {

			switch (fs.geTestingMethod()) {
			case CROSS_VALIDATED:
				cv.add(fs);
				break;
			case DIRECT:
				direct.add(fs);
				break;
			case LIST_EXPANSION_DIRECT:
				leNormal.add(fs);
				break;
			case LIST_EXPANSION_CROSS_VALIDATED:
				leCV.add(fs);
				break;

			}

		};

		addStrategy.accept(FusionStrategyRepository.NORMAL);
		// addStrategy.accept(FusionStrategyRepository.RANDOM);
		addStrategy.accept(FusionStrategyRepository.KBT);
		addStrategy.accept(FusionStrategyRepository.KBT_ClassPropertyId_Threshold);
		addStrategy.accept(FusionStrategyRepository.KBT_PropertyId_Threshold);
		addStrategy.accept(FusionStrategyRepository.Regression);
		addStrategy.accept(FusionStrategyRepository.Regression_ClassProperty);
		addStrategy.accept(FusionStrategyRepository.Regression_PropertyId);

		addStrategy.accept(FusionStrategyRepository.HierarchyTS);
		addStrategy.accept(FusionStrategyRepository.KBT_HierarchyTS_Weighted);
		// addStrategy.accept(FusionStrategyRepository.KBT_HierarchyTS_Resized);
		addStrategy.accept(FusionStrategyRepository.KBT_HierarchyTS_Multiplied);
		// addStrategy.accept(FusionStrategyRepository.CS_WEIGHTED);
		// addStrategy.accept(FusionStrategyRepository.CS_LIST_EXPANSION);
		// leNormal.add(FusionStrategyRepository.NORMAL);

		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();
		groupedValues = scoresFilterer.filterByHiearicalTimeStamps(groupedValues);
		PerformanceResultsWriter performanceResultsWriter = new PerformanceResultsWriter();

		analyzeAccuracy("", folderPath, performanceResultsWriter, "HierarchyFiltered_", methodTypes, groupedValues, direct, new ArrayList<>(), cv,
				new ArrayList<>(), new ArrayList<>(), false, true);

	}

	private void getPerformanceTimeStampsFilteredColumnScores() throws IOException {
		String folderPath = resultsFolder + "/" + "timestampsColumnFiltered";
		MKDIR.createDirUnchecked(new File(folderPath));

		dfmt.applyPattern(pattern);

		List<FusionDataType> methodTypes = new ArrayList<>();
		methodTypes.add(FusionDataType.REFERENCE);
		methodTypes.add(FusionDataType.STRING);
		methodTypes.add(FusionDataType.NUMERIC);
		methodTypes.add(FusionDataType.DATE);

		ArrayList<FusionStrategy> direct = new ArrayList<>();
		ArrayList<FusionStrategy> cv = new ArrayList<>();
		ArrayList<FusionStrategy> leNormal = new ArrayList<>();
		ArrayList<FusionStrategy> leCV = new ArrayList<>();

		Consumer<FusionStrategy> addStrategy = (fs) -> {

			switch (fs.geTestingMethod()) {
			case CROSS_VALIDATED:
				cv.add(fs);
				break;
			case DIRECT:
				direct.add(fs);
				break;
			case LIST_EXPANSION_DIRECT:
				leNormal.add(fs);
				break;
			case LIST_EXPANSION_CROSS_VALIDATED:
				leCV.add(fs);
				break;

			}

		};

		addStrategy.accept(FusionStrategyRepository.NORMAL);
		// addStrategy.accept(FusionStrategyRepository.RANDOM);
		addStrategy.accept(FusionStrategyRepository.KBT);
		addStrategy.accept(FusionStrategyRepository.KBT_ClassPropertyId_Threshold);
		addStrategy.accept(FusionStrategyRepository.KBT_PropertyId_Threshold);
		addStrategy.accept(FusionStrategyRepository.Regression);
		addStrategy.accept(FusionStrategyRepository.Regression_ClassProperty);
		addStrategy.accept(FusionStrategyRepository.Regression_PropertyId);
		addStrategy.accept(FusionStrategyRepository.ColumnTS);
		addStrategy.accept(FusionStrategyRepository.KBT_ColumnTS_Max_Weighted);
		//
		addStrategy.accept(FusionStrategyRepository.KBT_ColumnTS_Max_Resized);
		addStrategy.accept(FusionStrategyRepository.KBT_ColumnTS_Max_Multiplied);
		// // addStrategy.accept(FusionStrategyRepository.CS_WEIGHTED);
		// // addStrategy.accept(FusionStrategyRepository.CS_LIST_EXPANSION);
		// // leNormal.add(FusionStrategyRepository.NORMAL);

		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();
		groupedValues = scoresFilterer.filterByColumnTimeStamps(groupedValues);
		PerformanceResultsWriter performanceResultsWriter = new PerformanceResultsWriter();

		analyzeAccuracy("", folderPath, performanceResultsWriter, "ColumnFiltered_", methodTypes, groupedValues, direct, new ArrayList<>(), cv,
				new ArrayList<>(), new ArrayList<>(), false, true);

	}

	public void getPerformanceWithLists() throws IOException {
		String folderPath = resultsFolder + "/" + "lists";
		MKDIR.createDirUnchecked(new File(folderPath));

		dfmt.applyPattern(pattern);

		List<FusionDataType> methodTypes = new ArrayList<>();
		methodTypes.add(FusionDataType.REFERENCE);
		methodTypes.add(FusionDataType.STRING);
		methodTypes.add(FusionDataType.NUMERIC);
		methodTypes.add(FusionDataType.DATE);
		methodTypes.add(FusionDataType.LINK);

		ArrayList<FusionStrategy> direct = new ArrayList<>();
		ArrayList<FusionStrategy> cv = new ArrayList<>();
		ArrayList<FusionStrategy> leNormal = new ArrayList<>();
		ArrayList<FusionStrategy> leCV = new ArrayList<>();

		Consumer<FusionStrategy> addStrategy = (fs) -> {

			switch (fs.geTestingMethod()) {
			case CROSS_VALIDATED:
				cv.add(fs);
				break;
			case DIRECT:
				direct.add(fs);
				break;
			case LIST_EXPANSION_DIRECT:
				leNormal.add(fs);
				break;
			case LIST_EXPANSION_CROSS_VALIDATED:
				leCV.add(fs);
				break;

			}

		};

		addStrategy.accept(FusionStrategyRepository.NORMAL);
		// addStrategy.accept(FusionStrategyRepository.RANDOM);
		addStrategy.accept(FusionStrategyRepository.KBT);
		// addStrategy.accept(FusionStrategyRepository.CS_WEIGHTED);
		addStrategy.accept(FusionStrategyRepository.CS_LIST_EXPANSION);
		leNormal.add(FusionStrategyRepository.NORMAL);

		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();

		// mediumListTypes = listTypeFilterer.getDataType(ColumnDataType.string,
		// mediumListTypes);
		PerformanceResultsWriter performanceResultsWriter = new PerformanceResultsWriter();
		GroupedValues nonListTypes = listTypeFilterer.getNonListTypes(groupedValues);
		analyzeAccuracy("", folderPath, performanceResultsWriter, "NON-Lists_", methodTypes, nonListTypes, direct, new ArrayList<>(), cv, new ArrayList<>(),
				new ArrayList<>(), true, true);
		//
		// Map<TripleId, MatchedValues> shortListTypes =
		// listTypeFilterer.getListTypes(ListType.SHORT, groupedValues);
		// analyzeAccuracy("ShortLists-Creation_", methodTypes, shortListTypes,
		// direct, cv, new ArrayList<>(), new ArrayList<>(), false);
		//
		GroupedValues mediumListTypes = listTypeFilterer.getListTypes(ListType.MEDIUM, groupedValues);
		// mediumListTypes =
		// listTypeFilterer.getDataType(ColumnDataType.reference,
		// mediumListTypes);
		analyzeAccuracy("", folderPath, performanceResultsWriter, "MediumLists-Creation_", methodTypes, mediumListTypes, direct, new ArrayList<>(), cv,
				new ArrayList<>(), new ArrayList<>(), true, true);
		analyzeAccuracy("", folderPath, performanceResultsWriter, "MediumLists-Expansion_", methodTypes, mediumListTypes, new ArrayList<>(), new ArrayList<>(),
				new ArrayList<>(), leNormal, leCV, true, true);
		//
		GroupedValues longListTypes = listTypeFilterer.getListTypes(ListType.LONG, groupedValues);
		analyzeAccuracy("", folderPath, performanceResultsWriter, "LongLists-Creation_", methodTypes, longListTypes, direct, new ArrayList<>(), cv,
				new ArrayList<>(), new ArrayList<>(), true, true);
		analyzeAccuracy("", folderPath, performanceResultsWriter, "LongLists-Expansion_", methodTypes, longListTypes, new ArrayList<>(), new ArrayList<>(),
				new ArrayList<>(), leNormal, leCV, true, true);

		GroupedValues allListsType = listTypeFilterer.getListTypes(groupedValues);
		analyzeAccuracy("", folderPath, performanceResultsWriter, "AllLists-Creation_", methodTypes, allListsType, direct, new ArrayList<>(), cv,
				new ArrayList<>(), new ArrayList<>(), true, true);
		analyzeAccuracy("", folderPath, performanceResultsWriter, "AllLists-Expansion_", methodTypes, allListsType, new ArrayList<>(), new ArrayList<>(),
				new ArrayList<>(), leNormal, leCV, true, true);
	}

	public void getPerformanceWWW() throws IOException {

		String folderPath = resultsFolder + "/" + "www";

		MKDIR.createDirUnchecked(new File(folderPath));

		dfmt.applyPattern(pattern);

		List<FusionDataType> methodTypes = new ArrayList<>();
		methodTypes.add(FusionDataType.REFERENCE);
		methodTypes.add(FusionDataType.STRING);
		methodTypes.add(FusionDataType.NUMERIC);
		methodTypes.add(FusionDataType.DATE);

		ArrayList<FusionStrategy> direct = new ArrayList<>();
		ArrayList<FusionStrategy> directScored = new ArrayList<>();
		ArrayList<FusionStrategy> cv = new ArrayList<>();
		ArrayList<FusionStrategy> leNormal = new ArrayList<>();
		ArrayList<FusionStrategy> leCV = new ArrayList<>();

		Consumer<FusionStrategy> addStrategy = (fs) -> {

			switch (fs.geTestingMethod()) {
			case CROSS_VALIDATED:
				cv.add(fs);
				break;
			case DIRECT:
				if (fs.getFusionMethod() == FusionMethod.SCORED) {
					directScored.add(fs);
				} else {

					direct.add(fs);
				}
				break;
			case LIST_EXPANSION_DIRECT:
				leNormal.add(fs);
				break;
			case LIST_EXPANSION_CROSS_VALIDATED:
				leCV.add(fs);
				break;

			}

		};

		addStrategy.accept(FusionStrategyRepository.NORMAL);
		// addStrategy.accept(FusionStrategyRepository.RANDOM);
		addStrategy.accept(FusionStrategyRepository.KBT);
		// addStrategy.accept(FusionStrategyRepository.CS_WEIGHTED);
		addStrategy.accept(FusionStrategyRepository.PR_PROBABILITIES);

		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();

		// mediumListTypes = listTypeFilterer.getDataType(ColumnDataType.string,
		// mediumListTypes);
		PerformanceResultsWriter performanceResultsWriter = new PerformanceResultsWriter();
		analyzeAccuracy("", folderPath, performanceResultsWriter, "Fusion_", methodTypes, groupedValues, direct, directScored, cv, new ArrayList<>(),
				new ArrayList<>(), false, true);
	}

	void analyzeAccuracy(String dataset, String folderPath, PerformanceResultsWriter performanceResultsWriter, String prefix, List<FusionDataType> methodTypes,
			GroupedValues groupedValues, ArrayList<FusionStrategy> direct, ArrayList<FusionStrategy> directScored, ArrayList<FusionStrategy> cv,
			ArrayList<FusionStrategy> leNormal, ArrayList<FusionStrategy> leCV, boolean fuseLists, boolean write) throws IOException {

		for (FusionStrategy fs : direct) {
			getPerformance(dataset, prefix, performanceResultsWriter, methodTypes, groupedValues, null, fs, fuseLists);
		}

		for (FusionStrategy fs : directScored) {
			getPerformanceWithScores(dataset, performanceResultsWriter, methodTypes, groupedValues, fs, null, fs.getName(), fuseLists);
		}

		if (!cv.isEmpty()) {
			int length = cv.size();
			getPerformanceWithCrossValidation(dataset, prefix, performanceResultsWriter, methodTypes, groupedValues, null,
					cv.toArray(new FusionStrategy[length]), fuseLists);
		}

		if (!leNormal.isEmpty()) {
			testListExpansionNormal(dataset, prefix, performanceResultsWriter, methodTypes, groupedValues, null,
					leNormal.toArray(new FusionStrategy[leNormal.size()]), fuseLists);
		}

		if (!leCV.isEmpty()) {
			testListExpansionCrossValidation(dataset, prefix, performanceResultsWriter, methodTypes, groupedValues, null,
					leCV.toArray(new FusionStrategy[leCV.size()]), fuseLists);
		}

		if (write) {
			for (ResultsCategory rc : ResultsCategory.values()) {
				FileWriter fw = new FileWriter(folderPath + "/" + rc.name() + ".csv", true);
				fw.write(performanceResultsWriter.getNoHeader(rc, 0));
				fw.close();

			}
		}
	}

	Locale locale = new Locale("en", "UK");
	String pattern = "###.##";
	DecimalFormat dfmt = (DecimalFormat) NumberFormat.getNumberInstance(locale);

	public void getPerformanceWithCrossValidation(String dataset, String prefix, PerformanceResultsWriter performanceResultsWriter,
			List<FusionDataType> methodTypes, GroupedValues groupedValues, InputProfileStore inputProfiledData, FusionStrategy[] fusionRunTypes,
			boolean fuseLists) throws IOException {

		int length = fusionRunTypes.length;

		Map<Integer, CreatedSets> createdSetsMap = learningSetsRepository.getCreatedSets(4, groupedValues);

		// ThresholdContainer[] probabilityThresholds = new
		// ThresholdContainer[length];
		Scorer[] scorers = new Scorer[length];
		String[] names = new String[length];
		{
			int i = 0;
			for (FusionStrategy strategy : fusionRunTypes) {
				scorers[i] = this.scorerRepository.get(strategy.getScoringMethod());
				// ThresholdContainer threshold =
				// scoreFusionThresholdFinder.findOptimumThresholdCrossValidation(createdSetsMap,
				// strategy, false);
				// probabilityThresholds[i] = threshold;
				names[i] = prefix + strategy.getName();

				i++;
			}
		}

		System.out.println();
		System.out.println();
		// Match and group all the values

		List<FusionDataType> dataTypes = new ArrayList<>();
		dataTypes.addAll(methodTypes);

		List<FusionPreproccesser> preprocessors = new ArrayList<>();
		preprocessors.add(FusionPreproccesser.NONE);

		@SuppressWarnings("unchecked")
		List<FusionPerformance<PerformancePR>>[] performanceList = new List[length];

		for (int i = 0; i < length; i++) {
			performanceList[i] = new ArrayList<>();
		}

		Map<FusionStrategy, List<RegressionScores>> rsListMap = new HashMap<>();
		Map<FusionStrategy, List<RegressionScoresClassPropertyId>> rsCPListMap = new HashMap<>();

		for (Entry<Integer, CreatedSets> createdSets : createdSetsMap.entrySet()) {

			for (int i = 0; i < length; i++) {

				FusionStrategy fusionRunType = fusionRunTypes[i];
				Scorer scoresGetter = scorers[i];

				ScoresContainer probabilities = scoresGetter.getScores(groupedValues, createdSets.getValue().getLearningSet(),
						createdSets.getValue().getTestingSet());

				if (probabilities instanceof RegressionScores) {
					List<RegressionScores> rsList = CHMHelper.getFromMap(rsListMap, fusionRunType, () -> new ArrayList<>());
					rsList.add((RegressionScores) probabilities);
				}

				if (probabilities instanceof RegressionScoresClassPropertyId) {
					List<RegressionScoresClassPropertyId> rsCPList = CHMHelper.getFromMap(rsCPListMap, fusionRunType, () -> new ArrayList<>());
					rsCPList.add((RegressionScoresClassPropertyId) probabilities);
				}

				ThresholdContainer probabilityThreshold = scoreFusionThresholdFinderUsingScorer.findOptimumThresholdCrossValidation(
						createdSets.getValue().getLearningSet(), probabilities, fusionRunType.getThresholdType(), dataTypes, preprocessors, fusionRunType,
						fuseLists);

				Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType,
						createdSets.getValue().getTestingSet(), probabilities, fuseLists, probabilityThreshold);

				Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

				FusionPerformance<PerformancePR> fusionPerformance = accuracyCalculator.analyzeAccuracy(createdSets.getValue().getTestingSet(), fusedValues,
						fuseLists);
				performanceList[i].add(fusionPerformance);
			}

		}

		for (Entry<FusionStrategy, List<RegressionScores>> e : rsListMap.entrySet()) {
			List<RegressionScores> rsList = e.getValue();
			Double intercept = rsList.get(0).getIntercept();
			Map<String, Double> weights = rsList.get(0).getWeights();
			for (int i = 1; i < rsList.size(); i++) {
				Map<String, Double> weights2 = rsList.get(i).getWeights();

				for (Entry<String, Double> entry : weights.entrySet()) {
					entry.setValue(entry.getValue() + weights2.get(entry.getKey()));
				}

				intercept += rsList.get(i).getIntercept();
			}

			System.out.println(e.getKey().getName());
			System.out.println(" - INTERCEPT: " + (intercept / rsList.size()));
			for (Entry<String, Double> entry : weights.entrySet()) {
				System.out.println(" - " + entry.getKey() + ": " + (entry.getValue() / rsList.size()));
			}
		}

		for (Entry<FusionStrategy, List<RegressionScoresClassPropertyId>> e : rsCPListMap.entrySet()) {
			List<RegressionScoresClassPropertyId> rsList = e.getValue();
			System.out.println(e.getKey().getName());
			Map<ClassPropertyId, Map<String, Double>> weightsMap = new HashMap<>(rsList.get(0).getWeightsMap());
			Map<ClassPropertyId, Double> interceptsMap = new HashMap<>(rsList.get(0).getInterceptsMap());

			for (Entry<ClassPropertyId, Map<String, Double>> cpEntry : weightsMap.entrySet()) {
				Map<String, Double> weights = cpEntry.getValue();

				for (int i = 1; i < rsList.size(); i++) {
					Map<String, Double> weights2 = rsList.get(i).getWeights(cpEntry.getKey());

					for (Entry<String, Double> entry : weights.entrySet()) {
						entry.setValue(entry.getValue() + weights2.get(entry.getKey()));
					}

					interceptsMap.put(cpEntry.getKey(), interceptsMap.get(cpEntry.getKey()) + rsList.get(i).getIntercept(cpEntry.getKey()));
				}

				System.out.println(" - " + cpEntry.getKey());
				System.out.println(" - INTERCEPT: " + (interceptsMap.get(cpEntry.getKey()) / rsList.size()));
				for (Entry<String, Double> entry : weights.entrySet()) {
					System.out.println(" -  - " + entry.getKey() + ": " + (entry.getValue() / rsList.size()));
				}
			}

		}
		for (int i = 0; i < length; i++) {
			FusionStrategy fusionRunType = fusionRunTypes[i];
			Scorer scoresGetter = scorers[i];

			ScoresContainer probabilities = scoresGetter.getScores(groupedValues, groupedValues, groupedValues);

			ThresholdContainer probabilityThreshold = scoreFusionThresholdFinderUsingScorer.findOptimumThresholdCrossValidation(groupedValues, probabilities,
					fusionRunType.getThresholdType(), dataTypes, preprocessors, fusionRunType, fuseLists);

			Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType, groupedValues, probabilities,
					fuseLists, probabilityThreshold);

			Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

			FusionPerformance<PerformancePR> performance = accuracyCalculator.analyzeAccuracy(groupedValues, fusedValues, fuseLists);

			FusionPerformance<PRFMPerformance> averagePerformance = accuracyCalculator.averageAccuracies(performanceList[i]);

			System.out.println();
			System.out.println();
			System.out.println();
			System.out.println();

			// System.out.println("Threshold" + probabilityThreshold);

			System.out.println(names[i]);

			System.out.println();
			writePerformance(performanceResultsWriter, dataset, names[i], performance, averagePerformance, true, true, false, false);

		}

	}

	private void testListExpansionCrossValidation(String dataset, String prefix, PerformanceResultsWriter performanceResultsWriter,
			List<FusionDataType> methodTypes, GroupedValues groupedValues, Object object, FusionStrategy[] fusionRunTypes, boolean fuseLists)
					throws IOException {

		Map<Integer, FilteredValues> setsFilters = listExpansionFilterCreator.createByKBList(8);

		int length = fusionRunTypes.length;

		ThresholdContainer[] probabilityThresholds = new ThresholdContainer[length];
		Scorer[] scorers = new Scorer[length];
		String[] names = new String[length];
		{
			int i = 0;
			for (FusionStrategy strategy : fusionRunTypes) {
				scorers[i] = this.scorerRepository.get(strategy.getScoringMethod());
				ThresholdContainer threshold = scoreFusionThresholdFinder.findOptimumThresholdListExpansion(setsFilters, groupedValues, strategy, false);
				probabilityThresholds[i] = threshold;
				names[i] = prefix + "_" + strategy.getName() + "_" + dfmt.format(threshold) + "_THRESHOLD";

				i++;
			}
		}

		System.out.println();
		System.out.println();
		// Match and group all the values

		List<FusionDataType> dataTypes = new ArrayList<>();
		dataTypes.addAll(methodTypes);

		List<FusionPreproccesser> preprocessors = new ArrayList<>();
		preprocessors.add(FusionPreproccesser.NONE);

		@SuppressWarnings("unchecked")
		List<FusionPerformance<PerformancePR>>[] performanceList = new List[length];

		for (int i = 0; i < length; i++) {
			performanceList[i] = new ArrayList<>();
		}

		for (Entry<Integer, FilteredValues> setsFilter : setsFilters.entrySet()) {

			GroupedValues filteredGroupedValues = listTypeFilterer.filterBySetFilter(setsFilter.getValue(), groupedValues);

			for (int i = 0; i < length; i++) {
				FusionStrategy fusionRunType = fusionRunTypes[i];
				Scorer scoresGetter = scorers[i];
				ThresholdContainer probabilityThreshold = probabilityThresholds[i];

				ScoresContainer probabilities = scoresGetter.getScores(filteredGroupedValues, null, null);

				Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType, groupedValues,
						probabilities, fuseLists, probabilityThreshold);

				Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

				FusionPerformance<PerformancePR> fusionPerformance = accuracyCalculator.analyzeAccuracy(groupedValues, fusedValues, fuseLists,
						setsFilter.getValue());
				performanceList[i].add(fusionPerformance);
			}

		}

		for (int i = 0; i < length; i++) {
			FusionStrategy fusionRunType = fusionRunTypes[i];
			Scorer scoresGetter = scorers[i];
			ThresholdContainer probabilityThreshold = probabilityThresholds[i];

			ScoresContainer probabilities = scoresGetter.getScores(groupedValues, null, null);

			Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType, groupedValues, probabilities,
					fuseLists, probabilityThreshold);

			Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

			FusionPerformance<PerformancePR> performance = accuracyCalculator.analyzeAccuracy(groupedValues, fusedValues, fuseLists);

			FusionPerformance<PRFMPerformance> averagePerformance = accuracyCalculator.averageAccuracies(performanceList[i]);

			System.out.println();
			System.out.println();
			System.out.println();
			System.out.println();

			// System.out.println("Threshold" + probabilityThreshold);

			System.out.println(names[i]);

			System.out.println();
			writePerformance(performanceResultsWriter, dataset, names[i], performance, averagePerformance, true, true, false, false);

		}

	}

	private void testListExpansionNormal(String dataset, String prefix, PerformanceResultsWriter performanceResultsWriter, List<FusionDataType> methodTypes,
			GroupedValues groupedValues, Object object, FusionStrategy[] fusionRunTypes, boolean fuseLists) throws IOException {

		Map<Integer, FilteredValues> setsFilters = listExpansionFilterCreator.createByKBList(5);

		int length = fusionRunTypes.length;

		// Double[] probabilityThresholds = new Double[length];
		// ScoresGetter[] scoresGetters = new ScoresGetter[length];
		String[] names = new String[length];
		{
			int i = 0;
			for (FusionStrategy strategy : fusionRunTypes) {
				// scoresGetters[i] = strategy.getScoresGetter();
				// double threshold =
				// scoreFusionThresholdFinder.findOptimumThresholdListExpansion(setsFilters,
				// groupedValues, strategy, scoresGetters[i], false);
				// probabilityThresholds[i] = threshold;
				names[i] = prefix + "_" + strategy.getName() + "LIST_TESTING";

				i++;
			}
		}

		System.out.println();
		System.out.println();
		// Match and group all the values

		List<FusionDataType> dataTypes = new ArrayList<>();
		dataTypes.addAll(methodTypes);

		List<FusionPreproccesser> preprocessors = new ArrayList<>();
		preprocessors.add(FusionPreproccesser.NONE);

		@SuppressWarnings("unchecked")
		List<FusionPerformance<PerformancePR>>[] performanceList = new List[length];

		for (int i = 0; i < length; i++) {
			performanceList[i] = new ArrayList<>();
		}

		for (Entry<Integer, FilteredValues> setsFilter : setsFilters.entrySet()) {

			for (int i = 0; i < length; i++) {
				FusionStrategy fusionRunType = fusionRunTypes[i];
				// ScoresGetter scoresGetter = scoresGetters[i];
				// Double probabilityThreshold = probabilityThresholds[i];

				// probabilities =
				// scoresGetter.getScores(setsFilter.getKey().toString(),
				// instanceScores, columnScores, tableScores, hostScores);

				Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType, groupedValues, fuseLists);

				Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

				FusionPerformance<PerformancePR> fusionPerformance = accuracyCalculator.analyzeAccuracy(groupedValues, fusedValues, fuseLists,
						setsFilter.getValue());
				performanceList[i].add(fusionPerformance);
			}

		}

		for (int i = 0; i < length; i++) {
			FusionStrategy fusionRunType = fusionRunTypes[i];
			// ScoresGetter scoresGetter = scoresGetters[i];
			// Double probabilityThreshold = probabilityThresholds[i];

			// probabilities = scoresGetter.getScores("", instanceScores,
			// columnScores, tableScores, hostScores);

			Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType, groupedValues, fuseLists);

			Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

			FusionPerformance<PerformancePR> performance = accuracyCalculator.analyzeAccuracy(groupedValues, fusedValues, fuseLists);

			FusionPerformance<PRFMPerformance> averagePerformance = accuracyCalculator.averageAccuracies(performanceList[i]);

			System.out.println();
			System.out.println();
			System.out.println();
			System.out.println();

			// System.out.println("Threshold" + probabilityThreshold);

			System.out.println(names[i]);

			System.out.println();
			writePerformance(performanceResultsWriter, dataset, names[i], performance, averagePerformance, true, true, false, false);

		}

	}

	public void getPerformance(String dataset, String prefix, PerformanceResultsWriter performanceResultsWriter, List<FusionDataType> methodTypes,
			GroupedValues groupedValues, InputProfileStore inputProfiledData, FusionStrategy fusionRunType, boolean fuseLists) throws IOException {

		System.out.println();
		System.out.println();
		// Match and group all the values

		List<FusionDataType> dataTypes = new ArrayList<>();
		dataTypes.addAll(methodTypes);

		List<FusionPreproccesser> preprocessors = new ArrayList<>();
		preprocessors.add(FusionPreproccesser.NONE);

		Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType, groupedValues, fuseLists);

		Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

		FusionPerformance<PerformancePR> performance = accuracyCalculator.analyzeAccuracy(groupedValues, fusedValues, fuseLists);

		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println();

		String name = prefix + fusionRunType.getName();
		System.out.println(name);

		System.out.println();
		writePerformance(performanceResultsWriter, dataset, name, performance, performance, true, true, false, false);

	}

	public void getPerformanceWithScores(String dataset, PerformanceResultsWriter performanceResultsWriter, List<FusionDataType> methodTypes,
			GroupedValues groupedValues, FusionStrategy fusionRunType, ScoresContainer scoresContainer, String name, boolean fuseLists) throws IOException {

		ThresholdContainer probabilityThreshold = scoreFusionThresholdFinder.findOptimumThresholdForPageRank();
		name += "_" + dfmt.format(probabilityThreshold) + "_THRESHOLD";
		System.out.println();
		System.out.println();
		// Match and group all the values

		List<FusionDataType> dataTypes = new ArrayList<>();
		dataTypes.addAll(methodTypes);

		List<FusionPreproccesser> preprocessors = new ArrayList<>();
		preprocessors.add(FusionPreproccesser.NONE);

		Map<FusionRun, Map<TripleId, FusedValue>> fusedValuesMap = fusionRunner.run(dataTypes, preprocessors, fusionRunType, groupedValues, scoresContainer,
				fuseLists, probabilityThreshold);

		Map<TripleId, FusedValue> fusedValues = CHMHelper.flattenMap(fusedValuesMap);

		FusionPerformance<PerformancePR> performance = accuracyCalculator.analyzeAccuracy(groupedValues, fusedValues, fuseLists);

		System.out.println();
		System.out.println();
		System.out.println();
		System.out.println();

		System.out.println(name);

		System.out.println();
		writePerformance(performanceResultsWriter, dataset, name, performance, performance, true, true, false, false);

	}

	private <T extends PERFMPerformanceI> void writePerformance(PerformanceResultsWriter performanceResultsWriter, String dataset, String name,
			FusionPerformance<PerformancePR> performance, FusionPerformance<T> averagePerformance, boolean printTotal, boolean printDataType,
			boolean printClassId, boolean printClassPropertyId) throws IOException {

		{
			int index = performanceResultsWriter.getCurrentIndex(ResultsCategory.TOTAL);
			String[] prefix = ResultsCategory.getNormalPrefix(name);
			performanceResultsWriter.write(ResultsCategory.TOTAL, prefix, performance.getTotal(), averagePerformance.getTotal());
			// if (printTotal) {
			// System.out.println(performanceResultsWriter.getFrom(ResultsCategory.TOTAL,
			// index));
			// }
		}

		{
			int index = performanceResultsWriter.getCurrentIndex(ResultsCategory.DATA_TYPE);

			Map<ColumnDataType, T> averageAccuracies = averagePerformance.getPerDataType();
			for (Entry<ColumnDataType, PerformancePR> e : performance.getPerDataType().entrySet()) {
				String[] prefix = ResultsCategory.getDataTypePrefix(name, e.getKey());
				performanceResultsWriter.write(ResultsCategory.DATA_TYPE, prefix, e.getValue(), averageAccuracies.get(e.getKey()));
			}

			// if (printDataType) {
			// System.out.println(performanceResultsWriter.getFrom(ResultsCategory.DATA_TYPE,
			// index));
			// }
		}

		{

			Map<ClassId, T> averageAccuracies = averagePerformance.getPerClassId();
			for (Entry<ClassId, PerformancePR> e : performance.getPerClassId().entrySet()) {
				String[] prefix = ResultsCategory.getClassIdPrefix(name, e.getKey());
				performanceResultsWriter.write(ResultsCategory.CLASS_ID, prefix, e.getValue(), averageAccuracies.get(e.getKey()));
			}

		}

		{

			int index = performanceResultsWriter.getCurrentIndex(ResultsCategory.CLASS_PROPERTY_ID);

			Map<ClassPropertyId, T> averageAccuracies = averagePerformance.getPerProperty();
			for (Entry<ClassPropertyId, PerformancePR> e : performance.getPerProperty().entrySet()) {
				ColumnDataType cdt = ontologyRepository.getDataType(e.getKey());
				String[] prefix = ResultsCategory.getClassPropertyIdPrefix(name, cdt, e.getKey());
				performanceResultsWriter.write(ResultsCategory.CLASS_PROPERTY_ID, prefix, e.getValue(), averageAccuracies.get(e.getKey()));
			}
			// System.out.println(performanceResultsWriter.getFrom(ResultsCategory.CLASS_PROPERTY_ID,
			// index));

		}

		{

			int index = performanceResultsWriter.getCurrentIndex(ResultsCategory.CLASS_PROPERTY_ID_WITH_DATASET);

			Map<ClassPropertyId, T> averageAccuracies = averagePerformance.getPerProperty();
			for (Entry<ClassPropertyId, PerformancePR> e : performance.getPerProperty().entrySet()) {
				ColumnDataType cdt = ontologyRepository.getDataType(e.getKey());
				String[] prefix = ResultsCategory.getClassPropertyIdWithDatasetPrefix(dataset, name, cdt, e.getKey());
				performanceResultsWriter.write(ResultsCategory.CLASS_PROPERTY_ID_WITH_DATASET, prefix, e.getValue(), averageAccuracies.get(e.getKey()));
			}
			System.out.println(performanceResultsWriter.getFrom(ResultsCategory.CLASS_PROPERTY_ID_WITH_DATASET, index));

		}

	}

	@FunctionalInterface
	public static interface ScoresGetter {
		ScoresContainer getScores(String prefix, InstanceScores instanceScores, ColumnScores columnScores, TableScores tableScores, HostScores hostScores);
	}

}
