package de.mannheim.uni.expansion.tablefusion.optimization;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import javax.inject.Inject;
import javax.inject.Named;

import com.opencsv.CSVParser;
import com.opencsv.CSVReader;
import com.opencsv.CSVWriter;

import de.mannheim.uni.expansion.data.dbpedia.repository.DBpediaRepositoryNewInterface;
import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.tablefusion.NamedProperties;
import de.mannheim.uni.expansion.tablefusion.core.fusing.stores.FusedValue;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.tablefusion.evaluation.EvaluationNormalizer;
import de.mannheim.uni.expansion.tablefusion.evaluation.getters.CorrectGetter;
import de.mannheim.uni.expansion.tablefusion.evaluation.getters.ReadableGetter;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.PairData;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.ValuePair;
import de.mannheim.uni.expansion.tablefusion.services.fusion.NewFuserOptionsBuilder;
import de.mannheim.uni.expansion.tablefusion.services.fusion.NewFuserService;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionDataType;
import de.mannheim.uni.expansion.tablefusion.services.fusion.runner.FusionStrategyRepository;
import de.mannheim.uni.expansion.tablefusion.services.grouping.GroupedValuesRepository;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.CSVSerializer;
import de.mannheim.uni.expansion.utils.CSVSerializer.CSVType;
import de.mannheim.uni.expansion.utils.MKDIR;
import de.mannheim.uni.expansion.utils.PersistentParallel;

public class ThresholdExporter {

	private final GroupedValuesRepository groupedValuesRepository;
	private final String optimizingFolder;

	private final NewFuserService basicFuserService;

	private final DBpediaRepositoryNewInterface dbPediaRepository;
	private final CSVSerializer csvSerializer;

	private final EvaluationNormalizer evaluationNormalizer;

	private final String thresholdsDirPath;
	private final String thresholdsOutputPath;
	private final String thresholdsInputPath;

	private final SimilarityCalculator similarityCalculator;

	@Inject
	public ThresholdExporter(GroupedValuesRepository groupedValuesRepository, @Named(NamedProperties.OPTIMIZING_FOLDER) String optimizingFolder,
			NewFuserService basicFuserService, DBpediaRepositoryNewInterface dbPediaRepository, CSVSerializer csvSerializer,
			SimilarityCalculator similarityCalculator, EvaluationNormalizer evaluationNormalizer) {
		this.groupedValuesRepository = groupedValuesRepository;
		this.optimizingFolder = optimizingFolder;
		this.basicFuserService = basicFuserService;
		this.dbPediaRepository = dbPediaRepository;
		this.csvSerializer = csvSerializer;
		this.similarityCalculator = similarityCalculator;
		this.evaluationNormalizer = evaluationNormalizer;

		thresholdsDirPath = this.optimizingFolder + "/thresholds";
		thresholdsOutputPath = thresholdsDirPath + "/output";
		thresholdsInputPath = thresholdsDirPath + "/input";
	}

	public void exportThresholdData() throws IOException {
		GroupedValues groupedValues = groupedValuesRepository.getGroupedValues();

		Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sourceSorted = groupedValuesRepository.getSourceSorted(3);

		Map<ClassPropertyId, Double> statistics = groupedValuesRepository.getStatistics(3);

		File thresholdsDir = new File(thresholdsDirPath);
		MKDIR.createDirUnchecked(thresholdsDir);

		File thresholdsOutputDir = new File(thresholdsOutputPath);
		MKDIR.createDirUnchecked(thresholdsOutputDir);

		File thresholdsInputDir = new File(thresholdsInputPath);
		MKDIR.createDirUnchecked(thresholdsInputDir);

		int resultSize = 115;
		double maxProportion = 0.035;

		List<PairData> pairData = getPairData(FusionDataType.STRING, groupedValues, statistics, sourceSorted, resultSize, maxProportion);

		write(thresholdsOutputPath + "/string.csv", pairData, (fusedValue, dbPediaValue) -> {
			return "";
		} , (o) -> {
			return o.toString();
		});

		List<PairData> datePairData = getPairData(FusionDataType.DATE, groupedValues, statistics, sourceSorted, resultSize, maxProportion);
		SimpleDateFormat dt = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

		write(thresholdsOutputPath + "/date.csv", datePairData, (fusedValue, dbPediaValue) -> {
			return "";
		} , (o) -> {
			return dt.format((Date) o);
		});

		List<PairData> numericPairData = getPairData(FusionDataType.NUMERIC, groupedValues, statistics, sourceSorted, resultSize, maxProportion);
		DecimalFormat dfmt = new DecimalFormat(",000.0");

		write(thresholdsOutputPath + "/numeric.csv", numericPairData, (fusedValue, dbPediaValue) -> {
			double sim = similarityCalculator.numericSimilarity.get(dbPediaValue, fusedValue);
			if (sim > 0.95) {
				return "1";
			}
			return "0";

		} , (o) -> {
			return dfmt.format((Double) o);
		});
	}

	private Creator<AtomicInteger[]> atomicIntegerCreator = () -> {
		AtomicInteger[] array = new AtomicInteger[4];
		for (int i = 0; i < 4; i++) {
			array[i] = new AtomicInteger();
		}
		return array;
	};

	DecimalFormat dfmtThreshold = new DecimalFormat("0.00");
	DecimalFormat dfmtAccuracy = new DecimalFormat("0.0000");

	public void findThreshold(Map<ValuePair, Boolean> valueMap, SimilarityMeasure sm) {

		double total = valueMap.size();
		double thresholdStep = 0.01;
		int numberOfSteps = 101;

		// 0 : TP
		// 1 : TN
		// 2 : FP
		// 3 : FN

		ConcurrentHashMap<Integer, AtomicInteger[]> map = new ConcurrentHashMap<>();

		PersistentParallel.runAndFinish(valueMap.entrySet(), (e) -> {

			double similarity = sm.get(e.getKey().getDBPediaValue(), e.getKey().getFusedValue());

			for (int i = 0; i < numberOfSteps; i++) {
				double threshold = i * thresholdStep;

				boolean simCorrect = false;
				if (similarity >= threshold)
					simCorrect = true;

				boolean labelCorrect = e.getValue();

				AtomicInteger[] countCorrect = CHMHelper.getFromCHM(map, i, atomicIntegerCreator);

				if (simCorrect && labelCorrect) {
					// True Positive
					countCorrect[0].incrementAndGet();
				} else if (!simCorrect && !labelCorrect) {
					// True Negative
					countCorrect[1].incrementAndGet();
				} else if (simCorrect & !labelCorrect) {
					// False Positive
					countCorrect[2].incrementAndGet();
				} else if (!simCorrect && labelCorrect) {
					// False Negative
					countCorrect[3].incrementAndGet();
				}

			}

		} , "Finding Threshold");

		Map<Integer, Double> precisionMap = new HashMap<>();
		Map<Integer, Double> recallMap = new HashMap<>();
		Map<Integer, Double> fMeasureMap = new HashMap<>();

		for (int i = 0; i < numberOfSteps; i++) {
			AtomicInteger[] countCorrect = map.get(i);
			int tp = countCorrect[0].get();
			int tn = countCorrect[1].get();
			int fp = countCorrect[2].get();
			int fn = countCorrect[3].get();

			double precision = ((double) tp) / (tp + fp);
			double recall = ((double) tp) / (tp + fn);
			double fMeasure = 2.0 * ((precision * recall) / (precision + recall));

			precisionMap.put(i, precision);

			recallMap.put(i, recall);

			fMeasureMap.put(i, fMeasure);
		}

		int maxThreshold = fMeasureMap.entrySet().stream().max((o1, o2) -> {
			return Double.compare(o1.getValue(), o2.getValue());
		}).get().getKey();

		for (int i = 0; i < numberOfSteps; i++) {
			double threshold = i * thresholdStep;
			double precision = precisionMap.get(i);
			double recall = recallMap.get(i);
			double fMeasure = fMeasureMap.get(i);

			System.out.print("Threshold: " + dfmtThreshold.format(threshold) + ", Precision: " + dfmtAccuracy.format(precision) + ", Recall: "
					+ dfmtAccuracy.format(recall) + ", F-Measure: " + dfmtAccuracy.format(fMeasure));
			if (i == maxThreshold) {

				System.out.println(", OPTIMUM");
			} else {
				System.out.println();
			}

		}

	}

	public Map<ValuePair, Boolean> read(String filePath, CSVType csvType) throws IOException {
		FileReader fr = new FileReader(filePath);
		CSVReader csvReader = new CSVReader(fr, 1, new CSVParser());
		try {
			Map<ValuePair, Boolean> ret = new HashMap<>();

			for (String[] line = csvReader.readNext(); line != null; line = csvReader.readNext()) {
				Object dbPediaValue = csvSerializer.getObject(csvType, line[0]);
				Object fusedValue = csvSerializer.getObject(csvType, line[1]);
				boolean correct;
				if ("1".equals(line[7])) {
					correct = true;
				} else if ("0".equals(line[7])) {
					correct = false;
				} else {
					throw new IllegalStateException("Invalid value for correct column: " + line[7]);
				}

				ret.put(new ValuePair(fusedValue, dbPediaValue), correct);
			}

			return ret;
		} finally {
			csvReader.close();
		}

	}

	public List<PairData> getPairData(FusionDataType methodType, GroupedValues groupedValues, Map<ClassPropertyId, Double> statistics,
			Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sourceSorted, int resultSize, double maxProportion) {
		NewFuserOptionsBuilder fuserOptionsBuilder = new NewFuserOptionsBuilder(FusionStrategyRepository.NORMAL, false, methodType.getDataType());

		Map<TripleId, FusedValue> fusedValues = basicFuserService.fuse(fuserOptionsBuilder, groupedValues, false);

		Map<ClassPropertyId, Map<InstanceId, PairData>> valuePairsMap = evaluationNormalizer.getValuePairsMap(fusedValues, sourceSorted);

		statistics = evaluationNormalizer.normalizeStatistics(statistics, valuePairsMap, resultSize, maxProportion);

		List<PairData> pairData = evaluationNormalizer.getPairData(valuePairsMap, statistics, sourceSorted, resultSize);

		return pairData;
	}

	public void write(String filePath, List<PairData> pairData, CorrectGetter correctGetter, ReadableGetter readableGetter) throws IOException {
		FileWriter fw = new FileWriter(filePath);
		CSVWriter csvWriter = new CSVWriter(fw);

		int columnsSize = 8;
		String[] headerLine = new String[columnsSize];
		headerLine[0] = "DBpediaSerialized";
		headerLine[1] = "FusedSerialized";
		headerLine[2] = "Class";
		headerLine[3] = "Property";
		headerLine[4] = "Instance";
		headerLine[5] = "DBpediaReadable";
		headerLine[6] = "FusedReadble";
		headerLine[7] = "Correct";

		csvWriter.writeNext(headerLine);

		for (PairData vd : pairData) {
			String[] line = new String[columnsSize];
			line[0] = csvSerializer.getString(vd.getValuePair().getDBPediaValue());
			line[1] = csvSerializer.getString(vd.getValuePair().getFusedValue());
			line[2] = vd.getTripleId().getClassId().getFinalPart();
			line[3] = vd.getTripleId().getPropertyId().getFinalPart();
			line[4] = vd.getTripleId().getInstanceId().getReadableString();
			line[5] = readableGetter.get(vd.getValuePair().getDBPediaValue());
			line[6] = readableGetter.get(vd.getValuePair().getFusedValue());
			line[7] = correctGetter.get(vd.getValuePair().getFusedValue(), vd.getValuePair().getDBPediaValue());

			csvWriter.writeNext(line);
		}

		csvWriter.close();
	}

}
