package de.mannheim.uni.expansion.tablefusion.optimization;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import javax.inject.Inject;
import javax.inject.Named;

import com.opencsv.CSVParser;
import com.opencsv.CSVReader;
import com.opencsv.CSVWriter;

import de.mannheim.uni.expansion.tablefusion.NamedProperties;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityCalculator;
import de.mannheim.uni.expansion.tablefusion.core.similarity.SimilarityMeasure;
import de.mannheim.uni.expansion.tablefusion.evaluation.getters.CorrectGetter;
import de.mannheim.uni.expansion.tablefusion.evaluation.getters.ReadableGetter;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.PairData;
import de.mannheim.uni.expansion.tablefusion.evaluation.stores.ValuePair;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.CSVSerializer;
import de.mannheim.uni.expansion.utils.CSVSerializer.CSVType;
import de.mannheim.uni.expansion.utils.PersistentParallel;

public class ThresholdFinder {

	private final String optimizingFolder;

	private final CSVSerializer csvSerializer;

	private final String thresholdsDirPath;
	private final String thresholdsOutputPath;
	private final String thresholdsInputPath;

	private final SimilarityCalculator similarityCalculator;

	@Inject
	public ThresholdFinder(@Named(NamedProperties.OPTIMIZING_FOLDER) String optimizingFolder, CSVSerializer csvSerializer,
			SimilarityCalculator similarityCalculator) {

		this.optimizingFolder = optimizingFolder;
		this.csvSerializer = csvSerializer;
		this.similarityCalculator = similarityCalculator;

		thresholdsDirPath = this.optimizingFolder + "/thresholds";
		thresholdsOutputPath = thresholdsDirPath + "/output";
		thresholdsInputPath = thresholdsDirPath + "/input";
	}

	public void findThreshold() throws IOException {

		System.out.println("String");
		Map<ValuePair, Boolean> stringMap = read(this.thresholdsInputPath + "/string.csv", CSVType.String);
		findThreshold(stringMap, similarityCalculator.matchingStringSimilarity);

		System.out.println("Date");
		Map<ValuePair, Boolean> dateMap = read(this.thresholdsInputPath + "/date.csv", CSVType.Date);
		findThreshold(dateMap, similarityCalculator.matchingDateSimilarity);

		System.out.println("Numeric");
		Map<ValuePair, Boolean> numericMap = read(this.thresholdsInputPath + "/numeric.csv", CSVType.Double);
		findThreshold(numericMap, similarityCalculator.matchingNumericSimilarity);

		System.out.println("String - Fusion");
		findThreshold(stringMap, similarityCalculator.stringSimilarity);

		System.out.println("String - Fusion - old");
		findThreshold(stringMap, similarityCalculator.stringSimilarityOld);

		System.out.println("Date - Fusion");
		findThreshold(dateMap, similarityCalculator.dateSimilarity);

		System.out.println("Numeric - Fusion");
		findThreshold(numericMap, similarityCalculator.numericSimilarity);

	}

	private Creator<AtomicInteger[]> atomicIntegerCreator = () -> {
		AtomicInteger[] array = new AtomicInteger[4];
		for (int i = 0; i < 4; i++) {
			array[i] = new AtomicInteger();
		}
		return array;
	};

	DecimalFormat dfmtThreshold = new DecimalFormat("0.00");
	DecimalFormat dfmtAccuracy = new DecimalFormat("0.0000");

	public void findThreshold(Map<ValuePair, Boolean> valueMap, SimilarityMeasure sm) {

		double total = valueMap.size();
		double thresholdStep = 0.01;
		int numberOfSteps = 101;

		// 0 : TP
		// 1 : TN
		// 2 : FP
		// 3 : FN

		ConcurrentHashMap<Integer, AtomicInteger[]> map = new ConcurrentHashMap<>();

		PersistentParallel.runAndFinish(valueMap.entrySet(), (e) -> {

			double similarity = sm.get(e.getKey().getDBPediaValue(), e.getKey().getFusedValue());

			for (int i = 0; i < numberOfSteps; i++) {
				double threshold = i * thresholdStep;

				boolean simCorrect = false;
				if (similarity >= threshold)
					simCorrect = true;

				boolean labelCorrect = e.getValue();

				AtomicInteger[] countCorrect = CHMHelper.getFromCHM(map, i, atomicIntegerCreator);

				if (simCorrect && labelCorrect) {
					// True Positive
					countCorrect[0].incrementAndGet();
				} else if (!simCorrect && !labelCorrect) {
					// True Negative
					countCorrect[1].incrementAndGet();
				} else if (simCorrect & !labelCorrect) {
					// False Positive
					countCorrect[2].incrementAndGet();
				} else if (!simCorrect && labelCorrect) {
					// False Negative
					countCorrect[3].incrementAndGet();
				}

			}

		} , "Finding Threshold");

		Map<Integer, Double> precisionMap = new HashMap<>();
		Map<Integer, Double> recallMap = new HashMap<>();
		Map<Integer, Double> fMeasureMap = new HashMap<>();

		for (int i = 0; i < numberOfSteps; i++) {
			AtomicInteger[] countCorrect = map.get(i);
			int tp = countCorrect[0].get();
			int tn = countCorrect[1].get();
			int fp = countCorrect[2].get();
			int fn = countCorrect[3].get();

			double precision = ((double) tp) / (tp + fp);
			double recall = ((double) tp) / (tp + fn);
			double fMeasure = 2.0 * ((precision * recall) / (precision + recall));

			precisionMap.put(i, precision);

			recallMap.put(i, recall);

			fMeasureMap.put(i, fMeasure);
		}

		int maxThreshold = fMeasureMap.entrySet().stream().max((o1, o2) -> {
			return Double.compare(o1.getValue(), o2.getValue());
		}).get().getKey();

		for (int i = 0; i < numberOfSteps; i++) {
			double threshold = i * thresholdStep;
			double precision = precisionMap.get(i);
			double recall = recallMap.get(i);
			double fMeasure = fMeasureMap.get(i);

			System.out.print("Threshold: " + dfmtThreshold.format(threshold) + ", Precision: " + dfmtAccuracy.format(precision) + ", Recall: "
					+ dfmtAccuracy.format(recall) + ", F-Measure: " + dfmtAccuracy.format(fMeasure));
			if (i == maxThreshold) {

				System.out.println(", OPTIMUM");
			} else {
				System.out.println();
			}

		}

	}

	public Map<ValuePair, Boolean> read(String filePath, CSVType csvType) throws IOException {
		FileReader fr = new FileReader(filePath);
		CSVReader csvReader = new CSVReader(fr, 1, new CSVParser());
		try {
			Map<ValuePair, Boolean> ret = new HashMap<>();

			for (String[] line = csvReader.readNext(); line != null; line = csvReader.readNext()) {
				Object dbPediaValue = csvSerializer.getObject(csvType, line[0]);
				Object fusedValue = csvSerializer.getObject(csvType, line[1]);
				boolean correct;
				if ("1".equals(line[7])) {
					correct = true;
				} else if ("0".equals(line[7])) {
					correct = false;
				} else {
					throw new IllegalStateException("Invalid value for correct column: " + line[7]);
				}

				ret.put(new ValuePair(fusedValue, dbPediaValue), correct);
			}

			return ret;
		} finally {
			csvReader.close();
		}

	}

	public void write(String filePath, List<PairData> pairData, CorrectGetter correctGetter, ReadableGetter readableGetter) throws IOException {
		FileWriter fw = new FileWriter(filePath);
		CSVWriter csvWriter = new CSVWriter(fw);

		int columnsSize = 8;
		String[] headerLine = new String[columnsSize];
		headerLine[0] = "DBpediaSerialized";
		headerLine[1] = "FusedSerialized";
		headerLine[2] = "Class";
		headerLine[3] = "Property";
		headerLine[4] = "Instance";
		headerLine[5] = "DBpediaReadable";
		headerLine[6] = "FusedReadble";
		headerLine[7] = "Correct";

		csvWriter.writeNext(headerLine);

		for (PairData vd : pairData) {
			String[] line = new String[columnsSize];
			line[0] = csvSerializer.getString(vd.getValuePair().getDBPediaValue());
			line[1] = csvSerializer.getString(vd.getValuePair().getFusedValue());
			line[2] = vd.getTripleId().getClassId().getFinalPart();
			line[3] = vd.getTripleId().getPropertyId().getFinalPart();
			line[4] = vd.getTripleId().getInstanceId().getReadableString();
			line[5] = readableGetter.get(vd.getValuePair().getDBPediaValue());
			line[6] = readableGetter.get(vd.getValuePair().getFusedValue());
			line[7] = correctGetter.get(vd.getValuePair().getFusedValue(), vd.getValuePair().getDBPediaValue());

			csvWriter.writeNext(line);
		}

		csvWriter.close();
	}

}
