package de.mannheim.uni.expansion.tablefusion.profiling.timestamps;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.inject.Inject;
import javax.inject.Named;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.classes.ClassId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.data.ontology.model.repository.OntologyRepository;
import de.mannheim.uni.expansion.data.tables.model.AnnotatedTableMetaData;
import de.mannheim.uni.expansion.data.tables.reader.AnnotatedTablesKryoReader;
import de.mannheim.uni.expansion.tablefusion.NamedProperties;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.FusionPerformance;
import de.mannheim.uni.expansion.tablefusion.analysis.stores.FusionPerformanceSynchronizedCreator;
import de.mannheim.uni.expansion.tablefusion.analysis.writer.GenericAbstractResultsWriter;
import de.mannheim.uni.expansion.tablefusion.analysis.writer.ResultsCategory;
import de.mannheim.uni.tableprocessor.model.Table;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class TimeStampProfiler {

	private final AnnotatedTablesKryoReader kryoReader;
	private final OntologyRepository ontologyRepository;
	private final String resultsFolder;

	@Inject
	public TimeStampProfiler(AnnotatedTablesKryoReader kryoReader, OntologyRepository ontologyRepository,
			@Named(NamedProperties.RESULTS_FOLDER) String resultsFolder) {
		this.kryoReader = kryoReader;
		this.ontologyRepository = ontologyRepository;
		this.resultsFolder = resultsFolder;
	}

	public void profileTimeStamps() {

		FusionPerformanceSynchronizedCreator<TimeStampsSingleResults> creatorSingle = new FusionPerformanceSynchronizedCreator<>(new TimeStampsSingleResults());

		// FusionPerformanceSynchronizedCreator<TimeStampsAggregatedResults>
		// creatorAggregated = new FusionPerformanceSynchronizedCreator<>(
		// new TimeStampsAggregatedResults());

		kryoReader.runForEachTable((table) -> {

			boolean hasBeforeText = false;
			boolean hasAfterText = false;
			boolean hasInContextBefore = false;
			boolean hasInTableTitle = false;
			boolean hasInPageTitle = false;

			if (!table.getMetaData().getYearsBeforeText().isEmpty())
				hasBeforeText = true;
			if (!table.getMetaData().getYearsAfterText().isEmpty())
				hasAfterText = true;
			if (!table.getMetaData().getYearsInContextBefore().isEmpty())
				hasInContextBefore = true;
			if (!table.getMetaData().getYearsInTableTitle().isEmpty())
				hasInTableTitle = true;
			if (!table.getMetaData().getYearsInPageTitle().isEmpty())
				hasInPageTitle = true;

			Map<Integer, List<Integer>> columnYearsMap = table.getMetaData().getPropertiesYears();
			Map<Integer, PropertyId> columns = table.getMetaData().getProperties();

			int numOfColumns = 0;
			int numOfColumnsWithOwnTimeStamp = 0;

			Set<ClassPropertyId> propertyIdSet = new HashSet<>();
			Set<ColumnDataType> cdtSet = new HashSet<>();
			for (Entry<Integer, PropertyId> e : columns.entrySet()) {
				if (e.getValue() != null) {
					numOfColumns++;

					boolean hasColumn = false;
					if (columnYearsMap != null) {
						List<Integer> columnYears = columnYearsMap.get(e.getKey());
						if (columnYears != null && columnYears.size() > 0) {
							hasColumn = true;

						}
					}

					if (hasColumn) {
						numOfColumnsWithOwnTimeStamp++;
					}

					int numOfValues = 0;
					int hasRowCellYear = 0;
					for (Entry<Integer, Object> v : table.getTableData().getColumns().get(e.getKey()).getValues().entrySet()) {
						if (v.getValue() != null) {
							numOfValues++;
							if (hasCellYear(table.getTableData(), table.getMetaData(), v.getKey()))
								hasRowCellYear++;
						}
					}

					TimeStampsSingleResults res = new TimeStampsSingleResults();

					boolean atLeastOne = false;
					if (hasBeforeText) {
						res.beforeText(numOfValues);
						atLeastOne = true;
					}
					if (hasAfterText) {
						res.afterText(numOfValues);

						atLeastOne = true;
					}
					if (hasInContextBefore) {
						res.inContextVefore(numOfValues);
						atLeastOne = true;
					}

					if (hasInTableTitle) {
						res.inTableTitle(numOfValues);
						atLeastOne = true;
					}
					if (hasInPageTitle) {
						res.inPageTitle(numOfValues);
						atLeastOne = true;
					}
					if (hasColumn) {
						res.inColumnHeader(numOfValues);
						atLeastOne = true;
					}

					if (atLeastOne) {
						res.addTotalWithTS(numOfValues);
					} else {
						res.addTotalWithoutTS(numOfValues);
					}

					res.inRowCellValue(hasRowCellYear);

					creatorSingle.addTotal(res);

					ColumnDataType cdt = ontologyRepository.getDataType(table.getMetaData().getDbPediaClass(), e.getValue());
					creatorSingle.addPerClassId(table.getMetaData().getDbPediaClass(), res);
					if (cdt != null) {
						creatorSingle.addPerDataType(cdt, res);
						// creatorAggregated.addPerDataType(cdt, res2);
						cdtSet.add(cdt);
					} else
						System.out.println("CDT NULL FOR CLASSiD " + table.getMetaData().getDbPediaClass() + " and PropertyID " + e.getValue() + ".");

					ClassPropertyId cpid = new ClassPropertyId(table.getMetaData().getDbPediaClass(), e.getValue());
					creatorSingle.addPerProperty(cpid, res);
					// creatorAggregated.addPerProperty(cpid, res2);
					propertyIdSet.add(cpid);

				} else {
					System.out.println("UNMAPPED COLUMN");
				}
			}

			// TimeStampsAggregatedResults res = null;
			//
			// TimeStampsAggregatedResults res2 = null;
			// if (hasTable) {
			// res = new TimeStampsAggregatedResults(1, 1, numOfColumns,
			// numOfColumns, numOfColumnsWithOwnTimeStamp,
			// numOfColumnsWithOwnTimeStamp);
			// res2 = new TimeStampsAggregatedResults(1, 1, 0, 0, 0, 0);
			// } else {
			//
			// res = new TimeStampsAggregatedResults(1, 0, numOfColumns, 0,
			// numOfColumnsWithOwnTimeStamp, 0);
			// res2 = new TimeStampsAggregatedResults(1, 0, 0, 0, 0, 0);
			// }
			//
			// creatorAggregated.addTotal(res);
			// creatorAggregated.addPerClassId(table.getMetaData().getDbPediaClass(),
			// res);
			//
			// for (ColumnDataType cdt : cdtSet) {
			// creatorAggregated.addPerDataType(cdt, res2);
			// }
			//
			// for (ClassPropertyId classPropertyId : propertyIdSet) {
			// creatorAggregated.addPerProperty(classPropertyId, res2);
			// }

			// e.getMetaData()
		});

		TimeStampsAggregatedResultsWriter aggreagtedWriter = new TimeStampsAggregatedResultsWriter();
		TimeStampsSingleResultsWriter singleWriter = new TimeStampsSingleResultsWriter();

		try {
			writePerformance(singleWriter, creatorSingle.get(), "single");

			// writePerformance(aggreagtedWriter, creatorAggregated.get(),
			// "aggregated");

			for (ResultsCategory rc : ResultsCategory.values()) {
				{
					FileWriter fw = new FileWriter(resultsFolder + "/TIMESTAMPS_SINGLE_" + rc.name() + ".csv", false);

					fw.write(singleWriter.getAll(rc));
					fw.close();
				}
				// {
				// FileWriter fw = new FileWriter(resultsFolder +
				// "/TIMESTAMPS_AGGREGATED_" + rc.name() + ".csv", false);
				// fw.write(aggreagtedWriter.getAll(rc));
				// fw.close();
				// }
			}

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private <E> void writePerformance(GenericAbstractResultsWriter<E> performanceResultsWriter, FusionPerformance<E> performance, String name)
			throws IOException {

		{
			int index = performanceResultsWriter.getCurrentIndex(ResultsCategory.TOTAL);
			String[] prefix = ResultsCategory.getNormalPrefix(name);
			performanceResultsWriter.write(ResultsCategory.TOTAL, prefix, performance.getTotal());
			System.out.println(performanceResultsWriter.getFrom(ResultsCategory.TOTAL, index));

		}

		{
			int index = performanceResultsWriter.getCurrentIndex(ResultsCategory.DATA_TYPE);
			for (Entry<ColumnDataType, E> e : performance.getPerDataType().entrySet()) {
				String[] prefix = ResultsCategory.getDataTypePrefix(name, e.getKey());
				performanceResultsWriter.write(ResultsCategory.DATA_TYPE, prefix, e.getValue());
			}

			System.out.println(performanceResultsWriter.getFrom(ResultsCategory.DATA_TYPE, index));

		}

		{

			for (Entry<ClassId, E> e : performance.getPerClassId().entrySet()) {
				String[] prefix = ResultsCategory.getClassIdPrefix(name, e.getKey());
				performanceResultsWriter.write(ResultsCategory.CLASS_ID, prefix, e.getValue());
			}

		}

		{

			for (Entry<ClassPropertyId, E> e : performance.getPerProperty().entrySet()) {
				ColumnDataType cdt = ontologyRepository.getDataType(e.getKey());
				String[] prefix = ResultsCategory.getClassPropertyIdPrefix(name, cdt, e.getKey());
				performanceResultsWriter.write(ResultsCategory.CLASS_PROPERTY_ID, prefix, e.getValue());
			}

		}

	}

	boolean hasCellYear(Table td, AnnotatedTableMetaData md, int rowIdx) {

		PropertyId activeYearsStartYear = new PropertyId("http://dbpedia.org/ontology/activeYearsStartYear");

		for (int i = 0; i < td.getColumns().size(); i++) {
			if (td.getColumns().get(i).getDataType() == ColumnDataType.date) {
				PropertyId propertyId = md.getProperty(i);
				if (propertyId == null || propertyId.equals(activeYearsStartYear)) {
					Object oYear = td.getColumns().get(i).getValues().get(rowIdx);
					if (oYear instanceof Date) {
						return true;
					}
				}
			}

		}

		return false;

	}
}
