package de.mannheim.uni.expansion.tablefusion.services.fusion.strategies;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.function.BiFunction;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.properties.PropertyId;
import de.mannheim.uni.expansion.tablefusion.analysis.ValueEvaluator;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.RegressionScores;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.RegressionScoresClassPropertyId;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.RegressionScoresPropertyId;
import de.mannheim.uni.expansion.tablefusion.core.scoring.packs.ScoresContainer;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.expansion.utils.containers.Pair;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;
import weka.classifiers.functions.LinearRegression;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instances;
import weka.core.SelectedTag;
import weka.core.SparseInstance;

public class RegressionScorer extends WekaScorer {

	private final ValueEvaluator valueEvaluator;

	@Inject
	public RegressionScorer(ValueEvaluator valueEvaluator) {
		super(valueEvaluator);
		this.valueEvaluator = valueEvaluator;
	}

	public <T> RegressionData<T> getaa(GroupedValues groupedValues, Map<String, ScoresContainer> scoresContainers,
			BiFunction<TripleId, ColumnDataType, T> getKey) {
		String[] names = new String[scoresContainers.size()];
		ScoresContainer[] containers = new ScoresContainer[scoresContainers.size()];

		{
			int i = 0;
			for (Entry<String, ScoresContainer> sc : scoresContainers.entrySet()) {
				names[i] = sc.getKey();
				containers[i] = sc.getValue();

				i++;
			}
		}

		ConcurrentHashMap<T, ConcurrentLinkedQueue<Pair<Boolean, Double[]>>> listMap = getListMap(groupedValues, containers, getKey);

		ConcurrentHashMap<T, double[]> weightsMap = new ConcurrentHashMap<>();
		ConcurrentHashMap<T, Double> interceptsMap = new ConcurrentHashMap<>();

		PersistentParallel3.runAndFinish(listMap.entrySet(), (entry) -> {

			Pair<double[], Double> weightsAndIntercept = getWeightsAndInterceptSplit(entry.getValue(), names);

			weightsMap.put(entry.getKey(), weightsAndIntercept.getFirst());

			interceptsMap.put(entry.getKey(), weightsAndIntercept.getSecond());
		} , "REGRESSION");

		return new RegressionData<T>(containers, names, weightsMap, interceptsMap);

		// finalConsumer.accept(containers, names, weightsMap, interceptsMap);
	}

	private Pair<double[], Double> getWeightsAndInterceptSplit(ConcurrentLinkedQueue<Pair<Boolean, Double[]>> list, String[] containerSwitches) {

		List<ConcurrentLinkedQueue<Pair<Boolean, Double[]>>> listOfQueues = new ArrayList<>();

		int length = 4;

		for (int i = 0; i < length; i++) {
			listOfQueues.add(new ConcurrentLinkedQueue<>());
		}

		int sizeOfSet = (int) Math.ceil((1.0 / length) * list.size());

		{
			int i = 0;
			Pair<Boolean, Double[]> pair = null;
			while ((pair = list.poll()) != null) {
				for (int j = 0; j < length; j++) {
					int highLim = (length - j) * sizeOfSet;
					int lowLim = (length - j - 1) * sizeOfSet;
					if (i < lowLim || i > highLim) {
						listOfQueues.get(j).add(pair);
					}
				}

				i++;
			}
		}

		double[] weights = new double[containerSwitches.length];
		Double intercept = 0.0;

		for (ConcurrentLinkedQueue<Pair<Boolean, Double[]>> queue : listOfQueues) {
			Pair<double[], Double> pair = getWeightsAndIntercept(queue, containerSwitches);

			for (int i = 0; i < containerSwitches.length; i++) {
				weights[i] += pair.getFirst()[i];
			}

			intercept += pair.getSecond();
		}

		for (int i = 0; i < containerSwitches.length; i++) {
			weights[i] = weights[i] / length;
		}
		intercept = intercept / length;

		return new Pair<double[], Double>(weights, intercept);

	}

	private Pair<double[], Double> getWeightsAndIntercept(ConcurrentLinkedQueue<Pair<Boolean, Double[]>> list, String[] containerSwitches) {

		int shortedContainersLength = 0;

		for (String cs : containerSwitches) {
			if (cs != null)
				shortedContainersLength++;
		}

		FastVector fv = new FastVector(shortedContainersLength);
		fv.addElement(new Attribute("Score"));
		{
			for (String cs : containerSwitches) {
				if (cs != null) {
					fv.addElement(new Attribute(cs));
				}
			}

		}

		int length = list.size();

		LinearRegression regression = new LinearRegression();
		regression.setEliminateColinearAttributes(false);
		regression.setAttributeSelectionMethod(new SelectedTag(LinearRegression.SELECTION_M5, LinearRegression.TAGS_SELECTION));

		// regression.setEliminateColinearAttributes(false);
		// regression.setAttributeSelectionMethod(new
		// SelectedTag(LinearRegression.SELECTION_GREEDY,
		// LinearRegression.TAGS_SELECTION));

		Instances data = new Instances("Regression", fv, length);

		// double[] y = new double[length];
		// double[][] x = new double[length][];

		for (Pair<Boolean, Double[]> l : list) {
			Double[] z = new Double[shortedContainersLength + 1];

			if (l.getFirst()) {
				z[0] = 1.0;
			} else {
				z[0] = 0.0;
			}

			Double[] scores = l.getSecond();
			int vp = 1;
			for (int i = 0; i < scores.length; i++) {
				if (containerSwitches[i] != null) {
					z[vp] = scores[i];
					vp++;
				}
			}

			int countNonNull = 0;
			for (Double s : z) {
				if (s == null) {
					countNonNull++;
				}
			}

			double[] instanceScores = new double[z.length - countNonNull];

			int[] index = new int[z.length - countNonNull];

			int j = 0;
			for (int i = 0; i < z.length; i++) {
				if (z[i] != null) {
					instanceScores[j] = z[i];
					index[j] = i;
					j++;
				}

			}

			data.add(new SparseInstance(1.0, instanceScores, index, z.length));
		}

		data.setClassIndex(0);

		// RealMatrix matrix = new Array2DRowRealMatrix(x);
		//
		// RealMatrix vt = new SingularValueDecomposition(matrix).getVT();
		//
		// x = matrix.multiply(vt).getData();

		// matrix = new Array2DRowRealMatrix(x);
		//
		// double[][] cov = new
		// Covariance(matrix.transpose().getData())..getCovarianceMatrix().getData();

		try {
			regression.buildClassifier(data);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		double[] weightsWithIntercept = regression.coefficients();

		double[] weights = new double[containerSwitches.length];

		int j2 = 0;
		for (int j = 1; j < weightsWithIntercept.length - 1; j++) {

			if (containerSwitches[j2] != null) {
				weights[j2] = weightsWithIntercept[j];

			} else {
				weights[j2] = 0.0;
			}

			j2++;
		}

		double intercept = weightsWithIntercept[weightsWithIntercept.length - 1];

		boolean runAgain = false;
		String[] newSwitches = new String[containerSwitches.length];
		for (int i = 0; i < containerSwitches.length; i++) {
			if (containerSwitches[i] != null && weights[i] < 0.0) {
				runAgain = true;
			} else {
				newSwitches[i] = containerSwitches[i];
			}
		}

		if (runAgain && false) {
			return getWeightsAndIntercept(list, newSwitches);
		} else {
			return new Pair<double[], Double>(weights, intercept);
		}

	}

	public RegressionScores get(GroupedValues groupedValues, Map<String, ScoresContainer> scoresContainers) {

		RegressionData<Boolean> data = getaa(groupedValues, scoresContainers, (tid, cdt) -> {
			return Boolean.TRUE;
		});

		return new RegressionScores(data.getNames(), data.getContainers(), data.getWeightsMap().get(Boolean.TRUE), data.getInterceptsMap().get(Boolean.TRUE));
	}

	public RegressionScoresClassPropertyId getClassPropertyId(GroupedValues groupedValues, Map<String, ScoresContainer> scoresContainers) {

		RegressionData<ClassPropertyId> data = getaa(groupedValues, scoresContainers, (tid, cdt) -> {
			return new ClassPropertyId(tid);
		});

		return new RegressionScoresClassPropertyId(data.getNames(), data.getContainers(), data.getWeightsMap(), data.getInterceptsMap());

	}

	public RegressionScoresPropertyId getPropertyId(GroupedValues groupedValues, Map<String, ScoresContainer> scoresContainers) {

		RegressionData<PropertyId> data = getaa(groupedValues, scoresContainers, (tid, cdt) -> {
			return tid.getPropertyId();
		});

		return new RegressionScoresPropertyId(data.getNames(), data.getContainers(), data.getWeightsMap(), data.getInterceptsMap());

	}

	private static class RegressionData<T> {
		private final ScoresContainer[] containers;
		private final String[] names;
		private final Map<T, double[]> weightsMap;
		private final Map<T, Double> interceptsMap;

		public RegressionData(ScoresContainer[] containers, String[] names, Map<T, double[]> weightsMap, Map<T, Double> interceptsMap) {
			super();
			this.containers = containers;
			this.names = names;
			this.weightsMap = weightsMap;
			this.interceptsMap = interceptsMap;
		}

		public ScoresContainer[] getContainers() {
			return containers;
		}

		public String[] getNames() {
			return names;
		}

		public Map<T, double[]> getWeightsMap() {
			return weightsMap;
		}

		public Map<T, Double> getInterceptsMap() {
			return interceptsMap;
		}

	}
}
