package de.mannheim.uni.expansion.tablefusion.services.grouping;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import javax.inject.Inject;

import de.mannheim.uni.expansion.data.ontology.model.ClassPropertyId;
import de.mannheim.uni.expansion.data.ontology.model.TripleId;
import de.mannheim.uni.expansion.data.ontology.model.instances.InstanceId;
import de.mannheim.uni.expansion.tablefusion.core.matching.GroupedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.MatchedValues;
import de.mannheim.uni.expansion.tablefusion.core.matching.impl.BasicGroupMatcher;
import de.mannheim.uni.expansion.tablefusion.core.matching.impl.ColumnMatchesGrouper;
import de.mannheim.uni.expansion.utils.CHMHelper;
import de.mannheim.uni.expansion.utils.CHMHelper.Creator;
import de.mannheim.uni.expansion.utils.MapSorter;
import de.mannheim.uni.expansion.utils.PersistentParallel;
import de.mannheim.uni.expansion.utils.PersistentParallel3;
import de.mannheim.uni.tableprocessor.model.TableColumn.ColumnDataType;

public class GroupedValuesRepository {

	private final BasicMatcherService matcherService;

	private final ColumnMatchesGrouper columnMatchesGrouper;
	private final BasicGroupMatcher groupMatcher;

	@Inject
	public GroupedValuesRepository(BasicMatcherService matcherService, ColumnMatchesGrouper columnMatchesGrouper, BasicGroupMatcher groupMatcher) {
		this.matcherService = matcherService;
		this.columnMatchesGrouper = columnMatchesGrouper;
		this.groupMatcher = groupMatcher;
	}

	GroupedValues groupedValues;

	public GroupedValues getGroupedValues() {

		if (groupedValues == null) {
			computeGroupedValues();
		}
		return groupedValues;

	}

	public Map<ClassPropertyId, Integer> getCount(int minCount, boolean groupTriples) {
		ConcurrentHashMap<ClassPropertyId, AtomicInteger> countsPerClass = new ConcurrentHashMap<>();

		Creator<AtomicInteger> aiCreator = () -> {
			return new AtomicInteger();
		};

		PersistentParallel3.runAndFinish(getGroupedValues().entrySet(), (e) -> {
			int count = e.getValue().getValues().size();
			if (count >= minCount) {
				ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey().getClassId(), e.getKey().getPropertyId());

				AtomicInteger countPerClass = CHMHelper.getFromCHM(countsPerClass, classPropertyId, aiCreator);
				if (groupTriples) {
					countPerClass.incrementAndGet();
				} else {
					countPerClass.addAndGet(count);
				}
			}

		} , "Calculate Class Property Statistics");

		Map<ClassPropertyId, Integer> countFinal = new HashMap<>();

		for (Entry<ClassPropertyId, AtomicInteger> e : countsPerClass.entrySet()) {
			countFinal.put(e.getKey(), e.getValue().get());
		}
		return countFinal;
	}

	public Map<ClassPropertyId, Integer> getSourceCount(int minCount) {
		return getCount(minCount, false);
	}

	public Map<ClassPropertyId, Double> getStatistics(int minCount) {

		Map<ClassPropertyId, Integer> countsPerClass = getCount(minCount, true);

		int countTotal = 0;

		for (Entry<ClassPropertyId, Integer> e : countsPerClass.entrySet()) {
			countTotal += e.getValue();
		}

		Map<ClassPropertyId, Double> statistics = new HashMap<>();
		for (Entry<ClassPropertyId, Integer> e : countsPerClass.entrySet()) {

			double countPerClass = e.getValue();

			statistics.put(e.getKey(), countPerClass / countTotal);
		}

		return statistics;

	}

	public Map<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> getSourceSorted(int minCount) {
		ConcurrentHashMap<ClassPropertyId, ConcurrentHashMap<InstanceId, Integer>> countsPerClass = new ConcurrentHashMap<>();

		Creator<ConcurrentHashMap<InstanceId, Integer>> mapCreator = () -> {
			return new ConcurrentHashMap<InstanceId, Integer>();
		};

		PersistentParallel.runAndFinish(getGroupedValues().entrySet(), (e) -> {
			int count = e.getValue().getValues().size();
			if (count >= minCount) {
				ClassPropertyId classPropertyId = new ClassPropertyId(e.getKey().getClassId(), e.getKey().getPropertyId());

				ConcurrentHashMap<InstanceId, Integer> countPerClass = CHMHelper.getFromCHM(countsPerClass, classPropertyId, mapCreator);
				countPerClass.put(e.getKey().getInstanceId(), count);
			}
		} , "Sort Grouped Values by Count");

		ConcurrentHashMap<ClassPropertyId, LinkedHashMap<InstanceId, Integer>> sortedMap = new ConcurrentHashMap<>();

		PersistentParallel.runAndFinish(countsPerClass.entrySet(), (e) -> {

			sortedMap.put(e.getKey(), MapSorter.sortHashMapByValuesD(e.getValue()));

		} , "Sort Grouped Values by Count");

		return sortedMap;
	}

	private synchronized void computeGroupedValues() {
		if (groupedValues == null) {
			MatcherOptionsBuilder matcherOptions;
			Map<TripleId, MatchedValues> groupedValues;

			matcherOptions = new MatcherOptionsBuilder()
					.withNewMatcher(groupMatcher, ColumnDataType.numeric, ColumnDataType.string, ColumnDataType.date, ColumnDataType.link)
					.withNewReferenceMatcher(columnMatchesGrouper);
			groupedValues = matcherService.matchAndGroup(matcherOptions);

			this.groupedValues = new GroupedValues(groupedValues);
		}
	}

	public Map<Integer, Integer> getDistribution() {
		GroupedValues groupedValues = getGroupedValues();

		ConcurrentHashMap<Integer, AtomicInteger> map = new ConcurrentHashMap<Integer, AtomicInteger>();

		PersistentParallel.runAndFinish(groupedValues.entrySet(), (e) -> {
			Integer count = e.getValue().getValues().size();
			AtomicInteger ai = CHMHelper.getFromCHM(map, count, () -> new AtomicInteger());
			ai.incrementAndGet();
		} , "Grouped Values Distribution");

		Map<Integer, Integer> ret = map.entrySet().stream().collect(Collectors.toMap(Entry::getKey, (e) -> {
			return e.getValue().get();
		}));

		return ret;
	}

}
