package de.mannheim.uni.matching.dbpedia.matchers.instance;

import de.mannheim.uni.matching.Matcher;
import de.mannheim.uni.matching.dbpedia.algorithm.PartialMatcher;
import de.mannheim.uni.matching.dbpedia.logging.MatchingLogger;
import de.mannheim.uni.matching.dbpedia.model.GoldStandard;
import de.mannheim.uni.matching.dbpedia.model.MatchingData;
import de.mannheim.uni.matching.dbpedia.model.MatchingParameters;
import de.mannheim.uni.matching.dbpedia.model.Similarities;
import de.mannheim.uni.matching.dbpedia.model.TableCell;
import de.mannheim.uni.matching.dbpedia.model.TableRow;
import de.mannheim.uni.matching.dbpedia.model.adapters.TableColumnToCellHierarchyAdapter;
import de.mannheim.uni.matching.dbpedia.model.adapters.TableRowToCellHierarchyAdapter;
import de.mannheim.uni.matching.dbpedia.model.adapters.TableRowUriMatchingAdapter;
import de.mannheim.uni.matching.secondline.Aggregate;
import de.mannheim.uni.matching.secondline.AggregationType;
import de.mannheim.uni.similarity.matrix.SimilarityMatrix;
import de.mannheim.uni.tableprocessor.model.TableColumn;
import de.mannheim.uni.utils.timer.Timer;

/**
 * Matcher for instances that uses existing similarity matrices and applies pruning in the end
 * @author Oliver
 *
 */
public class PruningInstanceMatcher extends PartialMatcher<TableRow> {

    public PruningInstanceMatcher(Similarities similarities,
            MatchingParameters matchingParameters, Timer rootTimer,
            GoldStandard goldStandard, MatchingLogger logger) {
        super(similarities, matchingParameters, rootTimer, goldStandard, logger);
    }
    
    private SimilarityMatrix<TableColumn> propertySimilarity;
    public SimilarityMatrix<TableColumn> getPropertySimilarity() {
        return propertySimilarity;
    }
    public void setPropertySimilarity(
            SimilarityMatrix<TableColumn> propertySimilarity) {
        this.propertySimilarity = propertySimilarity;
    }
    
    private SimilarityMatrix<TableCell> valueSimilarity;
    public SimilarityMatrix<TableCell> getValueSimilarity() {
        return valueSimilarity;
    }
    public void setValueSimilarity(SimilarityMatrix<TableCell> valueSimilarity) {
        this.valueSimilarity = valueSimilarity;
    }
    
    private SimilarityMatrix<TableRow> initialCandidateSimilarity;
    public SimilarityMatrix<TableRow> getInitialCandidateSimilarity() {
        return initialCandidateSimilarity;
    }
    public void setInitialCandidateSimilarity(
            SimilarityMatrix<TableRow> initialCandidateSimilarity) {
        this.initialCandidateSimilarity = initialCandidateSimilarity;
    }
    
    private double similarityThreshold;
    public double getSimilarityThreshold() {
        return similarityThreshold;
    }
    public void setSimilarityThreshold(double similarityThreshold) {
        this.similarityThreshold = similarityThreshold;
    }
    
    public SimilarityMatrix<TableRow> match(MatchingData data) {
        Timer t = Timer.getNamed("Candidate matching", getRootTimer());
        
        /*
         * combine value scores with property scores so that only the scores of the currently matched properties remain
         * => set all scores of unmapped combinations to 0
         */
        if(getMatchingParameters().isCollectMatchingInfo()) {
            System.out.print("candidate matching 1/2 ... ");
        }
        Timer tm = Timer.getNamed("CombineHierarchy: weight value-based similarities with property similarities", t);
        SimilarityMatrix<TableCell> inst = Matcher.multiplyParentSimilarity(getPropertySimilarity(), getValueSimilarity(), new TableColumnToCellHierarchyAdapter());
        tm.stop();
        if(getMatchingParameters().isCollectMatchingInfo()) {
            System.out.println("done");
        }
        
        //TODO this pruning here seems to make the results quite good, but also cancels out the value scores in most cases (i.e. only key matching is left ...)
        //inst.prune(0.1);
        
        
        /*
         * aggregate the combined value scores to candidate scores (row based)
         */
        if(getMatchingParameters().isCollectMatchingInfo()) {
            System.out.print("candidate matching 2/2 ... ");
        }
        Aggregate<TableRow, TableCell> a = new Aggregate<TableRow, TableCell>();
        a.setAggregationType(AggregationType.Sum);
        a.setCollectMatchingInfo(getMatchingParameters().isCollectMatchingInfo());
        tm = Timer.getNamed("Aggregate to candidates", t);
        SimilarityMatrix<TableRow> propWeightedInstanceCand = a.match(getInitialCandidateSimilarity(), inst, new TableRowToCellHierarchyAdapter());
        tm.stop();
        if(getMatchingParameters().isCollectMatchingInfo()) {
            System.out.println("done");
            System.out.println("Aggregation (value->instance) steps:");
            System.out.println(a.getLog().toString());
        }

        SimilarityMatrix<TableRow> candSim = propWeightedInstanceCand;
        
//        candSim.prune(getMatchingParameters().getInstanceScoreThreshold());
        candSim.prune(getSimilarityThreshold());

        //TODO we could try a 1:1 mapping here ...

        //getSimilarities().setCandidateSimilarity(candSim);
        
        if (getMatchingParameters().isCollectMatchingInfo()) {
            System.out.println(candSim.getOutput(null, getGoldStandard().getInstanceGoldStandard().values(), new TableRowUriMatchingAdapter(), null));
        }
        t.stop();
        
        return candSim;
    }
}
